/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.rqm.defects.web.client.DefectsManager");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");


dojo.require("net.jazz.ajax.core.internal.loader");
dojo.require("net.jazz.ajax.core.runtime.RegistryFactory");

dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.rqm.defects.web.client.internal.DefectsProviderClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");

(function() {

var INIT_UNINITIALIZED= 0;
var INIT_IN_PROGRESS= 1;
var INIT_DONE= 2;
var INIT_FAILURE= 3;

var loader					= net.jazz.ajax.core.internal.loader;
var RegistryFactory 		= net.jazz.ajax.core.runtime.RegistryFactory;

var DefectsProviderClient	= com.ibm.rqm.defects.web.client.internal.DefectsProviderClient;
var getCurrentProjectArea	= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var Status					= com.ibm.team.repository.web.foundation.Status;

var DefectsUIUtil           = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;

dojo.declare("com.ibm.rqm.defects.web.client.DefectsManager", null, {

	_defectsProviders: null,
	_currentProviderId: null,

	constructor: function() {
		this._initializationState= INIT_UNINITIALIZED;
		this._defectsProviders= null;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	ensureInitialized: function(monitor, initializationCallback) {
		this._runInitialized(dojo.hitch(this, function() {
			initializationCallback();
		}), monitor);
	},


	getCurrentDefectsProviderId: function() {
		return this._currentProviderId;
	},

	getDefectsProviders: function() {
		return this._defectsProviders;
	},

	getDefectsProvider: function(providerId) {
		return this._defectsProviders && this._defectsProviders[providerId]? this._defectsProviders[providerId].instance : null;
	},
	
	createDefect: function(linkType, itemIdOrUriArray, defectDescription, monitor, runAfterCreation) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.createDefect(linkType, itemIdOrUriArray, defectDescription, monitor, runAfterCreation);
		}), runAfterCreation, monitor);
	},
	
	deleteDefect: function(draftUri, ewiItemId, monitor, runAfterCreation) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.deleteDefect(draftUri, ewiItemId, monitor, runAfterCreation);
		}), runAfterCreation, monitor);
	},
	
	changeBlockingDefect: function(draftUri, resultItemId, blocking, monitor, runAfterCreation) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.changeBlockingDefect(draftUri, resultItemId, blocking,monitor, runAfterCreation);
		}), runAfterCreation, monitor);
	},

	queryExistingDefects: function(query, monitor, resultCallback) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.queryExistingDefects(query, monitor, resultCallback);
		}), resultCallback, monitor);
	},

	queryAssociatedDefects: function(linkType, isExecutionResultRelated, ewiItemIdOrUri, monitor, resultCallback) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.queryAssociatedDefects(linkType, isExecutionResultRelated, ewiItemIdOrUri, monitor, resultCallback);
		}), resultCallback, monitor);
	},

	associateDefect: function(linkType, linkDescription, monitor, runAfterAssociation) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.associateDefect(linkType, linkDescription, monitor, runAfterAssociation);
		}), runAfterAssociation, monitor);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_runWithCurrentProvider: function(runWithProvider, errorCallback, monitor) {
		this._runInitialized(dojo.hitch(this, function() {
			var provider= this._defectsProviders[this._currentProviderId].instance;
			
			if (provider == null || provider == undefined) {
				errorCallback(new Status(Status.ERROR, DefectsUIUtil.getMessage("labelUnknwonDefectsProvider", this._currentProviderId), null), null);
			} else if (!provider.isConfigured()) {
				errorCallback(new Status(Status.ERROR, DefectsUIUtil.getMessage("labelDefectsProviderNotInitialized", this._currentProviderId), null), null);
			} else {
				runWithProvider(provider);
			}
		}), monitor);
	},

	_runInitialized: function(runInitialized, monitor) {
		if (this._initializationState == INIT_UNINITIALIZED) {
			if (dojo.isObject(monitor)) {
				monitor.beginTask(DefectsUIUtil.getMessage("labelLoadDefectsProviderConfiguration"));
			}

			DefectsProviderClient.getDefectsProviders(dojo.hitch(this, function(status, result) {
				if (status.isOK()) {
					this._currentProviderId= result.currentDefectsProviderId;

					if (dojo.isObject(monitor)) {
						monitor.beginTask(DefectsUIUtil.getMessage("labelInitializeDefectsProviders"));
					}

					this._doInitialize(result.defectsProviders, dojo.hitch(this, function() {
						this._initializationState= INIT_DONE;
						runInitialized();
					}));
				} else {
					// FIXME we should propagate the status
					runInitialized();
				}
			}));
		} else {
			runInitialized();
		}
	},
	
	_doInitialize: function(defectsProviders, runAfterInitialization) {
		var extensionPoint= RegistryFactory.getRegistry().getExtensionPoint("com.ibm.rqm.defects.web.defectsProviders");

		var allProviderIds= [];
		if (defectsProviders) {
			this._defectsProviders = {};
			for (var i= 0; i < defectsProviders.length; i++) {
				this._defectsProviders[defectsProviders[i].id]= {
					id: defectsProviders[i].id,
					name: defectsProviders[i].name,
					providerImplementation: null,
					instance: null
				}
				allProviderIds.push(defectsProviders[i].id);
			}
	
			dojo.forEach(extensionPoint.getExtensions(), dojo.hitch(this, function(extension) {
				dojo.forEach(extension.getConfigurationElements(), dojo.hitch(this, function(configElement) {
					var id= configElement.getAttribute("id");
					var providerImplementation= configElement.getAttribute("providerImplementation");
					
					if (this._defectsProviders[id]) {
						if (this._defectsProviders[id].providerImplementation == null) {
							this._defectsProviders[id].providerImplementation= providerImplementation;
						} else {
							dojo.warn(DefectsUIUtil.getMessage("labelFoundDuplicateDefectsProvider", id));
						}
					} else {
						dojo.warn(DefectsUIUtil.getMessage("labelFountUnknownDefectsProvider", id));
					}
				}));
			}));
		}

		this._doInitializeProvider(allProviderIds, 0, function() {
			runAfterInitialization();
		});
	},

	_doInitializeProvider: function(allProviderIds, providerIndex, runAfterInitialization) {
		if (providerIndex < allProviderIds.length) {
			var providerDescriptor= this._defectsProviders[allProviderIds[providerIndex]];
			loader.load_async(providerDescriptor.providerImplementation, dojo.hitch(this, function() {
				var ctor= dojo.getObject(providerDescriptor.providerImplementation);
				var providerInstance= new ctor(providerDescriptor.id, providerDescriptor.name);
				providerInstance.initialize(dojo.hitch(this, function() {
					this._defectsProviders[providerDescriptor.id].instance= providerInstance;
					this._doInitializeProvider(allProviderIds, providerIndex + 1, runAfterInitialization);
				}));
			}));
		} else {
			runAfterInitialization();
		}
	},
	
	setUninitialized: function(){
		this._initializationState = INIT_UNINITIALIZED;
	},
	
	__sentinel: null // terminates this class definition
});

var MANAGER_INSTANCE= null;

com.ibm.rqm.defects.web.client.DefectsManager.getInstance= function() {
	if (MANAGER_INSTANCE == null) {
		var instance= new com.ibm.rqm.defects.web.client.DefectsManager();
		MANAGER_INSTANCE= instance;
	}
	
	return MANAGER_INSTANCE;
};

})();
