/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.rqm.defects.web.client.RequirementsManager");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");


dojo.require("net.jazz.ajax.core.internal.loader");
dojo.require("net.jazz.ajax.core.runtime.RegistryFactory");

dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.rqm.defects.web.client.internal.RequirementsProviderClient");
dojo.require("com.ibm.team.process.web.client.projectarea");

(function() {

var INIT_UNINITIALIZED= 0;
var INIT_IN_PROGRESS= 1;
var INIT_DONE= 2;
var INIT_FAILURE= 3;

var loader					= net.jazz.ajax.core.internal.loader;
var RegistryFactory 		= net.jazz.ajax.core.runtime.RegistryFactory;

var RequirementsProviderClient	= com.ibm.rqm.defects.web.client.internal.RequirementsProviderClient;
var getCurrentProjectArea	= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var Status					= com.ibm.team.repository.web.foundation.Status;

dojo.declare("com.ibm.rqm.defects.web.client.RequirementsManager", null, {

	_defectsProviders: null,
	_currentProviderId: null,

	constructor: function() {
		this._initializationState= INIT_UNINITIALIZED;
		this._requirementsProviders= {};
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	ensureInitialized: function(monitor, initializationCallback) {
		this._runInitialized(dojo.hitch(this, function() {
			initializationCallback();
		}), monitor);
	},


	getCurrentRequirementsProviderId: function() {
		return this._currentProviderId;
	},

	getRequirementsProviders: function() {
		return this._requirementsProviders;
	},

	getRequirementsProvider: function(providerId) {
		return this._requirementsProviders[providerId].instance;
	},

	queryExistingRequirements: function(query, monitor, resultCallback) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.queryExistingRequirements(query, monitor, resultCallback);
		}), resultCallback, monitor);
	},
	
	queryAssociatedRequirements: function(itemIdOrURI, monitor, resultCallback) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.queryAssociatedRequirements(itemIdOrURI, monitor, resultCallback);
		}), resultCallback, monitor);
	},

	associateRequirement: function(linkDescription, monitor, runAfterAssociation) {
		this._runWithCurrentProvider(dojo.hitch(this, function(provider) {
			provider.associateRequirement(linkDescription, monitor, runAfterAssociation);
		}), runAfterAssociation, monitor);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_runWithCurrentProvider: function(runWithProvider, errorCallback, monitor) {
		this._runInitialized(dojo.hitch(this, function() {
			var provider= this._requirementsProviders[this._currentProviderId].instance;
			
			if (provider == null || provider == undefined) {
				errorCallback(new Status(Status.ERROR, "Unknwon DefectsProvider " + this._currentProviderId, null), null);
			} else if (!provider.isConfigured()) {
				errorCallback(new Status(Status.ERROR, "DefectsProvider " + this._currentProviderId + " is not initialized", null), null);
			} else {
				runWithProvider(provider);
			}
		}), monitor);
	},

	_runInitialized: function(runInitialized, monitor) {
		if (this._initializationState == INIT_UNINITIALIZED) {
			if (dojo.isObject(monitor)) {
				monitor.beginTask("Load Requirements Provider Configuration");
			}

			RequirementsProviderClient.getRequirementsProviders(dojo.hitch(this, function(status, result) {
				if (status.isOK()) {
					this._currentProviderId= result.currentDefectsProviderId;

					if (dojo.isObject(monitor)) {
						monitor.beginTask("Initialize Requirements Providers");
					}

					this._doInitialize(result.defectsProviders, dojo.hitch(this, function() {
						this._initializationState= INIT_DONE;
						runInitialized();
					}));
				} else {
					// FIXME we should propagate the status
					runInitialized();
				}
			}));
		} else {
			runInitialized();
		}
	},
	
	_doInitialize: function(requirementsProviders, runAfterInitialization) {
		var extensionPoint= RegistryFactory.getRegistry().getExtensionPoint("com.ibm.rqm.defects.web.requirementsProviders");

		var allProviderIds= [];
		for (var i= 0; i < requirementsProviders.length; i++) {
			this._requirementsProviders[requirementsProviders[i].id]= {
				id: requirementsProviders[i].id,
				name: requirementsProviders[i].name,
				providerImplementation: null,
				instance: null
			}
			allProviderIds.push(requirementsProviders[i].id);
		}

		dojo.forEach(extensionPoint.getExtensions(), dojo.hitch(this, function(extension) {
			dojo.forEach(extension.getConfigurationElements(), dojo.hitch(this, function(configElement) {
				var id= configElement.getAttribute("id");
				var providerImplementation= configElement.getAttribute("providerImplementation");
				
				if (this._requirementsProviders[id]) {
					if (this._requirementsProviders[id].providerImplementation == null) {
						this._requirementsProviders[id].providerImplementation= providerImplementation;
					} else {
						dojo.warn("Found duplicate implementations for DefectsProvider with id %s", id);
					}
				} else {
					dojo.warn("Found implementation for unknown DefectsProvider with id %s", id);
				}
			}));
		}));

		this._doInitializeProvider(allProviderIds, 0, function() {
			runAfterInitialization();
		});
	},

	_doInitializeProvider: function(allProviderIds, requirementIndex, runAfterInitialization) {
		if (requirementIndex < allProviderIds.length) {
			var providerDescriptor= this._requirementsProviders[allProviderIds[requirementIndex]];
			loader.load_async(providerDescriptor.providerImplementation, dojo.hitch(this, function() {
				var ctor= dojo.getObject(providerDescriptor.providerImplementation);
				var providerInstance= new ctor(providerDescriptor.id, providerDescriptor.name);
				providerInstance.initialize(dojo.hitch(this, function() {
					this._requirementsProviders[providerDescriptor.id].instance= providerInstance;
					this._doInitializeProvider(allProviderIds, requirementIndex + 1, runAfterInitialization);
				}));
			}));
		} else {
			runAfterInitialization();
		}
	},
	
	__sentinel: null // terminates this class definition
});

var MANAGER_INSTANCE= null;

com.ibm.rqm.defects.web.client.RequirementsManager.getInstance= function() {
	if (MANAGER_INSTANCE == null) {
		var instance= new com.ibm.rqm.defects.web.client.RequirementsManager();
		MANAGER_INSTANCE= instance;
	}
	
	return MANAGER_INSTANCE;
};

})();
