/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Template");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Part");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Attribute");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Configuration");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Asset");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._Util");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.client.internal.transport.RawServiceRequest");
dojo.require("com.ibm.asq.common.web.client.internal.transport.RawTeamServerClient");

var ServiceResponseHandler= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Asset = com.ibm.asq.common.web.client.internal._catalog.Asset;
var Configuration = com.ibm.asq.common.web.client.internal._catalog.Configuration;
var Part = com.ibm.asq.common.web.client.internal._catalog.Part;
var Pointer = com.ibm.asq.common.web.client.internal._catalog.Pointer;
var Template = com.ibm.asq.common.web.client.internal._catalog.Template;
var ConfigurationCrudRawService = "com.ibm.rational.test.lm.catalog.common.IConfigurationCrudRawService";
var ReservationRawService = "com.ibm.rational.test.lm.common.IReservationRawService";
var SelectionCriteriaSearchRawService = "com.ibm.rational.test.lm.catalog.common.ISelectionCriteriaSearchRawService";
var _projectAreaId = null;
var RawTeamServerClient = com.ibm.asq.common.web.client.internal.transport.RawTeamServerClient;
var RawServiceRequest = com.ibm.asq.common.web.client.internal.transport.RawServiceRequest;

//
// Usage
// =====
//
// Class Catalog
// 
// The Catalog class (this file) provides the main interface to the
// catalog system. The system supports a template object, that is 
// fairly static (it only changes when the system is customized).  
// There is also a configuration object that supports specification of
// a model (a class of objects) and an Asset object which represents a
// specific instance of an object (also called a resolved 
// configuration).
//
// The Catalog class is the only class that needs to be included in 
// JavaScript code that uses the catalog system.  The class contains 
// methods which are factories for all other classes in the system.  
// Additionally, the Catalog class is the only class in the system 
// which has asynchronous methods.
//
// To initialize the template, call the initializeTemplates() method.
// This should only be called once in the current implementation, as it
// implements a singleton pattern.
//
// To get an enumeration of the templates, call the 
// getTemplateNames() method.  This returns an array of template 
// names.
// 
// Asset Templates are those that represent stand alone objects (things 
// that can be requested). To get an enumeration of the templates, call 
// the getAssetTemplateNames() method.  This returns an array of 
// template names.
// 
// To get the instance of the template, you call the 
// getTemplate(templateName) method. Note that the initializeTemplates()
// call is an asynchronous call, and you *MUST* wait for the response
// before making any subsequent calls into the catalog system.
// getTemplate(templateName) is a synchronous call and does not talk to 
// the server.  The template information is cached on the server, but 
// it takes about 6 seconds to populate the cache (in a development 
// environment). Note that this is cached for the up-time of the server,
// not for a particular browser session.  
//
// all methods on the Template, Configuration, Asset and Attribute
// objects are synchronous.
// 
// There are currently 5 apparent types of attribute values than can be
// contained in the catalog.  The simple types are numeric, string, and 
// timestamp (represented by a string).  There is also enumeration 
// (which is currently treated as a string) of which are two different 
// styles, Part types and enumerated types.  The final type is the 
// pointer type, and its value is a pair comprised of a uuid and a
// displayString.  The first instance of this is the owner attribute.  
// When you get the attribute, you will commonly get back an object 
// containing both the uuid and the displayString, but when you set it 
// you do not need to set the displayString.  It does not hurt
// to do so, and I would imagine the information is readily available, 
// so it may be best just to set the displayString as well.  The 
// current implementation has one instance of the pointer type to 
// support users.  In this case, the displayString is the name of the 
// user and the uuid is the itemId of the contributer in jazz.
//
// A template also contains a list of Parts.  A Part is a specific 
// instance of the type represented by the template.  For example, an 
// OperatingSystem template might contain a Part for Windows and a Part 
// for Linux.  Each template also contains a root Part, which is the 
// most generic instance of the type in the template.  For example, 
// in the OS template, there will be a root Part named 
// OperatingSystem.  Parts have attributes, which are the descriptors 
// for the Part, a parent (which will be null for the root Part), and 
// any number of child parts.
// Parts can be retrieved from the template in one of two ways.  First, 
// template.getPartNames() returns an array of all the parts in the 
// template.  A specific part can then be gotten by 
// template.getPart(partName).  The other way to get at the parts is by 
// calling template.getRoot().  This method returns the root Part of 
// the template.  Note that the name of the root Part will always be 
// the type name of the template.  The other Parts can then be gotten 
// via the child references in the part.
//
// There are two classes for interacting with assets in the catalog.  
// The first is Configuration.  A configuration is a representation of 
// a potential asset.  It is used for searches and for the creation of 
// new assets.  The second is Asset.  An Asset represents an instance 
// of an asset that exists in the database.  It is returned by searches 
// and is used for displaying, updating and deleting asset instance 
// data.
// 
// Configuration and Asset are very similar structurally and have many 
// similar methods.  Both contain a list of root parts which represent
// the assets (such as machines) which are contained in the object.  
// Each of these root Parts can be considered as a tree that can be 
// walked to get at the child parts and attributes of the object.  
// There is also a call to get all of the Parts in a single tree.  This 
// allows the data to be displaying in a more flattened fashion.  
// Editing of these assets is done by setting the values for the 
// Attributes of the Parts.
// The Part in a configuration and Asset supports methods getAttributeValue and
// setAttributeValue that provide convenient access to the value of an attribute.

/* pseudo code for the APIs:

    class Catalog {

        // Get all of the templates from the server and create 
        // Template objects for them.  Must be called prior to any
        // other calls into the catalog.
        initializeTemplates(responseHandler);
        
        // Get an array of the catalog's asset templates.  An asset
        // template contains Parts that can be realized (such as 
        // machines).
        String[]getAssetTemplateNames();
       
        // Get an array of the catalog's templates.
        String[]getTemplateNames();
       
        // Get a template from the catalog.
        Template getTemplate(String name);
        
        // Construct a Configuration.  This method can take a null 
        // parameter to construct an empty configuration or the 
        // name of an AssetTemplate to create a configuration with 
        // a rootPart of that type.
        Configuration createConfiguration(String);

        // Construct a Configuration.  This method can take a valid 
        // JSON object or JSON string.
        Configuration createConfigurationFromJson(Object json);

        // Construct a Configuration.  This method takes an Asset
		// as a parameter.  It copies all of the values from the
		// asset into the new Configuration.
        Configuration createConfigurationFromAsset(Asset asset);

        // Construct an Asset.  This method can take a null 
        // parameter to construct an empty configuration or the 
        // name of an AssetTemplate to create an Asset with 
        // a rootPart of that type.
        Asset createAsset(String);

        // Construct an Asset.  This method can take a valid 
        // JSON object or JSON string.
        Asset createAssetFromJson(Object json);

        // Construct a Pointer.  This object can be used to  
        // set the value of PointerType attributes.
        Pointer createPointer: function(name, uuid);
        
        // Asynchronous call to get an Asset by its Id and 
        // AssetType (such as "Machine").  The 
        // responseHandler's successful callback is called with the 
        // created Asset. 
        getAsset(responseHandler, String Id, String AssetType);

        // Asynchronous API to create an Asset on the server.  
        // The responseHandler's successful callback is 
        // called with the created Asset. 
        postCreateAsset(responseHandler, Asset asset);
        
        // Asynchronous API to search for Assets matching the
        // configuration.  The returned assets will be contain the 
        // data specified by the array of columns.  The 
        // responseHandler's successful callback is called with an 
        // array of asset arrays containing the requested data.
        // The searchParam object contains the following attributes:
        // configuration - required configuration to be matched
        // columns - required information on the column data to be returned.
        // index - optional numeric value of the asset within the
        //    configuration to be used in the search.
        // from - long integer value of the available from date
        // to - long integer value of the available to date.
        search(responseHandler,searchParam);

        // Asynchronous API to update an Asset.  The 
        // responseHandler's successful callback is called with the 
        // updated Asset. 
        updateAsset(responseHandler, Asset asset);
        
                
        // Asynchronous API to delete an Asset.  Multiple deletes 
        // are not considered an error.  The responseHandler's 
        // successful callback is called with an unspecified value. 
        deleteAsset(responseHandler, Asset asset);

        // Get a list of the names of "types" (ie Part names) to be 
        // used a left handed values in list displays.
        String[] Catalog.getDisplayTypes();
        
        // Get a list of the values for a display type
        String[] Catalog.getValuesForDisplayType(String displayType);
    }


    class Template {

        // Get the names of all the attributes in all of the Parts
        // in the template
        getAllAttributeNames();
        
        // Get the name of the template.
        String getName();
       
        // Get whether the template is an Asset Template.  An Asset 
        // Template is one that is a type that can exist 
        // independently for other types.  Types in an Asset 
        // template are things that could be requested.)
        boolean isAssetTemplate()
       
        // Get the names of the parts in the template.    Note that 
        // this list will never contain the name of the root Part. 
        String[] getPartNames();

        // Get a Part by name.
        Part getPart(String partName);

        // Get the template's root Part.
        Part getRoot();
       
    }

    class Part {

        // Get the name of the Part
        String getName();
       
        // Get the tag for the Part.  Each Part intsance has a unique tag
        // associated to it.  These tags are only only unique for a given
        // session and a given instance of a Part.  Thus if a part is fetched 
        // twice from the server, it will have a different tag in the second
        // instance.  The part tags can be used in the methods that access
        // parts from array attributes.
        String getTag();
        
        // Check if the part is a generic (ie Root) part
        boolean isGeneric();
        
        // Get the children of this Part.  For example, RedHat 
        // Linux might be a child of Linux.
        Part[] getChildParts();
       
        // Get a list of the names of all parts in the subtree of this
        // part.  Returns an array of strings.
        String[] getAllSubPartNames: function();
        
        // Get the parent of this Part.  This will return the template
        // if the Part is a root Part.
        Part getParent();
       
        // Get the names of the attributes for the Part.
        String[] getAttributeNames();
       
        // Get an attribute of the Part by its name.
        Attribute getAttribute(String attributeName);

        // Get an attribute value of the Part by its name.
		// If createPartIfMissing is true and the attribute value is null,
		// and the attribute represents a Part, a generic instance of the 
		// Part will be created and then returned.  createPartIfMissing
		// should NOT be called unless the attribute is a ConfigurationPart.
        Object getAttributeValue(String attributeName, createPartIfMissing);

        // Set an attribute value
        setAttributeValue(String attributeName, Object value);

        // Get the template that contains the Part
        Template getTemplate(); 
       
        // Get the root Part from the template that contains this 
        // Part. 
        Part getRoot();

        // Determine if an attribute is an array.
        boolean isAttributeArray(attributeName);

        // Get an attribute value of the Part by its name.
	    // If createPartIfMissing is true and the attribute value is null,
	    // and the attribute represents a Part, a generic instance of the 
	    // Part will be created and then returned.  This method is
        // only valid for non-arrays.
        object[] getAttributeArrayValues(attributeName, createPartIfMissing);
        
        // Get an attribute value of the Part by its name.
	    // If createPartIfMissing is true and the attribute value is null,
	    // and the attribute represents a Part, a generic instance of the 
	    // Part will be created and then returned.  This method is
        // only valid for arrays.
        object getAttributeArrayValue(attributeName, index, createPartIfMissing);
        
        // Get an attribute value of the Part by its name and part tag.
        // This method is only valid for part type attributes.
        // If createPartIfMissing is true and the attribute value is null,
        // and the attribute represents a Part, a generic instance of the 
        // Part will be created and then returned.  This method is
        // only valid for arrays.
        getAttributeArrayValueByTag(attributeName, tag, createPartIfMissing);

        // Add an attribute value of the Part.
        addAttributeArrayValue(attributeName, value);

        // Set an attribute value of the Part.
        setAttributeArrayValue(attributeName, index, value);

        // Set an attribute value of the Part.
        // This method is only valid for part type attributes.
        setAttributeArrayValueByTag(attributeName, tag, value);

        // Remove an attribute value of the Part.  This will
        // re-index the array of values.
        removeAttributeArrayValue(attributeName, index);

        // Remove an attribute value of the Part by the part tag.  This will
        // re-index the array of values.
        // This method is only valid for part type attributes.
        removeAttributeArrayValueByTag(attributeName, tag);

        // Mark an attribute as ignored in a search config. This
        // is only valid in a Configuration.
        setAttributeIgnored(attributeName, ignore);
 
        // Get if an attribute is ignored in a search config. This
        // is only valid in a Configuration.
        boolean isAttributeIgnored(attributeName);

}
    
    class Attribute {
       
        // Get the type of the attributes value.  Returns one of 
        // {'StringType', 'NumericType', 'PointerType',
        // 'ConfigurationPartType'}
        Object getType(); 
       
        // If the type of the attribute is 'ConfigurationPartType', 
        // this method can be called to get the specific name of 
        // the part.  Then getTemplate(partName) can be called to
        // get the detailed information about part names and 
        // attributes.
        String getPartTypeName();

        // If the type of the attribute is 'ConfigurationPartType', 
        // this method can be called to get all the possible type  
        // names for the part (really all the subtypes of the root).
		// This method can be called even on an Attribute from a Asset or 
		// Configuration where the value for the attribute is null.
		String[] getAllPartTypeNames();
        
        // If the type of the attribute is 'ConfigurationPartType', 
        // this method can be called to get root part from the template
        // for the specific attribute type.
		// This method can be called even on an Attribute from a Asset or 
		// Configuration where the value for the attribute is null.
		Part getRootPart();

        // If the type of the attribute is 'PointerType', 
        // this method can be called to get the pointer object 
        // that is associated with this attribute.  If the attribute
        // is an array, this method will return an array of Pointers.
        Object getPointer();
        
        // Determine if this attribute can contain an array of 
        // selected values.  If this method is true, the 
        // getArrayValues(), addArrayValue(), getArrayValue(),
        // setArrayValue() and removeArrayValue() methods become
        // valid. If this method is false, the getValue() and 
        // setValue() methods become valid. 
        boolean isArray();
       
        // Get the name of the attribute
        String getName();
       
        // Get the dotted long name of the attribute
        // The returned path is only valid on instance
        // attributes.  Not on attributes obtained 
        // from the template.
        String getPathName();
        
        // Get the values of the enum for this attribute.  This 
        // method is only valid for Enum types.
        String[] getEnumValues();
       
        // Get the current value of the attribute.  This method is 
        // only valid if the attribute is not an array. This will 
        // always be null for Attributes in Template Parts, but may 
        // be defined for the Attributes in Parts contained in 
        // Configurations and Assets.  
        Object getValue();
       
        // Get the current values of the attribute.  This method 
        // is only valid if the attribute is an array. This will 
        // always be null for Attributes in Template Parts, but may 
        // be defined for the Attributes in Parts contained in 
        // Configurations and Assets.
        Object[] getArrayValues();
       
        // Set the value of an attribute.  This method is only 
        // valid if the attribute is not an array. It is an error 
        // to attempt to set the attribute's value to an object of 
        // the incorrect type.  If there is an existing value 
        // present, it will be replaced by the new value.
        setValue(Object);

        // Add a value to the selection of values for the attribute. 
        // This method is only valid if the attribute is an array. 
        addArrayValue(object);
    
        // Get the value of the attribute at the given array
        // index.  This method is only valid if the attribute
        // is an array.
        Object getArrayValue(index);

        // Get the value of the attribute at the given array
        // index.  This method is only valid if the attribute
        // is an array.
        getArrayValueByTag(tag);

        // Set the value of the attribute at the given array
        // index.  This method is only valid if the attribute
        // is an array.
        setArrayValue(index, value);

        // Set the value of the attribute at the given array
        // tag.  If this method is called with a value of null,
        // the part at the corresponding index will be removed.
        // This method is only valid if the attribute
        // is an array.
        setArrayValueByTag(tag, value);

        // Remove the value of the attribute at the given array
        // index.  This method is only valid if the attribute
        // is an array.  This will re-index all of the values
        // in the array.
        removeArrayValue(index);

        // Remove the value of the attribute at the given tag
        // This method is only valid if the attribute
        // is an array.  This will re-index all of the values
        // in the array.
        removeArrayValueByTag(tag);

        // Determine if the attribute is required.`
        boolean isRequired();

        // Set the indicator that this attribute has been placed in the custom UI
        setMarked(isMarked);

        // Get the indicator that this attribute has been placed in the custom UI
        boolean isMarked();

        // Mark the attribute as ignored in a search config. This
        // is only valid in a Configuration.
        setIgnored(ignore);
 
        // Get if the attribute is ignored in a search config. This
        // is only valid in a Configuration.
        boolean isIgnored();

        // This method can be called to validate an attribute value
        // prior to attempting to set the value in the attribute.
        // This method is really only implemented for numeric types as
        // that type can have a complex search string.  For all other
        // types this method returns 0.  For numeric types a code is returned.
        //    VALIDATION_OK - The value is valid
        //    VALIDAIION_INVALID_CHARACTER - The value contains an invalid character.
        //    VALIDATION_INVALID_SEQUENCE - The value contains segments that are out
        //         of sequence, such an operator followed by an operator.
        //    VALIDATION_DUPLICATE_OPERATORS - The value contains two operators with
        //         the same type.
        //    VALIDATION_DISJOINTED - The search string will never contain results
        //         do to a disjointed pattern (such as "<1 >10").
        //    VALIDATION_INCOMPLETE - Incomplete Sequence.
        //    VALIDATION_EXTRA_COMPONENTS - Too many search components.
        int validate(value);
    }
    
    class Pointer {
        
        // Get the UUID associated with the pointer
    	String getUuid();

        // Set the UUID associated with the pointer
	    setUuid(uuid);
	
        // Get the display string associated with the pointer
	    getDisplayString();
	
        // Set the display string associated with the pointer
	    setDisplayString(displayString);

    }

    class Configuration {

    // Get the root attribute of the Configuration
    Attribute getRootAttribute();

    // Add a new root to the configuration.  The name of 
    // the type MUST be passed in to this method.
    addRoot(typeName);

    // Set a root Part of the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    setRootPart(index, typeName);

    // Remove a root Part from the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    removeRootPart(index);

    // Get the root Parts of the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    Part[] getRootParts();

    // Get a root Part of the Configuration.  This is only
    // valid for a Configuration where there can be multiple roots.
    Part getRootPart(index);

    // return the object as JSON.  Optionally pretty-print it.
    String toJson(prettyPrint);

    // Get a duplicate of the Configuration.
    Configuration clone();

    // Get the label for one of the assets
    String getLabel(index);

    // Get the label for one of the assets
    String getLabelByTag(tag);

    // Set the label for one of the assets
    setLabel(index, label);

    // Set the label for one of the assets
    setLabelByTag(tag, label);

    // Get the identifier for one of the assets
    // Identifiers are used to track the specific
    // reservation asociated to a configuration asset
    // in a request.
    String getIdentifier(index);

    // Get the identifier for one of the assets
    String getIdentifierlByTag(tag);

    // Set the identifier for one of the assets
    setIdentifier(index, identifier);

    // Set the identifier for one of the assets
    setIdentifierByTag(tag, identifier);

    // Inserts the root parts (along with all attribute values) from
    // a configuration into another configuration.
    insertConfig(config);

    // This method will set an attribute value based on a displayType
    // returned from Catalog.getDisplayTypes() and a value returned
    // from Catalog.getValuesForDisplayType(displayType).  No other
    // parameters are allowed.  This api assumes that there will
    // only be one value for each of the displayTypes.  Subsequent
    // calls to this method with the same displaytype will replace
    // the existing value with the new one.
    setValue(displayType, value);

    // This method will get an attribute value based on a displayType
    // returned from Catalog.getDisplayTypes().
    String getValue(displayType);

    }

    class Asset {

    // Get the root attribute of the Asset
    getRootAttribute: function(){

    // Set the root Part of the Asset.
    setRoot(typeName);

    // Set the root Part of the Asset.
    Part getRoot(typeName);

    // return the object as JSON.  Optionally pretty-print it.
    String toJson(prettyPrint);

    // Get a duplicate of the Asset.
    clone();

    }

*/

dojo.declare("com.ibm.asq.common.web.client.internal.Catalog", null,
{
    constructor: function()
    {
        this.serviceInterface = 'com.ibm.rational.test.lm.catalog.common.IConfigurationTemplateRawService';
        if (com.ibm.asq.common.web.client.internal.Catalog.theCatalog) {
            this.templateMap = com.ibm.asq.common.web.client.internal.Catalog.theCatalog.templateMap;
        }
        this._util = new com.ibm.asq.common.web.client.internal._catalog._Util();
    },
       
    /*
     * (non-API, do not use)
     *
     * Returns the correct service prefix based on the browser's security
     * context (e.g. http or https).
     */
     _getSecureOrPublic: function() {
        var regex= /^https/;
        return (location.href.match(regex) === null) ? "public" : "secure";
    },

    /*
     * (non-API, do not use)
     *
     * Returns the URL for our configuration crud calls.
     */
    _getConfigurationCrudUrl: function(opName) {
        return "/jazz/" +
               this._getSecureOrPublic() +
               "/service/com.ibm.rational.test.lm.catalog.common.IConfigurationCrudRawService/" +
               opName + 
               "?projectAreaItemId=" + com.ibm.team.process.web.projectArea.itemId;
    },
        
    /*
     * (non-API, do not use)
     *
     * Returns the URL for our provider calls.
     */
    _getProviderRestServiceUrl: function(opName) {
        return "/jazz/" +
               this._getSecureOrPublic() +
               "/service/com.ibm.rational.test.lm.catalog.common.IProviderRestService/" +
               opName;
    },
     
    /*
     * (non-API, do not use)
     *
     * Returns the project area id.  This is needed because we can't initially reference this
     * id.  It doesn't get set until after the login.
     */
    _getProjectAreaId: function() {
        if(_projectAreaId == null) {
			_projectAreaId = com.ibm.team.process.web.projectArea.itemId;
		}
		return _projectAreaId;
    },
    
    // Get all of the templates from the server and create Template objects for them.
    // Must be called prior to any other calls into the 
    // catalog.
    initializeTemplates: function(responseHandler) {
        var cat = com.ibm.asq.common.web.client.internal.Catalog;
        if(cat.theCatalog) {
            // Only need to initialize once.
            responseHandler._contextObject[responseHandler._callbackName]();
            return;
        }
        cat.theCatalog = {};

        if(cat.numTemplateTries == null){
			cat.numTemplateTries = 0;
		} 
		               
        // Attach the response handler for internal use
        dojo.xhrGet( {
            url: "/jazz/"+this._getSecureOrPublic()+"/service/com.ibm.rational.test.lm.catalog.common.IConfigurationTemplateRawService/assetTypes?projectAreaItemId=" + this._getProjectAreaId(),
            handleAs: "json",
            headers: { accept: "text/json" },
            load: dojo.hitch(this, function(assetTypes, ioArgs) {

                // Create the singleton catalog object
	            var theCatalog = cat.theCatalog;
                theCatalog.templateMap = [];
                this.templateMap = theCatalog.templateMap;

                var assetTypeArray = this._util._getArrayObject(assetTypes, "AssetTemplate", "AssetType");
				// On the first time a repository is used the catalog schema may not yet be initialised.
				// Try again until the repository is ready.
                if(assetTypeArray.length == 0){
                    delete(cat.theCatalog);
                    var message = MessageUtil.getMessage(MessageBundle.COMMON, "alertTemplateError");
                    alert(message);
                    
                    // the response handler expects an response object. At least
                    // pass in an object with the canned message.
					responseHandler.doErrorCallback({'message': message});
                    return;
                }

                var assetArray = [];
                
                // Callback for getting the templates
                var callbackObj = {
                    templateResponse: [],
                    rh: new ServiceResponseHandler({
                        _success: function(assetName, template, tr, numTemplates, crh){
                            tr.push({ name: assetName, result: "success" });
                            if (tr.length == numTemplates) {
                                var success = true;
                                for(var i=0; i<numTemplates; i++) {
                                    if(tr[i].result != "success") {
                                        success = false;
                                    }
                                }
                                if (success) {
                                    crh.doCallback(tr);
                                } else {
                                    crh.doErrorCallback(tr);
                                }
                            }
                        },
                        _failure: function(assetName, errObj, tr, numTemplates, crh){
                            tr.push({ name: assetName, result: errObj});
                            alert(MessageUtil.getMessage(MessageBundle.COMMON, "alertTemplateError"));
                            if (tr.length == numTemplates) {
                                crh.doErrorCallback(this._templateResponse);
                            }
                        }
                    }, 
                    "_success",
                    "_failure"),
                    callerRh: responseHandler
                }
                
                for(var i=0; i< assetTypeArray.length; i++) {
                    if (assetTypeArray[i].toplevel == "true") {
                        assetArray.push(assetTypeArray[i].name);
                    }
                }

                // Initialize a template for each asset in the array.
                this.numTemplates = assetArray.length;
                for (var i = 0; i < assetArray.length; i++) {
        			this._initializeTemplate(assetArray[i], callbackObj);
                }
            }),
            error: function(response,ioArgs){
                response.status = ioArgs.xhr.status;
				delete(cat.theCatalog);
                alert(MessageUtil.getMessage(MessageBundle.COMMON, "alertTemplateError"));
                responseHandler.doErrorCallback(response);
                return;
            }
        });
    },
    _initializeTemplate: function(asset, cbo){
        dojo.xhrGet( {
            url: "/jazz/"+this._getSecureOrPublic()+"/service/com.ibm.rational.test.lm.catalog.common.IConfigurationTemplateRawService/configurationTemplateByName?projectAreaItemId=" + this._getProjectAreaId() + "&name=" + asset,
            handleAs: "json",
            headers: { accept: "text/json" },
            asset: asset,
            cbo: cbo,
            load: dojo.hitch(this, function(template, ioArgs) {
                var asset = ioArgs.args.asset;
                var cbo = ioArgs.args.cbo;
                var responseHandler = cbo.rh;
                var t = this._processTemplate(template, true);
                responseHandler._contextObject[responseHandler._callbackName](asset, template, cbo.templateResponse, this.numTemplates, cbo.callerRh);
            }),
            error: function(response,ioArgs){
                response.status = ioArgs.xhr.status;
                var asset = ioArgs.args.asset;
                var cbo = ioArgs.args.cbo;
                var responseHandler = cbo.rh;
                responseHandler._contextObject[responseHandler._errorCallbackName](asset, response, cbo.templateResponse, this.numTemplates, cbo.callerRh);
            }
        });
    },
    // Get an array of the catalog's asset templates.  An asset
    // template contains Parts that can be realized (such as 
    // machines).
    getAssetTemplateNames: function(){
        var names = new Array();
        for(var template in this.templateMap) {
            if(this.templateMap[template].isAssetTemplate()){
                names.push(this.templateMap[template].getName());
            }
        }
        return names.sort();
    },
    // Get an array of the catalog's templates.
    getTemplateNames: function(){
        var names = new Array();
        for(var template in this.templateMap) {
            names.push(this.templateMap[template].getName());
        }
        return names.sort();
    },
    // Get a template from the catalog.
    getTemplate: function(templateName){
        return this.templateMap[templateName];
    },
    // Construct a Configuration.  This method can take a null 
    // parameter to construct an empty configuration or the 
    // name of an AssetTemplate to create a configuration with 
    // a rootPart of that type.
    createConfiguration: function(typeName){
        var config =  new Configuration(null, typeName);
        config.addRoot(typeName);
        return config;
    },
    // Construct a Configuration.  This method can take a valid 
    // JSON object or JSON string.
    createConfigurationFromJson: function(json){
        return new Configuration(json, null);
    },
    // Construct a Configuration.  This method takes an Asset
    // as a parameter.  It copies all of the values from the
    // asset into the new Configuration.
    createConfigurationFromAsset: function(asset){
        var config = this.createConfiguration(asset.typeName);
		var root = config.getRootPart(0);
		var oldRoot = asset.getRoot();
		// copy the values from the asset to the configuration
		root._copyValuesFromPart(oldRoot);
		return config;
    },
    // Construct an Asset.  This method can take a null 
    // parameter to construct an empty Asset or the 
    // name of an AssetTemplate to create a Asset with 
    // a rootPart of that type.
    createAsset: function(typeName){
        var asset =  new Asset(null, typeName);
        asset.setRoot(typeName);
        return asset;
    },
    // Construct an Asset.  This method can take a valid 
    // JSON object or JSON string.
    createAssetFromJson: function(json){
        return new Asset(json, null);
    },
	
	// create an configuration object that contains multiple root parts, one for each id.
	_createAssetArray: function(assetType, arrayOfIds){
		var asset = this.createAsset(assetType); 
		
		asset.instanceObj.Assets.AssetInstance = [];
		var assets = asset.instanceObj.Assets.AssetInstance;
		for (var i = 0; i < arrayOfIds.length; ++i){
			assets.push( {"uuid": arrayOfIds[i], "AssetType" : {"type" : assetType, "name": assetType } } );
							
		}
		return asset;		
	},
	// change the visibility of a collection of assets (lab resources). The assets are specified by an array of asset Ids.
	// The visibility is specified as "hide", "delete", or "restore"
	updateAssetsArrayVisibility: function(responseHandler, assetType, arrayOfAssetIds, visibility){
		var aa = this._createAssetArray(assetType,arrayOfAssetIds);
		var aaJson = dojo.toJson(aa.instanceObj);
	
    	this._updateAssetArrayVisibilityInternal(responseHandler,aaJson,visibility);
	},
	
	_updateAssetArrayVisibilityInternal: function(responseHandler,assetJson,action){
        var parms = { "projectAreaItemId": this._getProjectAreaId() };

        var method = "post" +
            action.substring(0,1).toUpperCase() +
            action.substring(1);

        var myServiceRequest= new RawServiceRequest( ConfigurationCrudRawService, 
                                                    method, 
                                                    parms,
                                                    null,
                                                    assetJson);
        
        RawTeamServerClient.invokeService( myServiceRequest, responseHandler );
	},
    // Construct a Pointer.  This object can be used to  
    // set the value of PointerType attributes.
    createPointer: function(name, uuid){
        return new Pointer(name, uuid);
    },
    // Asynchronous call to get an Asset by its Id.  The 
    // responseHandler's successful callback is called with the 
    // created Asset. 
    getAsset: function(responseHandler,configOrId,assetType){
        var configJson;
        if (typeof configOrId == 'string'){
            // an ID was specified.  We need to build a
            // Resolved Configuration for the query.
            var configFromId = this._getEmptyResolvedObject(configOrId, assetType);
            configJson = dojo.toJson(configFromId);
        }else{
            configJson = configOrId.toJson();
        } 
		var parms = { "projectAreaItemId": this._getProjectAreaId() };
        var myServiceRequest= new RawServiceRequest( ConfigurationCrudRawService, 
                                                    "postRetrieve", 
                                                    parms,
                                                    null,
                                                    configJson);
        
        var myResponseHandler = new ServiceResponseHandler(
            {
                _success: function(response){
                    var asset = new Asset(response, null);
                    responseHandler.doCallback(asset);
                },
                _failure: function(response){
                    responseHandler.doErrorCallback(response);
                }
            },
            "_success",
            "_failure"
		);
		                
        RawTeamServerClient.invokeService( myServiceRequest, myResponseHandler );
    },
	
    _getEmptyResolvedObject: function(id,assetType){
            var config = { Assets:
                { AssetInstance: {
                    uuid: id,
                    AssetType: {
                        type: assetType,
                        name: assetType,
                        Attributes: []
                    }
                }}};
            return config;
    },
    // Asynchronous API to create an Asset on the server from an 
    // Asset.  The responseHandler's successful 
    // callback is called with the created Asset. 
    postCreateAsset: function(responseHandler,asset){
        var parms = { "projectAreaItemId": this._getProjectAreaId() };
        var myServiceRequest= new RawServiceRequest( ConfigurationCrudRawService, 
                                                    "postCreate", 
                                                    parms,
                                                    null,
                                                    asset.toJson());
        
        var myResponseHandler = new ServiceResponseHandler(
            {
                _success: function(response){
                    var asset = new Asset(response, null);
                    responseHandler.doCallback(asset);
                },
                _failure: function(response){
                    responseHandler.doErrorCallback(response);
                }
            },
            "_success",
            "_failure"
        );
                        
        RawTeamServerClient.invokeService( myServiceRequest, myResponseHandler );
    },
    // Asynchronous API to search for Assets matching the
    // configuration.  The returned assets will be contain the 
    // data specified by the array of columns.  The 
    // responseHandler's successful callback is called with an 
    // array of asset arrays containing the requested data.  If the 
	// caller performs a straight search (by not including the to & from 
	// search parameters), the requested data will be augmented with per-
	// resource permissions information.  This information applies to 
	// the user's rights to perform the associated operations only; it does
	// NOT convey any further business logic (example: canReserve may be
	// true despite the fact that a returned resource is reserved for the
	// forseeable future - the flag indicates that the current user has
	// permission to attempt to reserve the resource, not that the 
	// resource is in a reservable state).  The permissions fields are:
    // 
    //   canAddToBf
	//   canAutomate
	//   canDiscover
    //   canEdit
	//   canReserve
	//
    // The searchParam object contains the following attributes:
    // configuration - required configuration to be matched
    // columns - required information on the column data to be returned.
	// visibility - optional; one of ["VISIBLE", "HIDDEN", "ARCHIVED"]; has no
	//    effect if from/to are specified
    // index - optional numeric value of the asset within the
    //    configuration to be used in the search.
    // from - optional long integer value of the available from date; must be
	//    specified with the to field
    // to - long integer value of the available to date; must be specified with
	//    the from field
    search: function(responseHandler,searchParam){
        var config = searchParam.configuration;
        if(searchParam.index != null){
            config = searchParam.configuration.getSubConfiguration(searchParam.index);
        }
            
        var service = "";
		var method = "";
		var parms = "";
        var additionalHeaders;
        if(searchParam.from && searchParam.from != ""
            && searchParam.to && searchParam.to != ""){
            service = ReservationRawService;
			method = "postSearchForAvailable";
			parms = null;
			additionalHeaders = {
				"projectAreaItemId": this._getProjectAreaId(),
                "selectioncriteria": searchParam.columns,
                "keeptags" : false,
                "visibility" : searchParam.visibility,
                "to": searchParam.to,
                "from": searchParam.from
			};
        }else{
            service = SelectionCriteriaSearchRawService;
			method = "postSelectioncriteriasearch";
			parms = null;
            additionalHeaders = { 
			    "projectAreaItemId": this._getProjectAreaId(),
                "selectioncriteria": searchParam.columns,
                "keeptags" : false,
                "visibility" : searchParam.visibility
			};
        }
        
		//This is a necessary call for IE.  IE does not accept null or undefined properties of an object.
        this._removeNulls( additionalHeaders );
        
        var myServiceRequest= new RawServiceRequest( service, 
                                                     method, 
                                                     parms,
													 additionalHeaders,
                                                     config.toJson());

        var myResponseHandler = new ServiceResponseHandler(
            {
                _success: function(searchResults){
                    responseHandler.doCallback(searchResults);
                },
                _failure: function(response){
                    responseHandler.doErrorCallback(response);
                }
            },
            "_success",
            "_failure"
        );

        RawTeamServerClient.invokeService( myServiceRequest, myResponseHandler );
    },

	_removeNulls: function( args )
	{
		for( var name in args )
		{
			if( args[ name ] == null ||
			    args[ name ] == undefined )
			{
				delete args[ name ];
			}
		}
		
		return( args );
	},
	
    // Asynchronous API to update an Asset.  The 
    // responseHandler's successful callback is called with the 
    // updated Asset. 
    updateAsset: function(responseHandler,asset){
        if (!asset.getId()){
            throw new Error("cannot update an asset unless it contains an ID - this asset is not resolved!");
        }
        var parms = { "projectAreaItemId": this._getProjectAreaId() };
        var myServiceRequest= new RawServiceRequest( ConfigurationCrudRawService, 
                                                    "postUpdate", 
                                                    parms,
                                                    null,
                                                    asset.toJson());
        
        var myResponseHandler = new ServiceResponseHandler(
            {
                _success: function(response){
                    var asset = new Asset(response, null);
                    responseHandler.doCallback(asset);
                },
                _failure: function(response){
                    responseHandler.doErrorCallback(response);
                }
            },
            "_success",
            "_failure"
        );
                        
        RawTeamServerClient.invokeService( myServiceRequest, myResponseHandler );
    },
    // Asynchronous API to delete an Asset.  Multiple deletes 
    // are not considered an error.  The responseHandler's 
    // successful callback is called with an unspecified value. 
    deleteAsset: function(responseHandler,asset){
        // TODO: rewrite to fit new data format
        if (!asset.getId()){
            throw new Error("cannot delete an asset unless it contains an ID - this asset is not resolved!");
        }
        var parms = { "projectAreaItemId": this._getProjectAreaId() };
        var myServiceRequest= new RawServiceRequest( ConfigurationCrudRawService, 
                                                    "postDelete", 
                                                    parms,
                                                    null,
                                                    asset.toJson());
        
        RawTeamServerClient.invokeService( myServiceRequest, responseHandler );
    },
    _processTemplate: function(object){
        // get the root name
        var template = new Template(object);
        var templateName = template.rootObj.type;
        // check if the template is already in the map
        if(this.templateMap[templateName] == null) {
            // Process the template and add it to the map
            var rootPart = this._processParts(template, template.rootObj, template);
            template.rootPart = rootPart;
            this.templateMap[templateName] = template;
        } else {
            // Use the existing one
            template = this.templateMap[templateName];
        }
        return template;
    },
    _processParts: function(parent, object, template){
        // Create a new Part
        var part = new Part(this, parent, null, object, null);
        part.isTemplatePart = true;
        this._processPartsInner(parent, part, template);
        return part;
    },
    _processPartsInner: function(parent, part, template){
        // Add the part to the template's part map
        template.partMap[part.getName()] = part;

        // Check the attibutes for this part only,
        // not for it's parent hierarchy.
        var attNames = new Array();
        var attributeArrayObject = part._util._getArrayObject(part.templateObj, "Attributes", "Attribute");
        for(var i=0; i<attributeArrayObject.length; i++) {
            attNames.push(attributeArrayObject[i].name);
        }

        for(var i=0; i<attNames.length; i++) {
            var att = part.getAttribute(attNames[i]);
            var type = att.getType();
            if(type == "ConfigurationPartType") {
                var newTemplate = this._processTemplate(att._getTemplateObj());
                template.referencedTemplateMap[newTemplate.getName()] = newTemplate;
            }
        }
        var subParts = part.getChildParts();
        for(var i=0; i<subParts.length; i++) {
            this._processPartsInner(part, subParts[i], template);
        }
    },
    // Get a list of the names of "types" (ie Part names) to be 
    // used a left handed values in list displays.
    getDisplayTypes: function() {
        var typeNames = new Array();
        typeNames.push("OperatingSystem");
        typeNames.push("CPU");
        var swTemplate = this.getTemplate("SoftwareInstall");
        var root = swTemplate.getRoot();
        var childParts = root.getChildParts();
        for(var i=0; i < childParts.length; i++) {
            typeNames.push(childParts[i].templateObj.name);
        }
        return typeNames.sort();
    },
    // Get a list of the values for a display type
    getValuesForDisplayType: function(displayType){
        if(displayType == "CPU"){
            var template = this.getTemplate("CPU");
            var part = template.getRoot();
            return part.getAllSubPartNames();
        }
        if(displayType == "OperatingSystem"){
            var template = this.getTemplate("OperatingSystem");
            var part = template.getRoot();
            return part.getAllSubPartNames();
        }
        var template = this.getTemplate("SoftwareInstall");
        var part = template.getPart(displayType);
        if(part == null){
            throw new Error("TypeName does not match an existing type");
        }
        return part.getAllSubPartNames();
    }
}    
)
})();
