/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function() { //module container

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var getCurrentPA            = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

com.ibm.asq.common.web.client.internal.PermissionsClient = {
    
	PERMISSION_SERVICE_URI: "com.ibm.rqm.process.deployment.rest.IPermissionRestService",
	
    OP_LOGIN: "com.ibm.rqm.login",
    OP_TEST_PLAN: "com.ibm.rqm.planning.testplan.save",
    OP_TEST_CASE: "com.ibm.rqm.planning.testcase.save",
	OP_TEST_CASE_EXE: "com.ibm.rqm.planning.testcase.execute",
    OP_TEST_SCRIPT: "com.ibm.rqm.planning.testscript.save",
    OP_TEST_SUITE: "com.ibm.rqm.execution.testsuite.save",
    OP_TEST_SUITE_EXE: "com.ibm.rqm.execution.testsuite.execute",
	OP_EXECUTION_STATE: "com.ibm.rqm.execution.result.state.save",
    OP_CONFIG_EWI: "com.ibm.rqm.configuration.ewi.save",
    OP_DATAPOOL: "com.ibm.rqm.datapool.datapool.save",
    OP_PROCESS_APPROVAL: "com.ibm.rqm.process.approval.save",
    OP_PROCESS_PHASE: "com.ibm.rqm.process.phase.save",
    OP_REQ_WORKITEM: "com.ibm.rqm.requirement.workitem.save",
    OP_CATEGORY_TYPE: "com.ibm.rqm.planning.categorytype.save",
    OP_CATEGORY: "com.ibm.rqm.planning.category.save",
    OP_TEMPLATE: "com.ibm.rqm.planning.template.save",
    OP_TEMPLATE_SECTION: "com.ibm.rqm.planning.template.section.save",	
    OP_EXECUTION_EWI: "com.ibm.rqm.execution.ewi.save",
    OP_EXECUTION_EWI_EXE: "com.ibm.rqm.execution.ewi.execute",
	OP_EXECUTION_RESULT: "com.ibm.rqm.execution.result.save",
	OP_SUITE_RESULT: "com.ibm.rqm.execution.testsuitelog.save",
    OP_ADAPTER: "com.ibm.rqm.adapter.registration.save",

    OP_LM_RESOURCE: "com.ibm.rqm.resource.save",
    OP_LM_RESOURCE_GROUP: "com.ibm.rqm.resourcegroup.save",
    OP_LM_TEST_ENVIRONMENT : "com.ibm.rqm.testenvironment.save",
    OP_LM_REQUEST: "com.ibm.rqm.request.save",
    OP_LM_DISCOVERY: "com.ibm.rational.test.lm.discovery.save",
    OP_LM_INVENTORY: "com.ibm.rational.test.lm.inventory.save",
    OP_LM_BUILDFORGE: "com.ibm.rational.test.lm.service.buildforge.save",
    OP_LM_VIRTUALIZATION: "com.ibm.rational.test.lm.virtualization.save",
	OP_LM_SCHEMA: "com.ibm.rqm.labresourceschema.save",
    OP_LM_COLLECTION: "com.ibm.rational.test.lm.collection.save",
	OP_LM_AUTOMATION: "com.ibm.rational.test.lm.automation.save",
    OP_ADMIN_OPTIONS: "com.ibm.rqm.option.save",
	    
	DEFAULT_ACTIONS: ["create", "edit", "archive"],
	ACTION_CREATE:  "create",
	ACTION_EDIT:    "edit",
	ACTION_ARCHIVE: "archive",
	ACTION_EXECUTE: "execute",
    ACTION_DELETE:  "delete",
	ACTION_LOCK:    "lock",

    ACTION_ADMINISTER:  "administer",
    ACTION_SYNCHRONIZE: "synchronize",
    ACTION_ADD:         "add",
    ACTION_RUN:         "run",
    ACTION_DEPLOY:      "deploy",
    
	_invokeService: function(responseHandler, method, args) {
		if (args && !args.processArea) {
			args.processArea = getCurrentPA().itemId;
		}
		if (args.action) {
			args.actions = [args.action];
		}		
		var serviceRequest= new ServiceRequest(this.PERMISSION_SERVICE_URI, method, args);
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
    checkPermissions: function(responseHandler, args) {
        //expected properties of 'args' object: operationId, action || actions[]  
		var jsonHandler = this.getJSONHandler(responseHandler);
        this._invokeService(jsonHandler, "getCheckPermission", args);
    },
	
    checkLicense: function(responseHandler, operationId) {
        //required param: operationId          
        var args = {
			operationId: operationId
		}
		//call TeamServerClient directly - do not inject Project Area by using _invokeService
		var serviceRequest = new ServiceRequest(this.PERMISSION_SERVICE_URI, "getCheckLicense", args);
		TeamServerClient.invokeService(serviceRequest, responseHandler);
    },
		
	/*
	 * Will do a recursive call to service to determine 
	 * the permissions for each operation specified:
	 *     operations:  [] of operation objects to check
	 *         { 
	 *             id:   operationId
	 *             actions:  [] of action ids
	 *         }
	 *             
	 */
    checkAllPermissions: function(responseHandler, operations) {
		this._checkAllPermissions(responseHandler, operations, {});		
    },	
	
    /*
     * Fetch and store all artifact permissions. Client can later use
     * the synchronous method to retrieve the stored permission.
     */
    fetchAndStoreAllPermissions: function(handler) {
        var operations = [
            {
                'id': this.OP_TEST_PLAN,
                'actions': this.DEFAULT_ACTIONS.concat([this.ACTION_LOCK])
            },
            {
                'id': this.OP_TEST_CASE,
                'actions': this.DEFAULT_ACTIONS.concat([this.ACTION_LOCK])
            },
			/*this.OP_TEST_CASE_EXE added since 1.0.1*/
			{
                'id': this.OP_TEST_CASE_EXE,
                'actions': [this.ACTION_EXECUTE]
            },
            {
                'id': this.OP_TEST_SCRIPT,
                'actions': this.DEFAULT_ACTIONS.concat([this.ACTION_LOCK])
            },
            {
                'id': this.OP_TEST_SUITE,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_TEST_SUITE_EXE,
                'actions': [this.ACTION_EXECUTE]
            },
            {
                'id': this.OP_CONFIG_EWI,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_EXECUTION_EWI,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_EXECUTION_EWI_EXE,
                'actions': [this.ACTION_EXECUTE]
            },
			{
                'id': this.OP_EXECUTION_RESULT,
                'actions': [this.ACTION_CREATE, this.ACTION_EDIT, this.ACTION_EXECUTE, this.ACTION_ARCHIVE]
            },
            {
                'id': this.OP_SUITE_RESULT,
                'actions': this.DEFAULT_ACTIONS
            },			
            {
                'id': this.OP_DATAPOOL,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_PROCESS_APPROVAL,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_PROCESS_PHASE,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_REQ_WORKITEM,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_CATEGORY_TYPE,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_CATEGORY,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_TEMPLATE,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_TEMPLATE_SECTION,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_ADMIN_OPTIONS,
                'actions': [this.ACTION_EDIT]
            },
            {
                'id': this.OP_LM_RESOURCE,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_LM_RESOURCE_GROUP,
                'actions': [this.ACTION_CREATE, this.ACTION_EDIT]
            },
			{
				'id': this.OP_LM_AUTOMATION,
				'actions': [this.ACTION_RUN]
			},
            {
                'id': this.OP_LM_TEST_ENVIRONMENT,
                'actions': this.DEFAULT_ACTIONS
            },
            {
                'id': this.OP_LM_REQUEST,
                'actions': [this.ACTION_CREATE, this.ACTION_ADMINISTER]
            },
            {
                'id': this.OP_LM_DISCOVERY,
                'actions': [this.ACTION_RUN]
            },
            {
                'id': this.OP_LM_VIRTUALIZATION,
                'actions': [this.ACTION_DEPLOY]
            },
            {
                'id': this.OP_LM_INVENTORY,
                'actions': [this.ACTION_SYNCHRONIZE, this.ACTION_DEPLOY]
            },
			{
                'id': this.OP_LM_SCHEMA,
                'actions': [this.ACTION_CREATE]
            },
            {
                'id': this.OP_ADAPTER,
                'actions': [this.ACTION_EDIT, this.ACTION_DELETE]
            },
			{
                'id': this.OP_LM_COLLECTION,
                'actions': [this.ACTION_EDIT]
			}    
        ];
        var self = this;
        var internalHandlerObj = {
            success: function(response) {
                self.storedPermission = response;
                handler.doCallback(response);
            },
            failure: function(e) {
                handler.doErrorCallback(e);
            }
        }
        var responseHandler = new ServiceResponseHandler(internalHandlerObj, "success", "failure");
        this.checkAllPermissions(responseHandler, operations);        
    },
    
    checkLogin: function(handler) {
        var self = this;
        var internalHandlerObj = {
            success: function(response) {
//                self.storedPermission = response;
                handler.doCallback(response);
            },
            failure: function(e) {
                handler.doErrorCallback(e);
            }
        }
        var responseHandler = new ServiceResponseHandler(internalHandlerObj, "success", "failure");
        this.checkLicense(responseHandler, this.OP_LOGIN);        
    },
    
   checkIsProjectAreaAdmin: function(handler)
   {
        var self = this;
        var internalHandlerObj = {
            success: function(response)
			{
                handler.doCallback(response);
            },
            failure: function(e)
			{
                handler.doErrorCallback(e);
            }
        }
        var responseHandler = new ServiceResponseHandler(internalHandlerObj, "success", "failure");
		this._invokeService(responseHandler, "getIsProjectAreaAdmin", {});
    },
 
     /*
     * Returns the previously fetched permission for an operation. This is a synchronous call and 
     * it's only valid if fetchAndStoreAllPermissions has been called.
     */
    getStoredPermission: function(operation) {
        if(this.storedPermission)
            return this.storedPermission[operation];
        return null;    
    },
	
	/*
	 * 
	 */
	getUserProjectAreas: function(responseHandler) {
        var serviceRequest = new ServiceRequest(this.PERMISSION_SERVICE_URI, "getUserProjectAreas", {});
        TeamServerClient.invokeService(serviceRequest, responseHandler);		
	},
    
	_checkAllPermissions: function(handler, operations, allPermissions) {
        //check permissions in order to properly render UI
        var opToCheck = null;
        for (var i = 0; i < operations.length; i++) {
			if (!allPermissions[operations[i].id]) {
				opToCheck = operations[i];
				break;
			}
		}
        if (opToCheck != null) {
            var self = this;
            var args = {
                operationId: opToCheck.id,
                actions: opToCheck.actions
            }
            var handlerObj = {
                success: function(opPermissions){
                    allPermissions[opToCheck.id] = opPermissions;
                    self._checkAllPermissions(handler, operations, allPermissions);
                },
                failure: function(e) {
                    handler.doErrorCallback(e);
                }
            }
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            this.checkPermissions(responseHandler, args);

        } else {
            handler.doCallback(allPermissions);
        }  		
	},
	
	/*
	 * Wrap the ServiceResponseHandler to automatically parse the JSON string from 
	 * some server calls and provide the JSON object to the original handler  
	 */
    getJSONHandler: function(originalHandler){
        var jsonHandlerObj = {
            success: function(/*String */ permissions){
                //wrap the JSON transformation                
                originalHandler.doCallback(dojo.fromJson(permissions));
            },
            failure: function(error){
                originalHandler.doErrorCallback(error);
            }
        };
        return new ServiceResponseHandler(jsonHandlerObj, "success", "failure");
    }	
};

})();
