/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.client.internal.QueryClient");

dojo.require("com.ibm.rqm.planning.web.client.internal.ProjectArea");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

(function() { //module container

// Constants (Note: These depend on corresponding values defined on the server)
var /* UIItemDTO */ ATTRIBUTE_ALL= { id: "all" };
var /* UIItemDTO */ OPERATOR_CONTAINS= { id: "contains" }; 

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var findIf					= net.jazz.ajax.toolbox.lang.findIf;
var removeIf				= net.jazz.ajax.toolbox.lang.removeIf;

var getCurrentProjectArea	= com.ibm.rqm.planning.web.client.internal.ProjectArea.getCurrentProjectArea;

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

com.ibm.rqm.planning.web.client.internal.QueryClient= {

	WI_QUERY_SERVICE_URI: "com.ibm.team.workitem.common.internal.rest.IQueryRestService",
	RQM_WRAPPED_QUERY_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.ITestPlanRestService",
//	QUERY_SERVICE_URI: "com.ibm.rqm.test.common.internal.rest.IQueryRestService",
	QUERY_SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.IFullTextRestService",
	
	///////////////////////////////////////////////////////////////////////////////////
	// Service invocation helpers
	///////////////////////////////////////////////////////////////////////////////////

	_invokeJsonWiService: function(responseHandler, method, appArgs) {
		var actualArgs= {};
		if (appArgs) {
			var jsonArgs= dojo.toJson(appArgs);
			actualArgs.jsonString= jsonArgs;
			actualArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		var serviceRequest= new ServiceRequest(this.WI_QUERY_SERVICE_URI, method, actualArgs);
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	_invokeWiQueryService: function(responseHandler, method, appArgs) {
		var self= this;
		
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 2) {
			var serviceRequest= new ServiceRequest(self.WI_QUERY_SERVICE_URI, method, { projectAreaItemId : getCurrentProjectArea().itemId });
		} else {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
			serviceRequest= new ServiceRequest(self.WI_QUERY_SERVICE_URI, method, appArgs);
		}
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},

	_invokeQueryService: function(responseHandler, method, appArgs) {
		var self= this;
		
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 2) {
			var serviceRequest= new ServiceRequest(self.QUERY_SERVICE_URI, method, { projectAreaItemId : getCurrentProjectArea().itemId });
		} else {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
			serviceRequest= new ServiceRequest(self.QUERY_SERVICE_URI, method, appArgs);
		}
		
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},

	_invokeWrappedQueryService: function(responseHandler, method, appArgs) {
		var self= this;
		
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 2) {
			var serviceRequest= new ServiceRequest(self.RQM_WRAPPED_QUERY_SERVICE_URI, method, { projectAreaItemId : getCurrentProjectArea().itemId });
		} else {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
			serviceRequest= new ServiceRequest(self.RQM_WRAPPED_QUERY_SERVICE_URI, method, appArgs);
		}
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
		
	///////////////////////////////////////////////////////////////////////////////////
	// IQueryRestService API
	///////////////////////////////////////////////////////////////////////////////////
	
	getGenericSearch: function(serviceResponseHandler, str)
	{
//		var args = {text: str, searchScope: null, constrainType: null};
		var args = {
			projectAreaItemId: getCurrentProjectArea().itemId,
			text: str
		};
		this._invokeQueryService(serviceResponseHandler, "getFind", args);
	},
	
	/**
	 * Loads a query by its itemId in partial state ({@link IQueryDescriptor#SMALL_PROFILE}, 
	 * excluding the expression.
	 * @return The query including its common attributes like name, as a QueryDTO in its JavaScript representation.
	 * 
	 * @see IQueryRestService#getQuery
	 */
	getQuery: function(serviceResponseHandler, itemId, includeCount) {
		if (!itemId)
			itemId= "";
		includeCount= includeCount ? true : false;
		this._invokeWiQueryService(serviceResponseHandler, "getQuery", {itemId: itemId, includeCount: includeCount });
	},
	
	/**
	 * Loads a query by its itemId in full state ({@link IQueryDescriptor#FULL_PROFILE}, 
	 * including the expression.
	 * @return The query including its expression tree as a QueryDTO in its JavaScript representation.
	 * 
	 * @see IQueryRestService#getQuery
	 */
	getCompleteQuery: function(serviceResponseHandler, itemId, includeCount) {
		if (!itemId)
			itemId= "";
		includeCount= includeCount ? true : false;
		this._invokeWiQueryService(serviceResponseHandler, "getQuery", {itemId: itemId, includeExpression: true, includeCount: includeCount});
	},
	
	/**
	 * See IQueryRestService#postSave
	 */
	save: function(query, serviceResponseHandler) {
		this._invokeJsonWiService(serviceResponseHandler, "postSave", query);
	},
	
	/**
	 * Executes a query and returns one page of the result set. 
	 * The use of the parameters is different when the query is initially
	 * executed and for subsequent calls to fetch additional data.
	 * 
	 * The initial call has to specify startIndex, maxResults and jsonQuery.
	 * Subsequent calls have to specifyx startIndex, maxResults and token. The
	 * jsonQuery parameters is optional and only used if the query result identified
	 * by token is not available anymore and the query has to be executed.
	 * 
	 * @param startIndex The index of the first work item to be included in the result page,
	 * 		relative to the entire result set.
	 * @param maxResults The number of results that should be returned.
	 * @param token The token identifying the result set. This token is available
	 * 		from the WorkItemSummaryPage that was returned in the initial call.
	 *		Note: This is not true yet. The token field has to be added to WorkItemSummaryPage first.
	 * @param query
	 * 		The query as retrieved by getQueriesForOwner or getPredefinedQueries or getQuery. If the
	 * 		query is stored in the repository, a simple object like { itemId: ... } is sufficient, too.
	 * @param orderAttributes
	 *      An array of attribute identifiers that describes which columns are to be sorted.
	 *		E.g. [ "modifiedDate" ]
	 * @param orderDirections
	 *		A boolean array of the same size as orderAttributes, that specifies for each attribute if
	 *		the order direction should be ascending (true) or descending (false).
	 *		E.g. [ true ]
	 * 
	 * Returns the JavaScript representation of an IWorkItemSummaryPage
	 */
	execute: function(startIndex, maxResults, token, query, orderAttributes, orderDirections, serviceResponseHandler) {
		var args= new Object();
		args.startIndex= startIndex;
		args.maxResults= maxResults;
		args.token= token;
		args.jsonQuery= dojo.toJson(query);
		args.orderAttributes= orderAttributes;
		args.orderDirections= orderDirections;
		//this._invokeWiQueryService(serviceResponseHandler, "postExecute", args);
		this._invokeWrappedQueryService(serviceResponseHandler, "postExecuteQuery", args);
	},
	
	/**
	 * See IQueryRestService#getResults
	 */
	getResults: function(args, serviceResponseHandler) {
		this._invokeWiQueryService(serviceResponseHandler, "getResults", args);
	},
	
	/**
	 * See IQueryRestService#getPredefinedQueries
	 */
	getPredefinedQueries: function(serviceResponseHandler, includeCount) {
		includeCount= includeCount ? true : false;
		this._invokeWiQueryService(serviceResponseHandler, "getPredefinedQueries", { includeCount: includeCount });
	},
	
	/**
	 * See IQueryRestService#getQueriesForOwner.
	 */
	getQueriesForOwner: function(appArgs, serviceResponseHandler) {
		this._invokeWiQueryService(serviceResponseHandler, "getQueriesForOwner", appArgs);
	},
	
	/**
	 * See IQueryRestService#getDefaultQuery.
	 */
	getDefaultQuery: function(serviceResponseHandler) {
		this._invokeWiQueryService(serviceResponseHandler, "getDefaultQuery");
	},
	
	/**
	 * See IQueryRestService#getOperators
	 */
	getOperators: function(/* String[] */ attributeIdentifiers, serviceResponseHandler) {
		this._invokeWiQueryService(serviceResponseHandler, "getOperators", { ids: attributeIdentifiers });
	},
	
	/**
	 * See IQueryRestService#getVariables
	 */
	getVariables: function(/* String[] */ attributeIdentifiers, serviceResponseHandler) {
		this._invokeWiQueryService(serviceResponseHandler, "getVariables", { ids: attributeIdentifiers });
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Client side query creation
	///////////////////////////////////////////////////////////////////////////////////
	
	/* QueryDTO */ createFulltextQuery: function(/* String */ queryName, /* String */ pattern) {
		
		var query= new Object(); // QueryDTO
		var expression= new Object(); //ExpressionDTO

		expression.attribute= ATTRIBUTE_ALL;
		expression.operator= OPERATOR_CONTAINS;
		expression.values= [ { id: pattern } ];
		
		query.name= queryName;
		query.expression= [ expression ];
		query.itemId= "";
		
		return query;
	}
	
};

})();
