/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog.Configuration");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._ConfigurationBase");
dojo.declare("com.ibm.asq.common.web.client.internal._catalog.Configuration", com.ibm.asq.common.web.client.internal._catalog._ConfigurationBase, 
{
    // Get the label for one of the assets
    getLabel: function(index){
        return this.rootAttribute.getLabel(index);
    },
    // Get the label for one of the assets
    getLabelByTag: function(tag){
        return this.rootAttribute.getLabelByTag(tag);
    },
    // Set the label for one of the assets
    setLabelByTag: function(tag, label){
        this.rootAttribute.setLabelByTag(tag, label);
    },
    // Set the label for one of the assets
    setLabel: function(index, label){
        this.rootAttribute.setLabel(index, label);
    },
    // Get the identifier for one of the assets
    getIdentifier: function(index){
        return this.rootAttribute.getIdentifier(index);
    },
    // Get the identifier for one of the assets
    getIdentifierByTag: function(tag){
        return this.rootAttribute.getIdentifierByTag(tag);
    },
    // Set the identifer for one of the assets
    setIdentifierByTag: function(tag, identifier){
        this.rootAttribute.setIdentifierByTag(tag, identifier);
    },
    // Set the identifier for one of the assets
    setIdentifier: function(index, identifier){
        this.rootAttribute.setIdentifier(index, identifier);
    },
    // Get a duplicate of the Configuration.
    clone: function() {
        var jsonStr = dojo.toJson(this.instanceObj,false);
        return new com.ibm.asq.common.web.client.internal._catalog.Configuration(jsonStr);
    },
    getSubConfiguration: function(index){
        var clonedConfig = this.clone();
        var newConfig = new com.ibm.asq.common.web.client.internal._catalog.Configuration(null, this.typeName);
        var part = clonedConfig.getRootPart(index);
        var rootAtt = newConfig.rootAttribute;
        rootAtt.addArrayValue(part);
        return newConfig;
    },
    insertConfig: function(config){
        // ensure that the types are the same
        if(this.typeName != config.typeName){
            throw new Error("Cannot insert " + config.typeName + " parts into a " + this.typeName + " Configuration.");
        }
        var tempConfig = config.clone();
        var parts = tempConfig.getRootAttribute().getArrayValues();
        var rootAtt = this.getRootAttribute();
        for(var i=0; i<parts.length; i++){
            rootAtt.addArrayValue(parts[i]);
        }
    },
    setValue: function(displayType, value){
 		this.setValueByIndex(0, displayType, value);
    },
	
    setValueByIndex: function(partIndex, displayType, value){
		if (partIndex >= this.getRootParts().length)
			return;
        var root = this.getRootPart(partIndex);
        if (displayType == "OperatingSystem") {
            var osAtt = root.getAttribute("operatingsystem");
            if (osAtt.getArrayValues().length != 0) {
                osAtt.setArrayValue(0, value);
            }
            else {
                osAtt.addArrayValue(value);
            }
        }
        else if (displayType == "CPU") {
            root.setAttributeValue("cpu", value);
        }
        else {
            var osAtt = root.getAttribute("operatingsystem");
            
            // auto create the OS part to hold the SW attribute.
            if (osAtt.getArrayValues().length == 0) {
                osAtt.addArrayValue(osAtt.getRootPart().getName());
            }
            var osPart = osAtt.getArrayValue(0);
            var swAtt = osPart.getAttribute("SoftwareInstall");
            // get the existing parts
            var parts = swAtt.getArrayValues();
            // check if a value for this subtype has already been set
            for (var i = 0; i < parts.length; i++) {
                var part = parts[i];
                if (part.isAncestor(displayType)) {
                    swAtt.setArrayValue(i, value);
                    return;
                }
            }
            swAtt.addArrayValue(value);
        }
    },
	
    getValue: function(displayType){
		return this.getValueByIndex(0, displayType);
	},

    getValueByIndex: function(partIndex, displayType){
		if (partIndex >= this.getRootParts().length)
			return;
        var root = this.getRootPart(partIndex);
        if (displayType == "OperatingSystem") {
            var osAtt = root.getAttribute("operatingsystem");
            var osPart = osAtt.getArrayValue(0);
            if(osPart != null){
                return osPart.getName();
            }
            return null;
        }
        else if (displayType == "CPU") {
            var cpuPart =  root.getAttributeValue("cpu");
            if(cpuPart != null){
                return cpuPart.getName();
            }
            return null;
        }
        else {
            var osAtt = root.getAttribute("operatingsystem");
            if (osAtt.getArrayValues().length > 0) {
                var osPart = osAtt.getArrayValue(0);
                var swAtt = osPart.getAttribute("SoftwareInstall");
                // get the existing parts
                var parts = swAtt.getArrayValues();
                // check for a value for this subtype
                for (var i = 0; i < parts.length; i++) {
                    var part = parts[i];
                    
                    // NOTE: if we ever going to disallow the 
                    // software root types, we should remove
                    // the condition of showing the root types
                    //
                    if (part.isAncestor(displayType) || part.getName() == displayType) {
                        return part.getName();
                    }
                }
            }
            return null;
        }
    }

    
});
})();
