/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*
 *
 *  This class is specific to machines.  It extends the lmContainer object.
 *
 */
dojo.require("dojo.io.iframe");
dojo.require("com.ibm.rational.test.lm.web.client.internal.lmContainer");
dojo.provide("com.ibm.rational.test.lm.web.client.internal.machine");

/*
 * Non-API object used for interacting with the server's import REST service. 
 */
dojo.declare("com.ibm.rational.test.lm.web.client.internal._importRest", com.ibm.rational.test.lm.web.client.internal.lmContainer, 
    {
		constructor: function()
		{
			this.setServiceInterface('com.ibm.rational.test.lm.common.IXmlImportRestService');
		},
		
		getAllowedCatalogAssetOps: function(responseHandler)
		{
			return this._invokeService(responseHandler, 'getAllowedCatalogAssetOps', { 'projectAreaItemId' : com.ibm.team.process.web.projectArea.itemId });
		},

		getUploadStatus: function(responseHandler, importId)
		{
			var args = {};
			
            args["id"] = importId;
			return this._invokeService(responseHandler, 'getStatus', args);
		}
	}
);

dojo.declare("com.ibm.rational.test.lm.web.client.internal.machine", com.ibm.rational.test.lm.web.client.internal.lmContainer, 
    
    {
	    constructor: function(){
        	this.setServiceInterface('com.ibm.rational.test.lm.common.IReservationRestService');
        	console.debug("initialized serviceInterface: "+this.serviceInterface);
			this._importRest = new com.ibm.rational.test.lm.web.client.internal._importRest();
    	},
    
        //
        //Any custom (non-CRUD) properties of a machine would go here.
        //
        _getHost : function(url) {
        	var h = url.match(/^[^\/]+\/\/[^\/]+\//);
        	return h;
        },
        /*
		 * (non-API, do not use)
		 *
		 * Returns the correct service prefix based on the browser's security
		 * context (e.g. http or https).
		 */
		_getSecureOrPublic: function() {
			var regex= /^https/;
			return (location.href.match(regex) === null) ? "public" : "secure";
		},

        /*
         * Returns an array of string IDs representing the import operations
         * that the current user is allowed to perform. The array may be empty!
         * The values that appear in the return array are limited to:
         * 
         *   1. "asset": the user may import new lab assets 
         *   2. "retrieve": the user may retrieve configuration instances
         *   3. "create": the user may create configuration instances
         *   4. "update": the user may update configuration instances
         *   5. "hide": the user may hide configuration instances
         *   6. "unhide": the user may reveal hidden configuration instances
         *   7. "delete": the user may delete configuration instances
         *   8. "undelete": the user may revive deleted configuration instances 
         */
        getAllowedCatalogAssetOps: function(responseHandler) {
			return this._importRest.getAllowedCatalogAssetOps(responseHandler);
		},

        /*
         * Upload a file whose contents are to be imported into the repository.
         * This method returns a deferred service call (notice that it does not
         * take a response handler).
         * 
         * formElement: the ID of the form element that contains the file 
         *              contents to be uploaded
         * opId: the particular import operation ID; must be one of the
         *       import-related strings returned by getAllowedCatalogAssetOps().
         *       
         * Returns a string UUID that should be passed to successive calls to 
         * getUploadStatus().
         */
        uploadImportFile: function(formElement, opId) {
            var formNode = dojo.byId(formElement);
            // var host = this._getHost(formNode.baseURI);
            // formNode.baseURI is undefined in IE, we don't need to specify the
            // hostname in the URL if it is on the same host anyway
            //
            
            var deferred = dojo.io.iframe.send({
                url: "/jazz/"+this._getSecureOrPublic()+"/service/com.ibm.rational.test.lm.common.IXmlImportRawService/"+opId+"?projectAreaItemId="+com.ibm.team.process.web.projectArea.itemId,
                method: "POST",
                handleAs: "text",
				timeout: 300000, // 5 mins
                form: formNode, // the dom node of the form containing the file reference. 
                load: function(importId, msg) {
                    console.debug('Registered import with ID ' + importId);
                    return importId;
                },
                error: function(type, msg) {
                    console.debug('Encountered upload error while importing.');
                    return type;
                }
            });
            return deferred;
        },

        /*
         * Get the current status of an upload previously triggered by a call to
         * uploadImportFile().  Returns a structured object that maps to the
         * ImportProgressDTO modelled type.  Data fields:
         * 
         * importId: The import ID included as an input to this method
         * importType: A string representing the type of import attempt; meant
         *             solely as a debugging aid
         * importStatus: An integer status enumeration value (enums TBD; for
         *               now this will always be 0 indicating success)
         * numImportableItems: The import type-appropriate number of items that
         *                     were expected to be imported based on the number
         *                     of such items deserialized from the input XML
         *                     (for now this will always be 1)
         * numImportedItems: The number of items that have successfully been
         *                   imported thus far
         * errorDetails: An array of data objects that describe the particular
         *               errors that have been encountered while attempting to
         *               execute the post-deserialization import operations
         *               (currently not populated)
         * 
         * Note that numImportableItems, numImportedItems and errorDetails are 
         * not valid fields until importStatus indicates that parsing has 
         * succeeded and/or the overall import attempt is complete.
         * 
         * Note further that the import attempt (identified by importId) will
         * only remain available on the server for a limited time (the record
         * is not persisted in the repository).  Any errors encountered in
         * calling this method likely indicate that the corresponding import
         * record has been retired.
         */
        getUploadStatus: function(responseHandler, importId) {
            return this._importRest.getUploadStatus(responseHandler, importId);
        },

        // Get the reservation info (array of ReservationDTOs) associated with 
        // a machine, where machine is specified with its uuid.
        getReservationsFromMachine: function(responseHandler, uuid) 
        {
            var args = {};
            args["uuid"] = uuid;
            return this._invokeService(responseHandler, "getReservationsFromMachine", args);        
        }
    }
)
