/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.rqm.adapter.rtc.web.provider.internal.TeamConcertRequirementsProvider");

dojo.require("com.ibm.rqm.defects.web.spi.RequirementsProvider");

dojo.require("com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient");
dojo.require("com.ibm.rqm.adapter.rtc.web.client.internal.TeamConcertConnectionClient");
dojo.require("com.ibm.rqm.adapter.rtc.web.ui.internal.config.TeamConcertWizard");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.foundation.Status");

dojo.require("com.ibm.rqm.adapter.rtc.web.client.internal.ExternalProjectAreaDescription");
dojo.require("com.ibm.team.process.web.client.projectarea");

(function() {

var RemoteTeamConcertClient			= com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient;
var TeamConcertConnectionClient		= com.ibm.rqm.adapter.rtc.web.client.internal.TeamConcertConnectionClient;
var TeamConcertWizard				= com.ibm.rqm.adapter.rtc.web.ui.internal.config.TeamConcertWizard;

var ServiceResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Status							= com.ibm.team.repository.web.foundation.Status;

var ExternalProjectAreaDescription	= com.ibm.rqm.adapter.rtc.web.client.internal.ExternalProjectAreaDescription;
var getCurrentProjectArea			= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

dojo.declare("com.ibm.rqm.adapter.rtc.web.provider.internal.TeamConcertRequirementsProvider", 
			 [com.ibm.rqm.defects.web.spi.RequirementsProvider], {

	_teamServerConnection: null,
	_externalProjectArea: null,

	constructor: function(id, name) {
	},

	// ---- DefectsManager implemention ------------------------------------------------------------------------------------

	getConnectionName: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.name : null;
	},

	getConnectionDetails: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.repositoryUrl : null;
	},

	getRepositoryUrl: function() {
		return this.getConnectionDetails();
	},

	getRepositoryDescription: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.description : null;
	},

	getProjectAreaName: function() {
		return this._teamServerConnection != null ? this._teamServerConnection.projectAreaName : null;
	},

	checkProviderStatus: function(resultCallback, cancelIfNotNotLoggedIn) {
		if (this._externalProjectArea == null) {
			resultCallback(new Status(Status.ERROR, "Not configured"));
		} else {
			var serviceCallback= function(status, response) {
				var resultStatus= status;
				try {
					if (status.isOK()) {
						if(!response.projectArea.initialized){
							resultStatus= new Status(Status.ERROR, "Project Area is not initialized");	
						}
					}
				} finally {
					resultCallback(resultStatus);
				}
			};

			RemoteTeamConcertClient.getProjectArea(this._externalProjectArea.getProjectAreaItemId(), this._externalProjectArea, serviceCallback, { requestLabel: "Check Requirement Provider Configuration", cancelIfNotLoggedIn: cancelIfNotNotLoggedIn });
		}
	},

	needsConfiguration: function() {
		return true;
	},

	isConfigured: function() {
		return this._externalProjectArea != null;
	},

	getConfigurationWizard: function() {
		return new TeamConcertWizard(this, "requirement");
	},

	initialize: function(runAfterInitialization) {
		TeamConcertConnectionClient.getDefaultTeamConcertConnection("requirement", dojo.hitch(this, function(status, result) {
			try {
				this._teamServerConnection= null;
				this._externalProjectArea= null;
				if (status.isOK()) {
					this._teamServerConnection= result;
	
					if (result != null) {
						this._externalProjectArea= new ExternalProjectAreaDescription(result.name, result.description, result.repositoryUrl, result.projectAreaItemId, result.projectAreaName);
					}
				} else {
					console.group("Failed to initialize TeamConcertDefectsProvider");
					status.log();
					console.groupEnd();
				}
			} finally {
				runAfterInitialization();
			}
		}));
	},


	setup: function(runAfterSetup, monitor) {
		monitor.beginTask("Initialize Requirements Provider...");

		// initialize again, to reload the just saved configuration including the Project Area UUID
		this.initialize(dojo.hitch(this, function() {

			var setupRequestOptions= { requestLabel: "Setup Requirements Provider" };

			RemoteTeamConcertClient.queryWorkItemAttributes({ identifier: "com.ibm.rqm.defect" }, this._externalProjectArea, dojo.hitch(this, function(status, result) {
				if (status.isOK()) {
					if(dojo.isArray(result.collection) && result.collection.length == 0) {
						var addAttributeParams= {
							identifier: "com.ibm.rqm.defect",
							type: "smallString"
						}

						RemoteTeamConcertClient.addWorkItemAttribute(addAttributeParams, this._externalProjectArea, dojo.hitch(this, function(status, result) {
							runAfterSetup(status);
						}), setupRequestOptions);
					} else {
						runAfterSetup(Status.OK_STATUS);
					}
				} else {
					runAfterSetup(status);
				}
				
			}), setupRequestOptions);
		}));
	},

	queryExistingRequirements: function(query, monitor, resultCallback) {
		monitor.beginTask("Search matching requirements...");
		var queryParams= {};
//		queryParams["type"]= "com.ibm.team.apt.workItemType.story";
		if (!isNaN(parseInt(query["id"]))) {
			queryParams["id"]= query["id"];
		} else if (dojo.isString(query["fullText"])) {
			queryParams["all"]= query["fullText"];
		} else {
			throw new Error("Invalid query descriptor");
		}

		RemoteTeamConcertClient.queryWorkItems(queryParams, this._externalProjectArea, dojo.hitch(this, function(status, response) {
			var resultStatus= status;
			var result= {};
			try {
				if (status.isOK() && response) {
					result= dojo.map(response.collection, function(entry) {
						var workItem= entry.workItem;
						return {
							label: workItem.id.value + ": " + workItem.summary.value,
							reference: entry._meta.locationHref,
							id: workItem.id.value,
							summary: workItem.summary.value
						}
					});
				}
			} catch (e) {
				resultStatus= new Status(Status.ERROR, "Failed to process query results", e);
			} finally {
				resultCallback(resultStatus, result);
			}
		}));
	},
	
	queryAssociatedRequirements: function(itemIdOrUri, monitor, runAfterQuery) {
		monitor.beginTask("Fetching associated requirements...");
		
		TeamConcertConnectionClient.getAssociatedWorkItems({ itemIdOrUri: itemIdOrUri }, dojo.hitch(this, function(status, response) {
			var resultStatus= status;
			var result= {};
			try {
				if (status.isOK() && response) {
					result= dojo.map(response, function(workItem) {
						return {
							reference: workItem.uri,
							id: workItem.id,
							summary: workItem.summary,
							label: workItem.summary
						}
					});
				}
			} catch (e) {
				resultStatus= new Status(Status.ERROR, "Failed to process query results", e);
			} finally {
				runAfterQuery(resultStatus, result);
			}
		}));
	},

	associateRequirement: function(linkDescription, monitor, runAfterAssociation) {
		monitor.beginTask("Associating Requirement...");
		
        var modificationParams = {
            "com.ibm.rqm.defect": "requirement@" + getCurrentProjectArea().itemId
        };
        
        var modificationLinkParams = {
        		"linkType": linkDescription.linkType,
        		"source": false,
        		"uri": linkDescription.itemIdOrUri,
        		"comment": linkDescription.comment
        };
        
        RemoteTeamConcertClient.modifyWorkItem(linkDescription.workItemId, modificationParams, this._externalProjectArea, dojo.hitch(this, function(status, response) {
        	
        }), { requestLabel: "Associate Requirement" });
        
		RemoteTeamConcertClient.modifyWorkItemLinks(linkDescription.workItemId, modificationLinkParams, this._externalProjectArea, dojo.hitch(this, function(status, response) {
			if (status.isOK()) {
				// now post a link to the RQM repository
				monitor.beginTask("Link Requirement to Test Case...");
				var draftLinkParams= {
					draftUri: linkDescription.workItemUri,
					itemIdOrUri: linkDescription.itemIdOrUri,
					linkType: "com.ibm.rqm.execution.linktype.eresultrelatedtodefect",
					id: linkDescription.workItemId,
					summary: linkDescription.label
				};

                TeamConcertConnectionClient.postDraftLink(draftLinkParams, function(status, response){
                    runAfterAssociation(status);
                });
			} else {
				runAfterAssociation(status);
			}
		}), { requestLabel: "Associate Requirement" });
	},

	// ---- implementation -------------------------------------------------------------------------------------------------	

	__sentinel: null // terminates this class definition
});

})();
 