/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.home.DashboardLeftNav");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardPage");
dojo.require("com.ibm.team.dashboard.web.ui.internal.LeftNav");

(function()
{
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var PlatformUI		= net.jazz.ajax.ui.PlatformUI;
	var ActionRegistry 	= null; // bind to ActionRegistry later

	dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.home.DashboardLeftNav", com.ibm.team.dashboard.web.ui.internal.LeftNav,
	{
		_populateProcess: function(ignoreMembership) {
			if (!this._processSection) {
				this._processSection = new com.ibm.rqm.planning.web.ui.internal.page.home.LeftNavProcess({
					leftNav: this,
					navInfo: this._navInfo,
					ignoreMembership: ignoreMembership
				});
				this._layout.addNavigationSection(this._processSection, Constants.LeftNav.PROCESS_AREAS_TITLE);
			} else {
				this._processSection.augment(this._navInfo);
			}
		}
	});

	dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.home.LeftNavCommands",
		[dijit._Widget, dijit._Templated], {
			
		templateString: "<ul dojoAttachPoint=\"_commands\" class=\"" + Constants.LeftNav.ROOT_CLASS + " no-bottom-margin\"></ul>",
		
		constructor: function(args) {
			this._personalSection = args.privateSection;
		},
		
		populate: function (args) {
			var li = document.createElement("li");
			li.className = Constants.LeftNav.COMMAND_CLASS;
			var link = document.createElement("a");
			link.setAttribute("href", "");
			link.innerHTML = Constants.LeftNav.NEW_DASHBOARD_COMMAND;
			this.connect(link, "onclick", this._handleNewPrivateDashboard);
			li.appendChild(link);
			this._commands.appendChild(li);
		},
	
		_handleNewPrivateDashboard: function(event) {
			event.preventDefault();
			
			var self = this;
			var userItemId = getAuthenticatedContributor().itemId;
			var serviceObj = {
				success: function(/* ResponseDTO */ responseDTO) {
					switch(responseDTO.status) {
						case "OK":
							var dashboardItem = responseDTO.data;
							self._personalSection.doAddDashboard(dashboardItem.getId(), dashboardItem.getTitle());
//							location.href= "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + Constants.DASHBOARD_ITEM_ID_PARAM + dashboardItem.getId();
							location.href = "#" + Constants.URL_PARAM_ACTION + "=com.ibm.rqm.planning.web.ui.viewDashboard"; 
							break;
						case "ERROR":
							alert(responseDTO.msg);
							break;
					}
				},
				failure: function(errorObj) {
					console.error(Constants.LeftNav.REMOTE_ERROR + errorObj.message);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.createDashboard(DashboardConstants.SCOPE_CONTRIBUTOR, userItemId, false, srh);
		}
			
	});

	dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavPersonal",
		[dijit._Widget, dijit._Templated], {
			
		templateString: "<ul dojoAttachPoint=\"_private\" class=\"" + Constants.LeftNav.ROOT_CLASS + "\">" +
		"<li dojoAttachPoint=\"_publicListItem\"><a dojoAttachPoint=\"_publicLink\"></a></li></ul>",
		
		constructor: function(args) {
			this._parent = args.leftNav;
			this._permissions = args.permissions;
		},
		
		populate:function() {
			this._publicLink.appendChild(document.createTextNode("My Profile"));
			this._publicLink.href = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + DashboardConstants.URL_PARAM_CONTRIBUTOR_ITEM_ID + "=" + getAuthenticatedContributor().itemId;
			dojo.addClass(this._publicListItem, Constants.LeftNav.COMMAND_CLASS);
			
			if (this._permissions["delete"]) {
				var deleteDiv = document.createElement("div");
				deleteDiv.className = Constants.LeftNav.DELETE_BUTTON_DIV_CLASS;
				deleteDiv.title = Constants.LeftNav.DELETE_DASHBOARD_TOOLTIP;
				this.connect(deleteDiv, "onclick", this._handleDeleteProfileDashboard);
				this._publicLink.appendChild(deleteDiv);
			}
			
			if (this._parent._navInfo) {
				this._createPersonalDashboardsList(this._parent._navInfo.personalDashboards);
			}
		},
		
		getListItem: function () {
			return this._publicListItem;
		},
		
		/*
		 * Add personal dashboards to the personal dashboard section.
		 */
		_createPersonalDashboardsList: function(/* DashboardDescriptorDTO[] */ descriptors) {
			if (!descriptors)
				return;
			for (var i=0;i<descriptors.length;i++) {
				this.doAddDashboard(descriptors[i].itemId, descriptors[i].title);
			}
		},
		
		doAddDashboard: function(/* string */ itemId, /* string */ title) {
			var li = document.createElement("li");
			li.id = itemId + Constants.LeftNav.LI_ID_SUFFIX;
			this._private.appendChild(li);
			var link = document.createElement("a");
			link.id = itemId + Constants.LeftNav.LINK_ID_SUFFIX;
			link.setAttribute("href", "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + Constants.DASHBOARD_ITEM_ID_PARAM + itemId);
			var span = document.createElement("span");
			span.appendChild(document.createTextNode(title));
			link.appendChild(span);
			
			if (this._permissions["delete"]) {
				var deleteDiv = document.createElement("div");
				deleteDiv.id = itemId;
				deleteDiv.className = Constants.LeftNav.DELETE_BUTTON_DIV_CLASS;
				deleteDiv.title = Constants.LeftNav.DELETE_DASHBOARD_TOOLTIP;
				this.connect(deleteDiv, "onclick", this._handleDeleteDashboard);
				link.appendChild(deleteDiv);
			}
			li.appendChild(link);
		},
		
		doRenameDashboard: function(/* DashboardItem */ dashboardItem) {
			var link = dojo.byId(dashboardItem.getId() + Constants.LeftNav.LINK_ID_SUFFIX);
			if (link)
				link.firstChild.innerHTML = dashboardItem.getTitle();
		},
		
		/* domNode */ findListNode: function(/* string */ itemId) {
			return dojo.byId(itemId + Constants.LeftNav.LI_ID_SUFFIX);
		},
		
		_handleDeleteDashboard: function(event) {
			event.preventDefault();
			event.cancelBubble = true;
			
			// are you sure?
			if (confirm(Constants.bind(Constants.LeftNav.DELETE_PRIVATE_DASHBOARD_CONFIRM,[event.target.parentNode.firstChild.firstChild.nodeValue]))) {
				this._parent._deleteDashboard(event.target.id);
			}
		},
		
		_handleDeleteProfileDashboard: function(event) {
			event.preventDefault();
			event.cancelBubble = true;
			
			var userItemId = getAuthenticatedContributor().itemId;
			var self = this;
			var serviceObj = {
				success: function(/* DashboardDescriptorDTO[] */ dashboardDescriptors) {
					if (dashboardDescriptors && dashboardDescriptors.length > 0) {
						// are you sure?
						if (confirm(Constants.LeftNav.DELETE_PUBLIC_DASHBOARD_CONFIRM)) {
							self._parent._deleteDashboard(dashboardDescriptors[0].itemId, userItemId, true);
						}
					}
					else 
						alert(Constants.LeftNav.DELETE_NO_PUBLIC_DASHBOARD);
				},
				failure: function(errorObj) {
					console.error(Constants.LeftNav.GET_PROCESS_DASHBOARD_ERROR + errorObj.message);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.getDashboards(DashboardConstants.SCOPE_CONTRIBUTOR, userItemId, true, srh);
		}
	});

	dojo.declare("com.ibm.rqm.planning.web.ui.internal.page.home.LeftNavProcess",
		[dijit._Widget, dijit._Templated], {
			
		templateString: "<ul dojoAttachPoint=\"_process\" class=\"" + Constants.LeftNav.ROOT_CLASS + "\">" +
		"</ul>",
		
		constructor: function(args) {
			this._parent = args.leftNav;
			this._ignoreMembership = args.ignoreMembership;
			if (args.ignoreMembership) {
				if (args.navInfo.projectAreas && args.navInfo.projectAreas.length > 0) 
					this._userHasProcessAreas = true;
			} else {
				for (var i = 0; i < args.navInfo.projectAreas.length; i++) {
					if (args.navInfo.projectAreas[i].member) {
						this._userHasProcessAreas = true;
						break;
					}
				}
			}
			if (this._userHasProcessAreas) {
				this._nonMemberListItems = new Array();
				this.onShowPartial = function() {
					this.setOtherAreasVisible(false);
				};
				this.onMaximize = function() {
					this.setOtherAreasVisible(true);
				};
			}
			this._createProjectAreaList(args.navInfo.projectAreas);
		},
		
		setOtherAreasVisible: function(/* boolean */ show) {
			this._isShowingOtherAreas = show;
			for (var i = 0; i < this._nonMemberListItems.length; i++) {
				if (show)
					dojo.removeClass(this._nonMemberListItems[i], "hidden");
				else
					dojo.addClass(this._nonMemberListItems[i], "hidden");
			}
		},
		
		augment: function(navInfo) {
			delete this._ignoreMembership;
			if (!this._userHasProcessAreas && dojo.hasClass(this._process.firstChild,Constants.LeftNav.LOADING_CLASS))
				this._process.removeChild(this._process.firstChild);
			this._currentInsertPosition = this._process.firstChild;
			this._createProjectAreaList(navInfo.projectAreas);
		},
		
		isLoaded: function() {
			return this._ignoreMembership ? false : true;
		},
		
		/*
		 * Create the project area tree in the left nav.
		 */
		_createProjectAreaList: function(projectAreas) {
			var ul = this._process;
			if (!projectAreas || projectAreas.length === 0) {
				var li = document.createElement("li");
				li.innerHTML += Constants.LeftNav.LOADING_PROJECT_AREAS_MESSAGE;
				dojo.addClass(li,Constants.LeftNav.LOADING_CLASS);
				ul.appendChild(li);
			} else {
				this._addTeamAreas(projectAreas, ul, 0, true);
			}
		},
		
		/*
		 * Add the team areas and project areas to the left-nav
		 */
		_addTeamAreas: function(teamAreaArr, parent, indent, isProject) {
			teamAreaArr.sort(function(a,b) {
				return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1;
			});
			for (var i=0; i<teamAreaArr.length; i++) {
				var teamArea = teamAreaArr[i];
				if (this._currentInsertPosition && this._currentInsertPosition.id == teamArea.itemId + Constants.LeftNav.LI_ID_SUFFIX) {
					this._currentInsertPosition = this._currentInsertPosition.nextSibling;
				} else {
					var li = document.createElement("li");
					if (this._userHasProcessAreas && !teamArea.member && !this._ignoreMembership) {
						if (!this._isShowingOtherAreas)
							dojo.addClass(li,"hidden");
						this._nonMemberListItems.push(li);
					}
					li.id = teamArea.itemId + Constants.LeftNav.LI_ID_SUFFIX;
					var link = document.createElement("a");
					var idType;
					if (isProject)
						idType = DashboardConstants.URL_PARAM_PROJECT_ITEM_ID;
					else
						idType = DashboardConstants.URL_PARAM_TEAM_ITEM_ID;
//					link.setAttribute("href", "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + idType + "=" + teamArea.itemId);
					link.setAttribute("href", "#" + Constants.URL_PARAM_ACTION + "=com.ibm.rqm.planning.web.ui.viewDashboard"); 
					link.appendChild(document.createTextNode(teamArea.name));
					link.style.marginLeft = (indent * 20) + "px";
					
					if (teamArea.permissions["delete"]) {
						var deleteDiv = document.createElement("div");
						deleteDiv.id = teamArea.itemId;
						deleteDiv.className = Constants.LeftNav.DELETE_BUTTON_DIV_CLASS;
						deleteDiv.title = Constants.LeftNav.DELETE_DASHBOARD_TOOLTIP;
						this.connect(deleteDiv, "onclick",this._handleDeleteDashboard);
						link.appendChild(deleteDiv);
					}
					
					li.appendChild(link);
					if (this._currentInsertPosition)
						parent.insertBefore(li, this._currentInsertPosition);
					else
						parent.appendChild(li);
				}
				if (teamArea.children) {
					this._addTeamAreas(teamArea.children, parent, indent + 1, false);
				}
			}
		},
		
		/* domNode */ findListNode: function(/* string */ itemId) {
			return dojo.byId(itemId + Constants.LeftNav.LI_ID_SUFFIX);
		},
		
		_handleDeleteDashboard: function(event) {
			event.preventDefault();
			event.cancelBubble = true;
			
			var self = this;
			var serviceObj = {
				success: function(/* DashboardDescriptorDTO[] */ dashboardDescriptors) {
					if (dashboardDescriptors && dashboardDescriptors.length > 0) {
						// are you sure?
						if (confirm(Constants.bind(Constants.LeftNav.DELETE_PROCESS_DASHBOARD_CONFIRM, [event.target.parentNode.firstChild.nodeValue]))) {
							self._parent._deleteDashboard(dashboardDescriptors[0].itemId, event.target.id);
						}
					}
					else 
						alert(Constants.bind(Constants.LeftNav.DELETE_NO_PROCESS_DASHBOARD, [event.target.parentNode.firstChild.nodeValue]));
				},
				failure: function(errorObj) {
					console.error(Constants.LeftNav.GET_PROCESS_DASHBOARD_ERROR + errorObj.message);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.getDashboards(DashboardConstants.SCOPE_TEAM_AREA, event.target.id, false, srh);
		}
	});

})();
