/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.FeedViews");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.FeedViewsContentArea");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.asq.common.web.client.internal.ProjectAliasClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");


(function(){
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
    var Constants 		= com.ibm.rqm.planning.web.client.internal.Constants;
    var ProjectAliasClient = com.ibm.asq.common.web.client.internal.ProjectAliasClient;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.FeedViews", 
                  com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor, {
        
        _feedViewsContentArea: null,
       
		constructor: function(){
            this.headerTitle = Constants.getMessage("labelAdministration");
        },
		
		postCreate: function(){
			// create the content area
            this._feedViewsContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.FeedViewsContentArea();
            
			// get the project area alias
			var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
	        var self = this;
	        var handler = {
	            success: function(alias) {
	            	self._feedViewsContentArea.setProjectAlias(alias);
	            },
	            error: function(e) {
	            	console.error(this.declaredClass + ": Error: " + e.message);
	            	self._feedViewsContentArea.setProjectAlias("Quality+Manager"); // the default project area
	            }
	        };
	        var responseHandler = new ServiceResponseHandler(handler, "success", "error");
	        ProjectAliasClient.getProjectAlias(responseHandler, {processAreaUUID : projectArea.itemId});
	        
			this.inherited("postCreate", arguments);
			
			// intro text
			var introContent = new FeedViews.IntroContentArea();
			this._contentArea.appendChild(introContent.domNode);
		},
		
        setNameContentPairs: function(){
        
	        this._nameContentPairs = [
	            {
	                "name": Constants.getMessage("labelExeResults"),
	                "content": this._feedViewsContentArea,
					"resourceType": "executionresult"
	            },
	            {
	                "name": Constants.getMessage("labelTestCases"),
	                "content": this._feedViewsContentArea,
					"resourceType": "testcase"
	            },
	            {
	                "name": Constants.getMessage("labelEWIs"),
	                "content": this._feedViewsContentArea,
					"resourceType": "executionworkitem"
	            },
				{
	                "name": Constants.getMessage("labelTestPlans"),
	                "content": this._feedViewsContentArea,
					"resourceType": "testplan"
	            },
	            {
	                "name": Constants.getMessage("labelTestScripts"),
	                "content": this._feedViewsContentArea,
					"resourceType": "testscript"
	            }
            ]
        },
        
        // overridden to handle switching the feed url
        displayContent: function(){
			this.inherited("displayContent", arguments);
            var name = this.getName();
            for (var i = 0; i < this._nameContentPairs.length; i++) {
                if (this._nameContentPairs[i].name == name) {
                	var resourceType = this._nameContentPairs[i].resourceType;
                	this._feedViewsContentArea.setFeedType(resourceType);
		            this._feedViewsContentArea.setName(name);
                    break;
                }
            }
        },
        
		__sentinel: null // terminates this class definition
        
    });
    
	dojo.declare("FeedViews.IntroContentArea", 
				 [com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractContentArea], {
				 
		// Overridden
		postCreate: function(){
			this.setName();
			this.setText();
		},
		
		// Overridden
		setName: function(){
		    this.name = Constants.getMessage("labelFeedViews");
		    dojox.data.dom.removeChildren(this._name);
		    var name = document.createTextNode(this.name);
		    this._name.appendChild(name);
		},
	        
		// Overridden
	     setText: function(){
	         this.text = Constants.getMessage("labelFeedViewsText");
	         dojox.data.dom.removeChildren(this._text);
	         var text = document.createTextNode(this.text);
	         this._text.appendChild(text);
	     },
	     
		// Overridden
        setContent: function(){
        },
	     
				 
		__sentinel: null // terminates this class definition
	});
    
})();
