/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.PreferencesEditor");

dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ActionBarContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactEditorContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.RelatedInformationContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.DateFormatContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.AdapterSelectionContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.WordWrapChoiceContentArea");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.QueryListsContentArea");

dojo.require("com.ibm.rqm.planning.web.client.internal.PreferenceClient");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){
	var ServiceResponseHandler			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var Constants 		= com.ibm.rqm.planning.web.client.internal.Constants;
	var AlertDialog 	= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	var PreferenceClient = com.ibm.rqm.planning.web.client.internal.PreferenceClient;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.PreferencesEditor", com.ibm.rqm.planning.web.ui.internal.view.administration.AbstractPropertyEditor, {
        /*
         * If at least one preference need logout/login to take effect, isHotSwitchable is flaged as false.
         * 
         * Need Logout/Login: Action Bar, Related Information
         * Nedd Not Logout/Login: TOC
         */
		isHotSwitchable: true,
		
		constructor: function(){
            this.headerTitle = Constants.getMessage("labelPreferences");
            this.defaultProperty = Constants.getMessage("labelActionBar");
        },
		
		postCreate: function(){
			this.inherited("postCreate", arguments);
			this._loadUserPreference();
		},
		
        setNameContentPairs: function(){
            var actionBarContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.ActionBarContentArea({onChange: dojo.hitch(this, "onChange")});
            var artifactEditorContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.ArtifactEditorContentArea({onChange: dojo.hitch(this, "onChange")});
			var relatedInformationContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.RelatedInformationContentArea({onChange: dojo.hitch(this, "onChange")});
            var dateFormatContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.DateFormatContentArea({onChange: dojo.hitch(this, "onChange")});
			var adapterSelectionContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.AdapterSelectionContentArea({onChange: dojo.hitch(this, "onChange")});
			var wordWrapContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.WordWrapChoiceContentArea({onChange: dojo.hitch(this, "onChange")});
            var queryListContentArea = new com.ibm.rqm.planning.web.ui.internal.view.administration.QueryListsContentArea({onChange: dojo.hitch(this, "onChange")});
			this._nameContentPairs = [{
                "name": Constants.getMessage("labelActionBar"),
                "content": actionBarContentArea
            },{
				"name":Constants.getMessage("labelArtifactEditorTOC"),
				"content": artifactEditorContentArea
			},{
                "name":Constants.getMessage("labelArtifactListQuerySettings"),
                "content": queryListContentArea
            },{
				"name":Constants.getMessage("labelRelatedInformation"),
				"content": relatedInformationContentArea
			},{
				"name":Constants.getMessage("labelDateFormat"),
				"content": dateFormatContentArea
			},{
				"name":Constants.getMessage("labelAdapterSelection"),
				"content": adapterSelectionContentArea
			},{
				"name":Constants.getMessage("labelEnableWordWrapPref"),
				"content": wordWrapContentArea
			}]
        },
        
        setButtonArea: function(){
			var inputLabel = document.createElement("label");
			var button = this.defaultButton = document.createElement("input");
			button.type = "button";
            button.value = Constants.getMessage("labelRestoreDefault");
			dojo.connect(button, "onclick", this, "_onRestoreDefault");
			inputLabel.appendChild(button);
            this._buttonArea.appendChild(inputLabel);
			
			var inputLabel = document.createElement("label");
			var button = this.okButton = document.createElement("input");
			button.type = "button";
			button.disabled = true;
            button.value = Constants.getMessage("labelOk");
			dojo.connect(button, "onclick", this, "_onOK");
			inputLabel.appendChild(button);
            this._buttonArea.appendChild(inputLabel);
			
			var inputLabel = document.createElement("label");
			var button = this.cancelButton = document.createElement("input");
			button.type = "button";
            button.value = Constants.getMessage("labelCancel");
			dojo.connect(button, "onclick", this, "_onCancel");
			inputLabel.appendChild(button);
            this._buttonArea.appendChild(inputLabel);

        },
		
		onCloseParentDialog: function(){
			//Empty function, connect it with the close() function of Parent SelectionDialog. Fire close() function when the Cancel button is clicked.
		},
		
		onChange: function(){
			var defaultButtonDiabled = true;
			var okButtonDiabled = true;
			for(var i = 0; i < this._nameContentPairs.length; i++){
				if(dojo.isFunction(this._nameContentPairs[i].content.matchingDefalutValues)){
					 defaultButtonDiabled = defaultButtonDiabled && this._nameContentPairs[i].content.matchingDefalutValues();
				}
				if(dojo.isFunction(this._nameContentPairs[i].content.matchingOldValues)){
					 okButtonDiabled = okButtonDiabled && this._nameContentPairs[i].content.matchingOldValues();
				}
			}
			this.defaultButton.disabled = defaultButtonDiabled;
			this.okButton.disabled = okButtonDiabled;
		},
		
		_onOK: function(){
			this.defaultButton.disabled = true;
			this.okButton.disabled = true;
			this.cancelButton.disabled = true;
			this._saveUserPreference();
        },
		
		_onCancel: function(){
			this.onCloseParentDialog();
		},
		
		_onRestoreDefault: function(){
			for(var i = 0; i < this._nameContentPairs.length; i++){
				if(this._nameContentPairs[i].content){
					this._nameContentPairs[i].content.restoreDefault();
				}
			}
			this.onChange();
		},
		
		_setUserPreference: function(userPreference){
			for(var i = 0; i < this._nameContentPairs.length; i++){
				this._nameContentPairs[i].content.setUserPreference(userPreference);
			}
			this.onChange();
		},
		
		_saveUserPreference: function(){
			var keyValues = [];
			for(var i = 0; i < this._nameContentPairs.length; i++){
				var keyValue = this._nameContentPairs[i].content.getValues();
				if(!this._nameContentPairs[i].content.matchingOldValues() && !this._nameContentPairs[i].content.isHotSwitchable()){
					this.isHotSwitchable = false;
				}
				for(var j = 0; j < keyValue.length; j++){
					keyValues.push(keyValue[j]);
				}
			}
			var responseHandler = new ServiceResponseHandler(this, "_savedSuccess", "_error");
            PreferenceClient.saveCurrentUserPreferences(keyValues, responseHandler);
		},
		
		_loadUserPreference: function(){
            PreferenceClient.getCurrentUserPreferences(this, this._setUserPreference);
		},
		
		_savedSuccess: function(){
			  this.onCloseParentDialog();
		},
		
		_error: function(e){
			console.debug(e);
		}
    });
})();
