/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.WorkItemClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.properties._workItem.QMWorkItemPageView");

(function(){ // module container
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var WorkItemClient = com.ibm.rqm.planning.web.client.internal.WorkItemClient;
    var EventTopics = com.ibm.team.workitem.web.ui.internal.page.EventTopics;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	
    var QMWorkItemPageView = com.ibm.rqm.planning.web.ui.internal.view.construction.properties._workItem.QMWorkItemPageView;
    var WORK_ITEM_EDITOR_ID = "__jazzWorkItemEditor";
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/QMWorkItemPane.html"),
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Called on creation
        ///////////////////////////////////////////////////////////////////////////////////				
        constructor: function() {
			this.isDialog = true;
            this._wiView = null;
            this._wiEditor = null;
            this._wiDialog = null;
            this._currentWorkItem = null;
        },
        
        postCreate: function(){
            if (this._wiView == null) {
                var self = this;
                this._wiView = new QMWorkItemPageView({
                    callback: dojo.hitch(self, "onClose")
                });
                this._wiView.setParent(this);
            }
            
            var msg = this.workItem != null ? (Constants.getMessage("msgOpeningWorkItem", this.workItem.id)) : this.title;
            var operation = this._wiView.getController().createOperation(msg, "newWorkItem");
            this._wiView.getController().startOperation(operation);
            var handlerObj = {
                success: function(workItem){
                    self.workItem = workItem;
                    self._wiView._multipaneContentWidget.removeCachedWidget(WORK_ITEM_EDITOR_ID, workItem.id);
                    self._wiView.showWorkItemEditor(workItem, "overview", false);                  
                    self._workItemEditorContainer.appendChild(self._wiView._multipaneContentWidget.domNode);
                    if (self.isDialog) {
						self._showAsDialog();
					}					
                    if (operation) {
                        self._wiView.getController().finishOperation(operation);
                    }
                    dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
                },
                error: function(error){
                    var msg = self.workItem != null ? Constants.getMessage("msgErrorOpeningWorkItem", this.workItem.id) : Constants.getMessage("msgErrorCreatingWorkItem");
                    self._wiView.getController().handleError(error, msg);
                    dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
                }
            };
            
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "error");
            var args = this.workItem != null ? {
                id: this.workItem.id
            } : {
                newWorkItem: "true"
            };
            if (args.newWorkItem && this.workItemTypeId) {
                args.typeId = this.workItemTypeId;
            }
            WorkItemClient.getWorkItemDTO(responseHandler, args);
        },
		
        _showAsDialog: function() {
            var dialogArgs = {
				primaryTitle: this.title,
                isClosable: true,
                showActions: false,
				zIndex : 100
            };                
            this._wiDialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            //this._wiDialog.setWidth(this.WIDTH);
            this._wiDialog.contentContainer.appendChild(this.domNode);
			
			//@Overridden from dijit._Widget - safety check that widgets being destroyed have a destroy method
			//only happen with WorkItem content
			this._wiDialog.destroyDescendants = function(){
	            dojo.forEach(this.getDescendants(), function(widget){ 
	                if (widget && dojo.isFunction(widget.destroy)) {
	                    widget.destroy();
	                }
	            });
	        }
                        
            if (this.cshelpId != null) {
                var cshelp = new CSHelp({contextID: this.cshelpId});
                cshelp.setButtonLabel(Constants.getMessage("labelWorkItemCSHelp", this.workItemTypeId));
                this._wiDialog.setSecondaryTitleNode(cshelp.domNode);
            }                   
            
            dojo.addClass(this._wiDialog.domNode, "qm-work-item-dialog");              
            dojo.connect(this._wiDialog, "close", this, this.doCallback);
			this._wiDialog.open();         
			this._wiView.setDialog(this._wiDialog);
			this._wiDialog.updatePosition();
        },		
        
        onClose: function(workItemId){
            this.workItemId = workItemId;
            if (this._wiDialog) {
                this._wiDialog.close();
            }
        },
        
        doCallback: function(){
            var self = this;
            var handlerObj = {
                success: function(workItemDTO){
                    self.callback.method.apply(self.callback.contextObj, [workItemDTO]);
                },
                error: function(){
                    console.error("Unable to retrieve WorkItemDTO for WorkItem: " + self.workItem.id);
                }
            }
            if (this.callback.returnAsDTO == true) {
				//deal with cancel empty workitem
				if (this.workItemId) {
					var responseHandler = new ServiceResponseHandler(handlerObj, "success", "error");
					WorkItemClient.getWorkItemDTO(responseHandler, {
						id: this.workItemId
					});
				}
				else {
					self.callback.method.apply(self.callback.contextObj, null);
				}
            } else {
                this.callback.method.apply(this.callback.contextObj, [this.workItemId]);
            }
        },
        
		handleCancelClick: function() {
			if (this._wiDialog) {
                this._wiDialog.close();
            }
		}	    		
    });
    
})();
