/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.common.RemoteExecutionScriptEditorSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.resourcebrowser.FileSelector");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.resourcebrowser.AdapterSelector");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.RemoteExecutionScriptClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function(){
var Constants 						= com.ibm.rqm.planning.web.client.internal.Constants;
var FileSelector 					= com.ibm.rqm.execution.web.ui.internal.view.resourcebrowser.FileSelector;
var AdapterSelector 				= com.ibm.rqm.execution.web.ui.internal.view.resourcebrowser.AdapterSelector;
var ScriptClient 					= com.ibm.rqm.execution.web.client.internal.execution.RemoteExecutionScriptClient;
var EditorUtil 						= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.common.RemoteExecutionScriptEditorSection",
			 null, 
{
	insideScriptEditor: true,
	browserUsage: false,
	adapterUsage: false,
	fileSeperater: "\\",//Windows by default
	
	postCreate: function(){
	},
	
	createBrowseSection: function(checkboxArea, browseArea, adapterArea){
		this.initialized = false;
		if(browseArea){
			dojox.data.dom.removeChildren(browseArea)
			var args= {
				singleSelect: this.insideScriptEditor, 
				scriptType: this.scriptType, 
				onClose: dojo.hitch(this, "getSelectedFiles"),
				parentEditor: this	
			}
			
			if(this.insideScriptEditor)
				args.script = this._script
			this.fileSelector = new FileSelector(args);
			browseArea.appendChild(this.fileSelector.domNode);
			this.browserUsage = true;
			if(this._script && !this._script.managedByAdapter){
				this.initialPath = this._script.sharePrefix ? this._script.sharePrefix : "";
				this.testPlanId = this.getScriptProperty(this._script.properties, "testPlanId");
				this.milestoneItemId = this.getScriptProperty(this._script.properties, "milestoneItemId");
				this.fileList = [];
				var resName = this._script.relativePath ? this._script.relativePath : "";
				var resURN = this.initialPath + resName;
				this.fileList.push({
					resourceName: resName,
					resourceURN: resURN
				})
			}	
			
		}
		if(adapterArea){
			dojox.data.dom.removeChildren(adapterArea)
			var args ={
					insideScriptEditor: this.insideScriptEditor, 
					scriptType: this.scriptType, 
					onClose: dojo.hitch(this, "getInfoFromAdapter"),
					parentEditor: this
					} 
			if(this.insideScriptEditor)
				args.script = this._script
			this.adapterSelector = new AdapterSelector(args);
			adapterArea.appendChild(this.adapterSelector.domNode);
			this.adapterUsage = true;
			if(this._script && this._script.managedByAdapter){
				var path = this._script.fullPath ? this._script.fullPath.substring(0, this._script.fullPath.lastIndexOf(this.getFileSeperater())) : "";
				var name = this._script.fullPath ? this._script.fullPath.substring(this._script.fullPath.lastIndexOf(this.getFileSeperater())+1, this._script.fullPath.length): "";
				var adapterId = this._script.adapterId ? this._script.adapterId : null;
				this.info ={
					selectedAdapter: this._script.adapterId,
					scriptPath: path,
					selectedScript: {name: name}	
					}
			}	
		}
		this.browseArea = browseArea;
		this.adapterArea = adapterArea;
		if( browseArea && adapterArea){
			if(!checkboxArea){
				console.error(Constants.getMessage("errMissSelectionArea"));
				return;
			}
			dojox.data.dom.removeChildren(checkboxArea)
			checkboxArea.appendChild(this._getSelectorArea());
			if(this._script && this._script.managedByAdapter == true){
				this._adapterRadioClicked()
			}else{
				this._browseRadioClicked()
			}
		}
		this.initialized = true;
	},

	getScriptProperty: function(props, name){
		var value = ""
		var properties = props && props.length > 0 ? props : [] 
		for(var i=0;i<properties.length;i++){
			if(properties[i].propertyName == name){
				value= properties[i].propertyValue;
				break;
			}
		}
		return value;
	},
	
	setScriptType: function(scriptType){
		if(this.adapterSelector)
			this.adapterSelector.setScriptType(scriptType);
		if(this.fileSelector)
			this.fileSelector.setScriptType(scriptType);	
	},
	
	getSelectedFiles: function(){
		var resources = this.fileSelector.getResources();
		this.fileList = resources.fileList;
		this.initialPath = resources.initialPath;
		this.testPlanId = resources.testPlanId;
		this.milestoneItemId = resources.milestoneItemId;
		if(this.insideScriptEditor)
			this.parentEditor.setDirty(true);
		if(dojo.isFunction(this.setSelectedFiles))
			this.setSelectedFiles(resources, this.initialPath, this.testPlanId, this.milestoneItemId);
	},
	
	getInfoFromAdapter: function(){
		this.info = this.adapterSelector.getAttributes();
		if(this.insideScriptEditor)
			this.parentEditor.setDirty(true);
		if(dojo.isFunction(this.setAdapterInfo))
			this.setAdapterInfo(this.info);
	},
	
	_getSelectorArea: function(){
		var browseRadio = document.createElement("input");
		browseRadio.type = 'radio';
		browseRadio.checked = true;
		browseRadio.name = "remote-section";
		browseRadio.id = this.id+"browseRadio";
		browseRadio.onclick = dojo.hitch(this, "_browseRadioClicked");
		
		var adapterRadio = document.createElement("input");
		adapterRadio.type = 'radio';
		adapterRadio.checked = false;
		adapterRadio.name = "remote-section";
		adapterRadio.id = this.id+"adapterRadio";
		adapterRadio.onclick = dojo.hitch(this, "_adapterRadioClicked");
		
		var browseLabel = document.createElement('label');
		browseLabel.setAttribute("for", "browseRadio");
		browseLabel.innerHTML = Constants.getMessage("labelBrowseForResource");
		var adapterLabel = document.createElement('label');
		adapterLabel.setAttribute("for", "adapterRadio");
		adapterLabel.innerHTML = Constants.getMessage("labelUseAdapter");
		
		dojo.style(browseLabel, "fontFamily", "Arial");
		dojo.style(browseLabel, "fontSize", "9pt");
		dojo.style(adapterLabel, "fontFamily", "Arial");
		dojo.style(adapterLabel, "fontSize", "9pt");
		
		var browseHelpText = document.createElement('label');
		browseHelpText.setAttribute("for", "browseRadio");
		browseHelpText.innerHTML = Constants.getMessage("labelBrowseForResourceHelpTxta") +"<br/>"+Constants.getMessage("labelBrowseForResourceHelpTxtb");
		dojo.style(browseHelpText, "fontFamily", "Arial");
		dojo.style(browseHelpText, "fontSize", "8pt");
		dojo.style(browseHelpText, "color", "#736D75");
		
		var adapterHelpText = document.createElement('label');
		adapterHelpText.setAttribute("for", "browseRadio");
		adapterHelpText.innerHTML = Constants.getMessage("labelUseAdapterHelpTxta")+"<br/>"+Constants.getMessage("labelUseAdapterHelpTxtb");
		dojo.style(adapterHelpText, "fontFamily", "Arial");
		dojo.style(adapterHelpText, "fontSize", "8pt");
		dojo.style(adapterHelpText, "color", "#736D75");
		
		var table = document.createElement('table');
		dojo.style(table, "marginLeft", "20px")
        dojo.style(table, "marginTop", "5px")

		//Browse
		var tr = table.insertRow( table.rows.length );
		var tdBrRad = tr.insertCell( tr.cells.length );
		tdBrRad.appendChild(browseRadio)
		var tdBrlab = tr.insertCell( tr.cells.length );
		tdBrlab.appendChild(browseLabel);
		var browseCSHelp = document.createElement('span');
		this._setCSHelpId(browseCSHelp, "rqm_browse_help.html", Constants.getMessage("labelBrowseHelp"));
		tdBrlab.appendChild(browseCSHelp);
		
		var trbrH = table.insertRow( table.rows.length );
		var tdbrH1 = trbrH.insertCell( trbrH.cells.length );
		var tdbrH2 = trbrH.insertCell( trbrH.cells.length );
		tdbrH2.appendChild(browseHelpText)
		
		var trEmpty = table.insertRow( table.rows.length );
		var tdEmpty = trEmpty.insertCell( trEmpty.cells.length );
		tdEmpty.colSpan = 2;
		var emptyLab = document.createElement('label');
		emptyLab.innerHTML = "&nbsp;";
		tdEmpty.appendChild(emptyLab);
		
		//Adapter
		var tr1 = table.insertRow( table.rows.length );
		var tdAdpRad = tr1.insertCell( tr1.cells.length );
		tdAdpRad.appendChild(adapterRadio)
		var tdAdplab = tr1.insertCell( tr1.cells.length );
		tdAdplab.appendChild(adapterLabel)
		var adpCSHelp = document.createElement('span');
		this._setCSHelpId(adpCSHelp, "rqm_adapter_help.html", Constants.getMessage("labelAdapterHelp"));
		tdAdplab.appendChild(adpCSHelp);
		
		var tradpH = table.insertRow( table.rows.length );
		var tdadpH1 = tradpH.insertCell( tradpH.cells.length );
		var tdadpH2 = tradpH.insertCell( tradpH.cells.length );
		tdadpH2.appendChild(adapterHelpText)
		
		var div = document.createElement('div');
		var headerSpan = document.createElement('span');
		headerSpan.innerHTML = this.getSelectorHeading();
		div.appendChild(headerSpan);
		div.appendChild(table);	
		return div;
	},
	
	getSelectorHeading: function(){
		return Constants.getMessage("labelCreateScriptSelector");
	},
	
	setFileSeperater: function(seperater){
		this.fileSeperater = seperater;
	},
	
	getFileSeperater: function(){
		if(this._script && this.insideScriptEditor){
			var seperater = this.getScriptProperty(this._script.properties, "fileSeperater");
			if(seperater != "" && this.initialized == false)
				this.fileSeperater = seperater;
		}
		return this.fileSeperater;
	},
	
	_setCSHelpId: function(node, helpId, helpLabel){
        if (node) {
	        dojox.data.dom.removeChildren(node);
            var args = {
                'contextID': helpId,
                'csHelpEngine': null,
                'csHelpProviderId': null
            };
            var cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
			cshelp.setButtonLabel(helpLabel);
            node.appendChild(cshelp.domNode);
        }
    },
	
	_browseRadioClicked: function(){
		if(dojo.byId(this.id+"browseRadio"))
			dojo.byId(this.id+"browseRadio").checked=true;
		if(dojo.byId(this.id+"adapterRadio"))
			dojo.byId(this.id+"adapterRadio").checked=false;
		
		this.adapterArea.style.display= "none";
		this.browseArea.style.display= "inline";
		this.browserUsage = true;
		this.adapterUsage = false;
		if(this.insideScriptEditor && this.initialized){
			this.parentEditor.setDirty(true);
		}	
		this.updateBrowseRadioListener();
	},
	
	_adapterRadioClicked: function(){
		if(dojo.byId(this.id+"browseRadio"))
			dojo.byId(this.id+"browseRadio").checked=false;
		if(dojo.byId(this.id+"adapterRadio"))
			dojo.byId(this.id+"adapterRadio").checked=true;
		this.adapterArea.style.display= "inline";
		this.browseArea.style.display= "none";
		
		this.browserUsage = false;
		this.adapterUsage = true;
		if(this.insideScriptEditor && this.initialized){
			this.parentEditor.setDirty(true);
		}
		this.updateAdapterRadioClicked();
	},
	
	updateAdapterRadioClicked: function() {
	},
	
	updateBrowseRadioListener: function() {
	},
	
	getAttributes: function(){
		var args = {};
		args.managedByAdapter = this.adapterUsage ? true : false;
		args.sharePrefix = this.initialPath ? this.initialPath : ""; 
		var relPath = this.initialPath && this.fileList && this.fileList[0] && this.fileList[0].resourceURN ? this.fileList[0].resourceURN.substring(this.initialPath.length, this.fileList[0].resourceURN.length) : "";
		args.relativePath = relPath;
		this.absolutePath = this.fileList && this.fileList[0] && this.fileList[0].resourceURN ? this.fileList[0].resourceURN : "";
		args.adapterId = this.info && this.info.selectedAdapter ? this.info.selectedAdapter.id : "";
		args.fullPath = this.info && this.info.scriptPath && this.info.selectedScript ? this.info.scriptPath+this.getFileSeperater()+this.info.selectedScript.name: "";
		
		return args;
	},
	
	validate: function(){
		return true;
	},
	
	supportStandAlone: function(){
		return true;
	},
	
	_isStandAlone: function(){
		if(!this.supportStandAlone()){
			return false;
		}
		if(this.browserUsage && (!this.initialPath || !this.fileList || this.fileList.length <=0 ))
			return true;
		else if(this.adapterUsage && (!this.info || !this.info.selectedAdapter || !this.info.scriptPath || !this.info.selectedScript))
			return true;
		else		
			return false;
	},
	
	setValidationError: function(/*Optional*/message){//To be called from the script extensions if validation error happens
		this.validationErrorShown = true;
		if(message){
			this.parentEditor.setSaving(false);
			this.parentEditor.getHeader().setProgressEnd();
			this.parentEditor.setHeaderStatus(Constants.VALIDATION_ERROR, Constants.HEADER_STATUS_ERROR, message);
			}
		else	
			this.parentEditor._saveError(Constants.getMessage("msgValidationFailedExcution"));
	},
	
	onSave: function(responseHandler, scriptObj, lastSaved){
	//Modify the script object here, call own service for this.
		this.validationErrorShown = false;
		if (!this.validate()) {
			if(!this.validationErrorShown)//If adapter does not call setValidation Error gracefully show a failure message.
				this.parentEditor._saveError(Constants.getMessage("msgValidationFailedExcution"));
			return;
		}
		var standAlone = false;
		if(this.insideScriptEditor && this._isStandAlone()){//Do only if inside the script editor, since there you can type.
			standAlone = true;
		}

		dojo.mixin(scriptObj, this.getAttributes());

		var properties = this.getProperties();
		//Save the file seperater with the script for later reference
		properties.push({propertyName: "fileSeperater", propertyValue: this.getFileSeperater(), propertyType: "string"});
		if(!standAlone && this.browserUsage)
		{
			if(this.testPlanId && this.testPlanId != "global")
				properties.push({propertyType: "testplan", propertyName:"testPlanId", propertyValue:this.testPlanId});
			if(this.milestoneItemId && this.milestoneItemId != "")
				properties.push({propertyType: "string", propertyName:"milestoneItemId", propertyValue:this.milestoneItemId});	
			properties.push({propertyType: "absolutePath", propertyName:"absolutePath", propertyValue:this.absolutePath});
		}	
		scriptObj.properties = properties;
		
		if(lastSaved)
			EditorUtil.setLastSaved(lastSaved, Constants.STATUS_SAVING);	
		ScriptClient.saveScript(responseHandler, scriptObj);
   },
	
	_error: function(errorObj){
		console.error(Constants.getMessage("errInRemoteExecutionScriptEditorSection"));
	}
});
})();
