/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.ServerHealth");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.ServerHealth");

dojo.require("dojox.data.dom");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");


(function(){
	
	var ServiceResponseHandler	= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
	var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
	var Constants				= com.ibm.rqm.planning.web.client.internal.Constants;

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.ServerHealth", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/ServerHealth.html"),

        SERVICE_URI: "com.ibm.rqm.planning.common.service.rest.IServerInfoRestService",

        //labels
        title: Constants.getMessage("labelServerHealth"),
        labelBundleVersions: Constants.getMessage("labelBundleVersions"),
        labelBundleName: Constants.getMessage("labelBundleName"),
        labelBundleVersion: Constants.getMessage("labelBundleVersion"),

        //dialog options
    	isDialog: true,
		width: "600px",
		
		            
        constructor: function(args) {            
            this._versions = [];
		},
		
		postCreate: function() {
            if (this.isDialog) {
                this._showAsDialog();
            }       
						
			var responseHandler = new ServiceResponseHandler(this, "initialize", "_error");			
			this._getAllBundleVersions(responseHandler);			
		},
		
		initialize: function(versions) {
			this._setVersions(versions);
			this._showVersions();
            if (this._dialog) {
                this._dialog.updatePosition();
            }
		},
		
		_showAsDialog: function() {
            var dialogArgs = {
                primaryTitle: this.title,
                isClosable: true,
                showActions: false
            };                
            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
            this._dialog.setWidth(this.width);
            this._dialog.contentContainer.appendChild(this.domNode);
            this._dialog.open();			
		},
		
		_setVersions: function(rawVersions) {
			rawVersions = rawVersions ? rawVersions : [];
			this._versions = [];		
            for (var i = 0; i < rawVersions.length; i++) {
				var tokens = rawVersions[i].split(":");
				var versionObj = {
                    name: tokens[0],
                    id: tokens[1]
                }
				this._versions.push(versionObj);				
			}
			this._versions.sort(function(val1, val2) {
	            if (val1.name < val2.name) {
	                return -1;
	            } else if (val1.name > val2.name) {
	                return 1;
	            }
	            return 0;
			});
        },
		
		_showVersions: function() {
			dojox.data.dom.removeChildren(this._versionsTbody);  
			for (var i = 0; i < this._versions.length; i++) {
				this._versionsTbody.appendChild(this._createVersionRow(this._versions[i]));
			}
		},		
		
		_createVersionRow: function(version){
			var row = document.createElement("tr");
			var name = document.createElement("td");
			name.innerHTML = version.name;		   
			var id = document.createElement("td");
            id.innerHTML = version.id;
                        
            row.appendChild(name);
			row.appendChild(id);
            return row;
        },
		
		
		//services methods
        _invokeService: function(responseHandler, method, appArgs) {      
            var serviceRequest= new ServiceRequest(this.SERVICE_URI, method, appArgs);
            TeamServerClient.invokeService(serviceRequest, responseHandler);
        },
		
        _getVersions: function(serviceResponseHandler) {
            this._invokeService(serviceResponseHandler, "getVersions", { });
        },		
        
        _getAllBundleVersions: function(serviceResponseHandler) {
            this._invokeService(serviceResponseHandler, "getAllBundleVersions", { });
        },		
		
		_error: function(e){
            console.error(this.declaredClass + ": error: " + (e ? e.message : ""));          
        }
	
    });
	
})();
