/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.SideActionPanel");

dojo.require("dijit.form.Button");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");

(function() {
     
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.SideActionPanel",
[net.jazz.ajax.ui.SideNavigationLayout],
{
    
    /**
     * Set the click handler and icon class for the primary action Dojo button
     */ 
    setPrimaryAction: function(handler, iconClass /* optional */, label) {
        var arg = {
            onClick: handler,
            label: label
        };
        
        if (iconClass) {
            arg["iconClass"] = iconClass;
        }
        // create and add the dojo button to the primary action section
        this._primaryAction = new dijit.form.Button(arg, this.getPrimaryActionSection());
    },
    
    /**
     * Return the primary action section.
     */
    getPrimaryActionSection: function() {
        return this._primaryActionContainer;
    },
    
    /**
     * Return the secondary action section.
     */
    getSecondaryActionsSection: function() {
        return this._secondaryActionsContainer;
    },
    
    /**
     * Return the views section.
     */
    getViewsSection: function() {
        return this._viewsContainer;
    },

    /**
     * Return the recently viewed section.
     */
    getRecentlyViewedSection: function() {
        return this._recentContainer;
    },

    /**
     * Return the favorites section.
     */
    getFavoritesSection: function() {
        return this._favoritesContainer;
    },

    _primaryActionContainer: null,
    _secondaryActionsContainer: null,
    _viewsContainer: null,
    _recentContainer: null,
    _favoritesContainer: null,
    _primaryAction: null,
    
    /**
     * Override the Jazz SideNavigationLayout to have specific action group sections.
     */
    postCreate: function() {
        com.ibm.rqm.planning.web.ui.internal.view.common.SideActionPanel.superclass.postCreate.apply(this, arguments);

        // add the primary action div
        this._primaryActionContainer = document.createElement("div");
        dojo.addClass(this._primaryActionContainer, "side-action-panel-div");
        this.addNavigationSection(this._primaryActionContainer);
        this._firstNavSectionLoaded = false;  // tweak not to show the section separator 

        // add the secondary action div
        this._secondaryActionsContainer = document.createElement("div");
        dojo.addClass(this._secondaryActionsContainer, "side-action-panel-div");
        this.addNavigationSection(this._secondaryActionsContainer);
        this._firstNavSectionLoaded = false;  // tweak not to show the section separator 

        // add the views section div
        this._viewsContainer = document.createElement("div");
        dojo.addClass(this._viewsContainer, "side-action-panel-div");
        this.addNavigationSection(this._viewsContainer);
        this._firstNavSectionLoaded = false;  // tweak not to show the section separator 

        // add the recently viewed section div
        this._recentContainer = document.createElement("div");
        dojo.addClass(this._recentContainer, "side-action-panel-div");
        this.addNavigationSection(this._recentContainer);
        this._firstNavSectionLoaded = false;  // tweak not to show the section separator 

        // add the favorites section div
        this._favoritesContainer = document.createElement("div");
        dojo.addClass(this._favoritesContainer, "side-action-panel-div");
        this.addNavigationSection(this._favoritesContainer);
        this._firstNavSectionLoaded = false;  // tweak not to show the section separator 
    },
    
    /**
     * The way Jazz animates the sliding to hide the panel is setting the position of
     * _leftNav from 0 to -180.  It will cover up the document body border if the panel
     * is not flush to the left.  Override to hide the show the _leftNav before and
     * after the animation to not blocking the border.
     */
    _toggleLeftNav: function(e) {
        var action = this._currentLeftNavState === "open" ? "hide" : "close";
        if (action === "hide") {
            com.ibm.rqm.planning.web.ui.internal.view.common.SideActionPanel.superclass._toggleLeftNav.apply(this, arguments);
            this._leftNav.style.visibility = "hidden";
        } else {
            this._leftNav.style.visibility = "VISIBLE";
            com.ibm.rqm.planning.web.ui.internal.view.common.SideActionPanel.superclass._toggleLeftNav.apply(this, arguments);
        }
    }
    
});

})();
