/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.ManualScriptEditorSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.ExecutionScriptElementList");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.DatapoolInTestScriptClient");

(function(){ // module container
   var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
   var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
   var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
   var DatapoolClient = com.ibm.rqm.planning.web.client.internal.construction.DatapoolInTestScriptClient;
   var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
   
   dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.ManualScriptEditorSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
      templateString: "<div dojoAttachPoint=\"_manualStepsSection\" class=\"com-ibm-rqm-planning-web-ui-internal-view-construction-extension-ManualScriptEditorSection\"></div>",
      
      constructor: function(args){
         this._script = dojo.clone(args.script);
      },
      
      postCreate: function(){
      },
      
      initialize: function(){
         var steps = this._script ? this._script.elements : [];
         dojox.data.dom.removeChildren(this._manualStepsSection);
         this._stepsEditor = new com.ibm.rqm.planning.web.ui.internal.view.construction.ExecutionScriptElementList();
         this._manualStepsSection.appendChild(this._stepsEditor.domNode);
         this._stepsEditor.initialize({
            parentController: this.parentController,
            steps: steps,
            parentCallBack: this.parentEditor
         });
      },
      
      //@Abstract: Override to contribute attributes to the parent's save
      getAttributes: function(){
         // not supported by DocumentRestService yet 
         return {
            elements: this.currentElements
         };
      },
      
	  validate: function(){
	  	this.currentElements = [];
		this._stepsEditor.setGlobalSaveInProgress();
         if (!this._stepsEditor.validateElements()) {
            return false;
         }
		this.currentElements = this._stepsEditor.getElements();
		return true;
	  },
	  
	  isEmpty: function(){
	  	return this.currentElements ? this.currentElements.length == 0 : false;
	  },
	  
      //@Overriden from AbstractArtifactEditorSection
      getActions: function(){
         return [];
      },
      
      //@Overridden from AbstractArtifactEditorSection
      saveFinished: function(savedItem){
         this._stepsEditor.updateSteps(savedItem.elements);
		 this._dealWithDatapool(savedItem.elements);
	  },
	  
	  _dealWithDatapool: function(elements) {
		 if(elements && elements.length) {
		 	var stepPair = new Array();
		 	for(var i =0; i<elements.length; i++) {
				var varArray = this._parseVariableId(elements[i].content);
				if (varArray && varArray.length > 0) {
					stepPair.push({
						stepItemId: elements[i].itemId,
						variables: varArray
					});
				}
			}
			if (stepPair.length > 0) {
				var param = dojo.toJson(stepPair);
				var callbackObj = {
					success: function(results){
					},
					failure: function(e){
						console.error("Datapool createLink failed");
					}
				}
				var responseHandler = new ServiceResponseHandler(callbackObj, "success", "failure");
				DatapoolClient.postDatapoolVariableLink2(responseHandler, {
					jsonString: param
				});
			}
		 }
      },
	  
	  _parseVariableId: function(content) {
	  	var newContent = unescape(content);
		var contentDiv = document.createElement('div');
		contentDiv.innerHTML = newContent;
//	  	var children = contentDiv.childNodes;
		var children = contentDiv.getElementsByTagName("BUTTON");//desDiv.childNodes;
		var newArray = [];
		for(var i=0; i<(children ? children.length : 0); i++) {
			newArray.push(children[i]);
		}
			
		var idArray = null;
		if(newArray && newArray.length) {
			idArray = new Array();
			for(var i=0; i<newArray.length; i++) {
				if(newArray[i].name == "variable") {
					var itemId = newArray[i].id;
					idArray.push(itemId);
				} 
			}
		}
		return idArray;
	  },
	  
	  _onDatapoolChange: function(){
		this._stepsEditor._removeAllVariables();
	  }
   });
})();
