//SuiteExec
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/*
 * ExecutionView for Suite
 * Author: vasrivas
 */
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.suite.SuiteExecution");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.ExecutionConsole");
dojo.require("com.ibm.rqm.execution.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function(){ 
	
	var ServiceResponseHandler			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var ERGClient 						= com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultGroupClient;
	var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
	var Utility 						= com.ibm.rqm.execution.web.ui.internal.util.Utility;
	var GlobalEvents 					= com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	var ERGMapper						= com.ibm.rqm.execution.web.client.internal.mapper.ExecutionResultGroupMapper;
	
	dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.suite.SuiteExecution", [com.ibm.rqm.execution.web.ui.internal.view.common.ExecutionConsole], {
		
		buttonAdded: false,
		consoleNode: false,
		titleSet: false,
		ergName: "",
		initialized: false,
		fetchingRequestUpdates: false,
		requestMap: {},
		
		initialize: function(){
		this.myURI = window.location.hash;
			if(!this.initialized){
				this._createExecutionTable();
				this.initialized = true;
				console.debug("creating timer");
				this._timer = setInterval(dojo.hitch(this, this._fetchRequestUpdates), this.getTimeout());
			}
			if(!this._timer){
				console.debug("creating timer from initialize");
				this._timer = setInterval(dojo.hitch(this, this._fetchRequestUpdates), this.getTimeout());
			}	
			this.fetchingRequestUpdates = true;//Since the whole ERG will be fetched first time, 
												   //let it fetch first and then call updates		
			var args = {}
			if(this.groupId)
				args.id = this.groupId;
			else if(this.itemId)
				args.itemId = this.itemId;	
			var responseHandler = new ERGMapper(this, "_setERGroup", "_error");
			ERGClient.getExecutionResultGroupDTOs(responseHandler, args);
		},
		
		_setERGroup: function(resultGroup){
			if(resultGroup && resultGroup.length > 0){
				if(resultGroup[0] && resultGroup[0].requests){
					this.ergId = resultGroup[0].id;
					this.ergName = resultGroup[0].name;
					this.requests = resultGroup[0].requests;
					this._executionTable.setERGId(this.ergId);
					this._createTable(this.requests);
				}	
			}else{
				this._createTable([]);
			}
			
			if(this._executionTable && !this.buttonAdded){
				this.buttonAdded = true;
				this.showResultButton = this._executionTable.viewHeaderNode.addRightViewAction(Constants.getMessage("labelCloseAndShowResult"), dojo.hitch(this, "onShowResult"), null, null);
				this.showResultButton.disabled = true;
			}
			if(this._executionTable && !this.titleSet){
				this.titleSet = true;
				this._executionTable.viewHeaderNode.setTitle(Constants.getMessage("labelTestSuiteExecution", this.ergName));
				this._executionTable.viewHeaderNode.setCSHelpId("rqm_suite_execution_overview.html");
				this._executionTable.viewHeaderNode.setIconClass("ewi-icon-48");
			}
			
			//After creation check for completion
			if(resultGroup && resultGroup[0] && resultGroup[0].requests){
				this.resultGroup = resultGroup[0];
				this._checkForCompletion(this.resultGroup);
			}
			
			// update browser title
			this._updateBrowserTitle();
			this.fetchingRequestUpdates = false;
		},
		
		_checkForCompletion: function(resultGroup){
			var completion = true;
			var elements = resultGroup.elements ? resultGroup.elements : [];
			var requests = resultGroup.requests ? resultGroup.requests : []; 
			if(resultGroup.sequentialExecution && (elements.length > 0) && (elements.length != requests.length)){
				completion = false;
			}else{
				this.requestMap = {};
				for(var i=0;i<requests.length;i++){
					if(requests[i].state && requests[i].state.id == "com.ibm.rqm.executionframework.common.requeststate.cancelled" && resultGroup.sequentialExecution){
						completion = true;//Seq exec will not proceed after this, enable result nav.
						break;
					}
					else if(parseInt(requests[i].progress) < 100){
						completion = false;
					}
					
				}
			}
			this.requestMap = {};
			for(var i=0;i<requests.length;i++){
				if(requests[i].progress == 100){
					this.requestMap[requests[i].id] = false;
				}else{
					this.requestMap[requests[i].id] = true;
				}
			}
			if(completion || this.isDone){
                // don't stop polling the data yet. Fetch it one more
                // time. The data could be incomplete at this point.
                // (i.e., the modified attribute could be invalid) 
                if (this.isDone) {
                    this.resultGroupId = this.ergId;
                    if (this._timer) {
                        clearInterval(this._timer);
                        delete this._timer;
                        console.debug("deleting timer");
                    }
                    this.showResultButton.disabled = false;
                } else {
                    // set the flag to true. The next round we'll
                    // stop the polling.
                    this.isDone = true;
                }
			}else{
				this.showResultButton.disabled = true;
			}
			this._updateTopSection();	
		},
		
		getTimeout: function(){
			return 10000;
		},
		
		onShowResult: function(){
			parent.location = Utility.showSuiteResult({id: this.resultGroupId});
			setTimeout(dojo.hitch(this, "publishClosure"), 1500);
		},
		
		publishClosure: function(){
			var args = {}
			if(this.groupId)
				args.groupId = this.groupId;
			if(this.itemId)
				args.itemId = this.itemId;
					
			var uri = Utility.runSuite(args);
			dojo.publish(GlobalEvents.CloseTab, [uri]);
		},
		
		_updateTopSection: function(){
			if(this.topSectionCreated){
				if(this.resultGroup.sequentialExecution){
					var requests = this.resultGroup.requests ? this.resultGroup.requests : [];
					var totalElements = this.resultGroup.elements.length;
					var completed = 0; var running = 0; var notrun = 0;
					for(var i=0;i<requests.length;i++){
						if(parseInt(requests[i].progress) < 100){
							running ++;
						}else if(parseInt(requests[i].progress) == 100){
							completed ++;
						}
					}
					
					var comp = document.getElementById(this.id+"_completedValue");
					if(comp)	comp.innerHTML = completed + " " + Constants.getMessage("labelTestCases");
					var run = document.getElementById(this.id+"_runningValue");
					if(run)		run.innerHTML = running +  " " + Constants.getMessage("labelTestCase");
					var notRun = document.getElementById(this.id+"_notRunValue");
					if(notRun)	notRun.innerHTML = (totalElements - (completed + running)) + " " + Constants.getMessage("labelTestCases");
				}
			}else{
				var table = document.createElement('table');
				table.width = "300px;"
				var tbody = document.createElement('tbody');
				table.appendChild(tbody);
				this._executionTable.viewHeaderNode.getSummaryNode().appendChild(table);
				
				var tr = document.createElement('tr');
				tbody.appendChild(tr);
				//Execution Mode
				var tdtype = document.createElement('td');
				tr.appendChild(tdtype);
				var typeLabel = document.createElement('label');
				typeLabel.innerHTML = Constants.getMessage("labelExecutionMode");
				tdtype.appendChild(typeLabel);
				var tdSeq = document.createElement('td');
				tr.appendChild(tdSeq);
				var seqLabel = document.createElement('label');
				seqLabel.innerHTML = this.resultGroup.sequentialExecution ? Constants.getMessage("labelSequential") : Constants.getMessage("labelParallel") ;
				tdSeq.appendChild(seqLabel);

				if(this.resultGroup.sequentialExecution){
					var tr1 = document.createElement('tr');
					tbody.appendChild(tr1);
					//Completed
					var tdcompLabel = document.createElement('td');
					tr1.appendChild(tdcompLabel);
					var compLabel = document.createElement('label');
					compLabel.innerHTML = Constants.getMessage("labelCompleted");
					tdcompLabel.appendChild(compLabel);
					var tdcompVal = document.createElement('td');
					tr1.appendChild(tdcompVal);
					var compVal = document.createElement('label');
					compVal.innerHTML = "";
					tdcompVal.appendChild(compVal);
					compVal.id = this.id + "_completedValue";
					
					var tr2 = document.createElement('tr');
					tbody.appendChild(tr2);
					//Running
					var tdrunningLabel = document.createElement('td');
					tr2.appendChild(tdrunningLabel);
					var runningLabel = document.createElement('label');
					runningLabel.innerHTML = Constants.getMessage("labelRunning");;
					tdrunningLabel.appendChild(runningLabel);
					var tdrunningVal = document.createElement('td');
					tr2.appendChild(tdrunningVal);
					var runningVal = document.createElement('label');
					runningVal.innerHTML = "";
					tdrunningVal.appendChild(runningVal);
					runningVal.id = this.id + "_runningValue";
					
					var tr3 = document.createElement('tr');
					tbody.appendChild(tr3);
					//Not run
					var tdnotRunLabel = document.createElement('td');
					tr3.appendChild(tdnotRunLabel);
					var notRunLabel = document.createElement('label');
					notRunLabel.innerHTML = Constants.getMessage("labelNotRun");;
					tdnotRunLabel.appendChild(notRunLabel);
					var tdnotRunVal = document.createElement('td');
					tr3.appendChild(tdnotRunVal);
					var notRunVal = document.createElement('label');
					notRunVal.innerHTML = "";
					tdnotRunVal.appendChild(notRunVal);
					notRunVal.id = this.id + "_notRunValue";
				}
				this.topSectionCreated = true;
				this._updateTopSection();
			}
		},
		
	// handler on artifacts changed
    onArtifactChanged: function(changeType, artifactType, artifactArray) {
        // only care about requests
        if (artifactType == Constants.TYPE_REQUEST) {
            if (changeType == GlobalEvents.ArtifactsAdded) {
                // add requests to the table
                if (!this.requests) {
                    this.requests = artifactArray;
                } else {
                    this.requests = this.requests.concat(artifactArray);
                }
				if(this._executionTable)
                	this._executionTable.setContent(this.requests);
            } else {
                // iterate through the array of artifacts
                for (var i = 0; i < artifactArray.length; i++) {
                    // iterate through the array of requests in the table
                    for (var j = 0; j < this.requests.length; j++) {
                        // handle the matching requests
                        if (artifactArray[i].id == this.requests[j].id) {
                            if (changeType == GlobalEvents.ArtifactsUpdated) {
                                // swap requests on update
                                this.requests.splice(j, 1, artifactArray[i]);
                                if(this._executionTable)
									this._executionTable.setContent(this.requests);
                            } else if (changeType == GlobalEvents.ArtifactsDeleted) {
                                // remove requests from table on delete
                                this.requests.splice(j, 1);
                                if(this._executionTable)
									this._executionTable.setContent(this.requests);
                            }
                            break;
                        }
                    }
                }
            }
			//additionally check if we can navigate to results
			this.resultGroup.requests = this.requests;
			this._checkForCompletion(this.resultGroup)
        }
    },
	
	//Override isDirty to clear Timer on tab close
	isDirty: function(){
		if(this._timer){
			clearInterval(this._timer);
			delete this._timer ;
			console.debug("deleting timer on tab close");
		}
		return false;
	},

	_error: function(errorObj){
		if(this._timer){
				clearInterval(this._timer);
				delete this._timer ;
				console.debug("deleting timer");
			}
		console.error("Error in Suite Execution "+ errorObj.message + " ");
	},
	
	_fetchRequestUpdates: function(){
		if(this.fetchingRequestUpdates || this.myURI != window.location.hash)
				return;
		this.fetchingRequestUpdates = true;
		var args = {}
		args.requestJsonString= dojo.toJson(this.requestMap);
		args.ergItemId = this.resultGroup.itemId;
		
		var responseHandler = new ERGMapper(this, "_receiveUpdatedRequests", "_error");
		ERGClient.getUpdatedRequests(responseHandler, args);
	},
	
	_receiveUpdatedRequests: function(requests){
		requests = requests ? requests : [];
		for(var i=0;i<requests.length;i++){
			var foundRequest = false;
			for(j=0;j<this.requests.length;j++){
				if(this.requests[j].id == requests[i].id){
					//Found updated request
					foundRequest = true;
					this.requests[j].progress = requests[i].progress;
					this.requests[j].state = requests[i].state;
					this.requests[j].modified = requests[i].modified;
					this.requests[j].propertiesMap = requests[i].propertiesMap;
				}
			}
			if(!foundRequest)
				this.requests = this.requests.concat([requests[i]]);
		}
		this.resultGroup.requests = this.requests;
		this._createTable(this.requests);
		this._checkForCompletion(this.resultGroup);
		this.fetchingRequestUpdates = false;
	},
	
	_updateBrowserTitle: function(){
		if (this.page){
			var name="";
			
		 	if (this.ergName) {
				name = this.ergName;
			}
			
			this.page.browserTitle =  Constants.getMessage("labelTestSuiteExecution", Constants.getMessage("labelColen") + " " + name);
			net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this.page.browserTitle);
			net.jazz.ajax.ui.PlatformUI.getWorkbench().browserTitle = this.page.browserTitle;	
		}
		
	}        
});
    
})();
