/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionHistoryTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("ExecutionHistoryTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {},
	tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){                
        if (callback == null && callbackData != null && dojo.isFunction(callbackData))
            callbackData();		
		if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null)
                callback(callbackData);
            else
                callback();
        } 
    }
});
dojo.declare("ExecutionHistoryTableView.Sorter",
			com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter,
{
	_sortComparator: function(obj1, obj2){
		if(this.sortCol >-1 && this.sortCol < 2){
			if(this.sortDir == TableConstants.SORT_DIR_ASCEND)
				return dojo.date.compare(obj1.longTime,obj2.longTime);
			else
				return dojo.date.compare(obj2.longTime,obj1.longTime);
		}
		else 
			return this.inherited("_sortComparator",arguments);
	},
	_groupComparator: function(obj1, obj2){
		var val1 = this.viewer.getContentProvider().getCellDisplayData(obj1, this.groupCol);
		var val2 = this.viewer.getContentProvider().getCellDisplayData(obj2, this.groupCol);
		if(val1 == val2) {
		// there's a secondary sort and it's not the same as the group
		// sort column, sort by it.
		if(this.sortCol > -1 && this.groupCol != this.sortCol)
			return this._sortComparator(obj1, obj2);

		return 0;
		}
		if(this.groupCol < 2)
			return dojo.date.compare(obj1.longTime,obj2.longTime);
		else
			return this.inherited("_groupComparator",arguments);
	}
});

dojo.declare("ExecutionHistoryTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    getNumColumns: function(content){
        return 7;
    },
    getColumnHeading: function(content, index){
        var headings = [
            Constants.getMessage("labelStartDate"),
			Constants.getMessage("labelStartTime"),
            Constants.getMessage("labelTotalRunTime"),
            Constants.getMessage("labelScriptName"),
			Constants.getMessage("labelScriptType"),
			Constants.getMessage("labelOverallVerdict"),
			Constants.getMessage("labelExeResult")
        ];
        return headings[index];
    },
    getColumnHeadingImage: function(content, atColumnIndex){
		return null;
    },
    getColumnHeadingAlignment: function(atColumnIndex){
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    getIsColumnGroupable: function(content, atColumnIndex){
        return (atColumnIndex <= 5 && atColumnIndex != 2);     
    },
    getIsColumnSortable: function(content, atColumnIndex){
        return (atColumnIndex <= 5);   
    },
    getColumnWidth: function(content, atColumnIndex){
        switch(atColumnIndex) {
            case 0: 
                return 10;
            case 1: 
                return 10;
            case 2: 
                return 15;
            case 3: 
                return 35;
            case 4: 
                return 10;
            case 5: 
                return 10;					
            case 6: 
                return 10;
        }
        return 0;
    },
    getContentObjects: function(content){
        return content;
    },
    getRowDataId: function(rowData){
        return "_table_row_for_execution_result_" + rowData.id;
    },
    getCellDisplayType: function(rowData, atColumnIndex){
        if(atColumnIndex == 6)
			return TableConstants.RENDER_TYPE_IMAGE;
		else if(atColumnIndex ==3)
			return TableConstants.RENDER_TYPE_LINK;	               
        return TableConstants.RENDER_TYPE_TEXT;
    },
    getCellAlignment: function(rowData, atColumnIndex){
        if(atColumnIndex == 6)
            return TableConstants.ALIGN_CENTER;
        return TableConstants.ALIGN_LEFT;
    },
	getCellDisplayData: function(rowData, atColumnIndex){
        var val = "";
        switch(atColumnIndex) {
            case 0: 
                val = rowData.date;
                break;
            case 1: 
                val = rowData.time;
                break;
            case 2: 
                val = rowData.duration;
                break;
            case 3: 
                val = rowData.script? rowData.script.name:"";
                break;          
            case 4: 
                val = rowData.script? (rowData.script.scriptType ? Constants.getExtensionDisplayMessage(rowData.script.scriptType.name) : Constants.getMessage("labelUnknown")): "";
                break;  
            case 5: 
                val = rowData.state;
                break;  
            case 6:
                val = "";
				break;
        }
        return val;
    },
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex){  
        var val = null;
		if(atColumnIndex == 3 && rowData.script) {
            val = {};
            val.href = Utility.getTestScriptURI(rowData.script.id, Utility.CONSTRUCTION_PERSPECTIVE);
			val.tooltip = Constants.getMessage("labelClickToOpenTestScript");				
        }
        if (atColumnIndex == 6) {
			val = {};
			val.image = rowData.icon;
			val.href = rowData.link;
        }		  		  		  
        return val;
    }
});

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionHistoryTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>',
    widgetsInTemplate: true,
    constructor: function() {
	 	this.results = null;
	 },
	postCreate: function(){
        this.inherited("postCreate", arguments);
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-EXE-HISTORY-TABLEVIEWER"                          
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);
        var content = this.results ? this.results : [];
        tableViewer.setContentPageSize(20);
        tableViewer.setContent(content);
        tableViewer.setContentProvider(new ExecutionHistoryTableView.TableProvider());
        tableViewer.setTableListener(new ExecutionHistoryTableView.TableListener());
		tableViewer.setSorter(new ExecutionHistoryTableView.Sorter());
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.queryName));
        tableViewer.show();
		this.tableViewer = tableViewer;
    },
	refresh:function(content){
		this.results = content ? content:[];
		this.tableViewer.setContent(this.results);
		this.tableViewer.refresh();
	},
	addAction:function(actionId,actionLabel,callbackData,iconClass,callback){
		return this.tableViewer.addAction(actionId,actionLabel,callbackData,iconClass,callback);
	}	
});

})();
