/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading: com.ibm.rqm.process.web.ui.internal.view.timebox.MainTimebox");
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.timebox.MainTimebox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.process.web.ui.internal.view.timebox.TimeboxTable");
dojo.require("com.ibm.rqm.process.web.ui.internal.view.timebox.KeyDateTable");
dojo.require("com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient");

(function(){
    var ImageCache = com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
    var TimeboxClient = com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient;
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.timebox.MainTimebox", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.process.web", "ui/internal/view/templates/timebox/MainTimebox.html"),
        
		labelTestScheduleTable: Constants.getMessage("labelTestScheduleTable"),
		labelKeyDateTable: Constants.getMessage("labelKeyDateTable"),
		
        constructor: function(){
			this.isEditable = null;
			this._itemId = null;
            this.parentController = null;
            this.parentSection = null;
           	this._timeboxWidget = null;
			this._editable = false;
        },
        
        postCreate: function(){
        
        },
        
        initialize: function(args){
            this._itemId = args.itemId;
			this.parentController = args.parentController;
            this.parentSection = args.parentSection ? args.parentSection : null;
			this.isEditable = args.isEditable;
			this._initialContent();
        },
        
        _initialContent: function(){
            dojox.data.dom.removeChildren(this._iterationTable);
			if (this._itemId != null) {
                var params = {
                    testPlanId: this._itemId
                }
                var responseHandler = new ServiceResponseHandler(this, "_addIterationTable", "_error");
                TimeboxClient.getTestPhasesDTO(responseHandler, params);
            }
            else {
                this._addIterationTable(null);
            }
        },
        
		getTimeboxWidget: function() {
           return this._timeboxWidget;
        },
		
		getKeydateWidget: function() {
           return this._keydateWidget;
        },
		
		_addIterationTable: function(appResults){
			//initialize time box table
            this._iterationResults = appResults;		
			this._timeboxWidget = new com.ibm.rqm.process.web.ui.internal.view.timebox.TimeboxTable();
			var self = this;
            this._timeboxWidget.initialize({
				itemId: this._itemId,
                timeboxResults: this._iterationResults,
				parentController: this.parentController,
				parentSection: this.parentSection != null ? this.parentSection : null,
				printable: this.parentSection != null ? this.parentSection.isPrintView() : false,
				tableType: "phase"
            });
            this._timeboxWidget.changeRowEditable(this.isEditable);
            dojox.data.dom.removeChildren(this._iterationTable);
            this._iterationTable.appendChild(this._timeboxWidget.domNode);
			
			dojox.data.dom.removeChildren(this._keydateTable);
			if (this._itemId != null) {
                var params = {
                    testPlanId: this._itemId
                }
                var responseHandler = new ServiceResponseHandler(this, "_addKeydateTable", "_error");
                TimeboxClient.getKeyDatesDTO(responseHandler, params);
            }
            else {
                this._addKeydateTable(null);
            }
		},
		
		_addKeydateTable: function(appResults) {	
			//initialize key date table
			this._keydateResults = appResults;//this._handleResult(appResults, 1);		
			this._keydateWidget = new com.ibm.rqm.process.web.ui.internal.view.timebox.KeyDateTable();
			var self = this;
            this._keydateWidget.initialize({
				itemId: this._itemId,
                timeboxResults: this._keydateResults,
				parentController: this.parentController,
				parentSection: this.parentSection != null ? this.parentSection : null,
				printable: this.parentSection != null ? this.parentSection.isPrintView() : false
            });
            this._keydateWidget.changeRowEditable(this.isEditable);
            dojox.data.dom.removeChildren(this._keydateTable);
            this._keydateTable.appendChild(this._keydateWidget.domNode);
        },
		
        _handleCheckClick: function() {
			this._editable = !this._editable;
			this._timeboxWidget.changeRowCheckbox(this._editable);
			this._keydateWidget.changeRowCheckbox(this._editable);
		},
		
        _error: function(errorObj){
            alert(errorObj.message);
        }
    });
    
})();
