/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.viewlets.DefectReportViewlet");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.viewlets.BaseReportViewlet");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var Viewlet 					= com.ibm.team.dashboard.web.ui.Viewlet;
var BaseReportViewlet 			= com.ibm.rqm.planning.web.ui.internal.view.viewlets.BaseReportViewlet;

var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;	
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
	
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.DefectReportViewlet",	
	BaseReportViewlet,
{
	/*
	 * Static Variable Container 
	 */
	_static: function() {},
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {
		// Call super
		this.inherited("init", arguments);

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		// Cascading parameters map
		this._groupParametersExecution = null;
		this._singleParametersGrouping = null;
		this._singleParametersShowKeyDates = null;
		this._singleParametersLabel = null;
		
		// Parameter URI names
		this._parameterURINames = new Array();
		this._setParameterURINames();
		
		// Get the current report preference value
		this._reportUUID = this.getPreference(this._staticCommon._PREFERENCE_ID_REPORT);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {
		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}
		
		// Preferences
		this._static._PREFERENCE_ID_PLAN = "plan";
		this._static._PREFERENCE_ID_ITERATION = "iteration";		// CML: what is this used for??
		this._static._PREFERENCE_ID_GROUPING = "grouping";
		this._static._PREFERENCE_ID_SHOWKEYDATES = "showKeyDates";
		this._static._PREFERENCE_ID_DRAWDATAPOINTLABELS = "isDrawDataPointLabels";
		
		// Cascading parameter groups
		this._static._PARAMETER_GROUP_NAME = "PlanIterationTestcase";
		this._static._PARAMETER_SINGLE_DRAWDATAPOINTLABELS = "isDrawDataPointLabels";
		
		// Single parameters
		this._static._PARAMETER_SINGLE_GROUPING = "grouping";
		this._static._PARAMETER_SINGLE_SHOWKEYDATES = "showKeyDates";
		
		// Work Item Micro Reports
		this._staticCommon._EXECUTION_TREND = "defects.arrival.resolution";
		this._staticCommon._microReportIDMap[this._staticCommon._EXECUTION_TREND] = "execution.DefectArrival";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_setParameterURINames: function() {
		// Test Plan
		this._parameterURINames.push("plan");
		// Iteration
		this._parameterURINames.push("iteration");		// CML: what is this used for??
		// Test case
		this._parameterURINames.push("grouping");
		// Machine
		this._parameterURINames.push("showKeyDates");
		// isDrawDataPointLabels
		this._parameterURINames.push("isDrawDataPointLabels");
		// Zoom
		this._parameterURINames.push(this._staticCommon._PARAMETER_NAME_ZOOM);
		// Width
		this._parameterURINames.push("width");
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_resetCascadingParameters: function() {
		this._groupParametersExecution = null;
		this._singleParametersGrouping = null;
		this._singleParametersShowKeyDates = null;
		this._singleParametersLabel = null;
	},

	/*
 	 * Public Method
 	 * 
 	 * Called to update viewlet content (e.g. Settings changed)
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
	    this._updateLoadingMessage(Constants.getMessage("labelRefreshingReport"));
		
		this._getParameters();
		
		//console.debug("Refresh parameter and values:");
		//this._debugParameters(this._parameterURINames, this.getParameterValues());
		
		this._getReport(
			this._parameterURINames,
			this.getParameterValues());
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */
	_getReport: function(parameterNames, parameterValues) {
		// debug parameters
		//console.debug("Original parameter and values:");
		//this._debugParameters(parameterNames, parameterValues);
				
		// if the parameterValues contains null value, remove all the parameters except for the 'zoom'
		var _report_parameterNames = new Array();
		var _report_parameterValues = new Array();
	
		_report_parameterNames = this._deleteNullNames(parameterValues,parameterNames);
		_report_parameterValues = this._deleteNullValues(parameterValues);

		
		// debug parameters
		// console.debug("New parameter and values:");
		// this._debugParameters(_report_parameterNames, _report_parameterValues);
		
		// Generate the hash key for the report request
		var hashKey = this._hashArray(parameterValues);
		// Check to see if the report is cached
		//var cachedHtml = this._getReportFromCache(hashKey);
		//var isReportCached = (cachedHtml != null);
		// Display whether the report is cached or live
		//this._displayCacheStatus(isReportCached);
		// Display cached report
		//if (isReportCached) {
			// Add the report to the display area
		//	this._addReportToViewlet(cachedHtml);
			// Service invocation unnecessary
		//	this.getSite().doneRefresh();
		//	return;
		//}
		// Generate the click-through URL
		this._generateClickThroughURL(
			this._reportUUID, 
			_report_parameterNames, 
			_report_parameterValues
		);
		
		// Update loading message
		this._updateLoadingMessage(Constants.getMessage("labelLoading"));
		// Show the loading message
		this._showLoadingArea();
		
		// Remember the zoom state at the time of this request
		var previousZoomState = this.isZoomed();
		
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(html) {
				self._processReportHTML(html, hashKey, previousZoomState);
				self.getSite().doneRefresh();
			},
			// Service failure
			failure: function(errorObject) {
				self._logError(errorObject);
				self._displayErrorMessage(Constants.getMessage("labelReportLoadingFailed"), errorObject);
				self.getSite().doneRefresh();
			}
		};
	
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		this._callRESTGetRenderedReport(
			handler, 
			this._reportUUID,
			_report_parameterNames, 
			_report_parameterValues);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @param Object currentSettings
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceOptions: function(preferenceId, currentSettings) {
		// Note:  Incomplete preference not handled because it is not dynamic
		// Get preference values
		if (preferenceId == this._static._PREFERENCE_ID_PLAN) {
			// Test Plan
			if (this._groupParametersExecution == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._groupParametersExecution == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionTestPlan();
			
		} else if (preferenceId == this._static._PREFERENCE_ID_ITERATION) {
			// Iteration
			if (this._groupParametersExecution == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._groupParametersExecution == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionIteration(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_GROUPING) {
			// Test Case
			if (this._singleParametersGrouping == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersGrouping == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionGrouping(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_SHOWKEYDATES) {
			// Machine
			if (this._singleParametersShowKeyDates == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersShowKeyDates == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionShowKeyDates(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_DRAWDATAPOINTLABELS) {
			// Machine
			if (this._singleParametersLabel == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersLabel == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionLabel(currentSettings);
			
		} else {
			var errorObject = new Error("Unrecognized preference: " + preferenceId);
			this._logError(errorObject);
			this._displayErrorMessage(Constants.getMessage("labelPreferenceLoadingFailed"), errorObject);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Array
 	 * @overrides super
 	 */		
	settingChanging: function(preferenceId) {
		var depedencies = new Array();
		if (preferenceId == this._static._PREFERENCE_ID_PLAN) {
			// Test Plan -> Iteration
			depedencies.push(this._static._PREFERENCE_ID_ITERATION);
		}
		return depedencies;
	},

    /*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_getParameters: function() {
		// Update loading message
		this._updateLoadingMessage(Constants.getMessage("labelLoading"));
		// Show the loading message
		this._showLoadingArea();
		// Asynchronous call
		this._getGroupParameters(
			this._static._PARAMETER_GROUP_NAME,
			this._callBackSuccessGetGroupParameters,
			this._callBackErrorGetGroupParameters);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array
 	 */	
	getParameterValues: function() {		
		var parameterValues = new Array();
		
		// Test Plan
		var testPlan = this.getPreference(this._static._PREFERENCE_ID_PLAN);		
		testPlan = this._normalizeParameterValue(testPlan);
		parameterValues.push(testPlan);
		
		// Iteration
		var iteration = this.getPreference(this._static._PREFERENCE_ID_ITERATION);		
		iteration = this._normalizeParameterValue(iteration);
		parameterValues.push(iteration);
		
		// Grouping
		var grouping = this.getPreference(this._static._PREFERENCE_ID_GROUPING);		
		grouping = this._normalizeParameterValue(grouping);
		parameterValues.push(grouping);
		
		// Show Key Dates
		var showKeyDates = this.getPreference(this._static._PREFERENCE_ID_SHOWKEYDATES);		
		showKeyDates = this._normalizeParameterValue(showKeyDates);
		parameterValues.push(showKeyDates);
		
		// is Draw Data Point Labels
		var pointLables = this.getPreference(this._static._PARAMETER_SINGLE_DRAWDATAPOINTLABELS);		
		pointLables = this._normalizeParameterValue(pointLables);
		parameterValues.push(pointLables);
		
		// Zoom
		parameterValues.push(this._quote(this.isZoomed()));
		
		// width
		parameterValues.push(this.container.domNode.clientWidth);
	
		return parameterValues;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return boolean
 	 * @overrides super
 	 */		
	_isSettingsEqual: function(oldSettings, newSettings) {

		if (oldSettings.preferences.plan != newSettings.preferences.plan) {
			return false;
		} else if (oldSettings.preferences.iteration != newSettings.preferences.iteration) {
			return false;
		} else if (oldSettings.preferences.grouping != newSettings.preferences.grouping) {
			return false;
		} else if (oldSettings.preferences.showKeyDates != newSettings.preferences.showKeyDates) {
			return false;
		}else if (oldSettings.preferences.isDrawDataPointLabels != newSettings.preferences.isDrawDataPointLabels) {
			return false;
		}
		
		return true;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 */		
	_getPreferenceOptionTestPlan: function() {
		// Get parameters		
		var executionResult = this._groupParametersExecution;		
		var testPlan = null;
		try {
			// Get test plan from the dto
			testPlan = executionResult.parameters[0];
		} catch(errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(testPlan, null);		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		return result;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionIteration: function(currentSettings) {
		// Get parameters
		var executionResult = this._groupParametersExecution;
		var iteration;
		try {
			// Get the current preference value for the team area
			// Cascades: Test Plan -> Iteration
			var testPlanValue = currentSettings.preferences.plan;
			var index = executionResult.parameters[0].values.indexOf(testPlanValue);
			var allIterations = executionResult.parameters[0].children;			
			iteration = allIterations[index];
		} catch(errorObject) {
			return this._getPreferenceFailureList();
		}

		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(iteration, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		return result;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionGrouping: function(currentSettings) {
		// Get parameters
		var grouping = this._singleParametersGrouping;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(grouping, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionShowKeyDates: function(currentSettings) {
		// Get parameters
		var showKeyDates = this._singleParametersShowKeyDates;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(showKeyDates, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionLabel: function(currentSettings) {
		// Get parameters
		var isDrawLable = this._singleParametersLabel;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(isDrawLable, null);
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			// this._addAllOption(result.options);
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},	
		
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @param com.ibm.team.reports.rest:ReportParameterGroupDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetGroupParameters: function(self, groupName, dto) {
		try {
			// Create a map out of the DTO
			var map = self._processDTOGroupParameters(dto);
			// Store global
			self._groupParametersExecution = dto;
		} catch (e) {
			self._groupParametersExecution = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
		// Debug
		//console.debug(dto);
		
		self._getSingleParameters(
			self._static._PARAMETER_SINGLE_GROUPING,
			self._callBackSuccessGetSingleParameterGrouping,
			self._callBackErrorGetSingleParameters);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameterGrouping: function(self, parameterName, dto) {
		try {
			// Create a custom object out of the DTO
			var result = self._processDTOSingleParameters(dto);
		
			// Store global
			self._singleParametersGrouping = result;
		} catch (e) {
			self._singleParametersGrouping = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
		self._getSingleParameters(
			self._static._PARAMETER_SINGLE_SHOWKEYDATES,
			self._callBackSuccessGetSingleParameterShowKeyDates,
			self._callBackErrorGetSingleParameters);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameterPointLables: function(self, parameterName, dto) {
		
		try {
			// Create a custom object out of the DTO
			var result = self._processDTOSingleParameters(dto);
		
			// Store global
			self._singleParametersLabel = result;
		} catch (e) {
			self._singleParametersLabel = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
		
		// Show the display area
		self._showDisplayArea();
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameterShowKeyDates: function(self, parameterName, dto) {
		
		try {
			// Create a custom object out of the DTO
			var result = self._processDTOSingleParameters(dto);
			// Store global
			self._singleParametersShowKeyDates = result;
		} catch (e) {
			self._singleParametersShowKeyDates = self._staticCommon._DTO_PARAMETERS_FAILURE;
		}
		
		
		self._getSingleParameters(
			self._static._PARAMETER_SINGLE_DRAWDATAPOINTLABELS,
			self._callBackSuccessGetSingleParameterPointLables,
			self._callBackErrorGetSingleParameters);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @return void
 	 */
	_callBackErrorGetSingleParameters: function(self, parameterName) {
		// Set error
		self.parameterName = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeParameterValue: function(value) {
		if (value == this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
			return this._quote("");
		}
		return this._quote(value);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @return void
 	 */
	_callBackErrorGetGroupParameters: function(self, groupName) {
		// Set error
		self._groupParametersExecution = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},
	
	/*
 	 * Private Method
 	 * Delete the parameter value array contains the null value
 	 * 
 	 * @param Array parameterValues
 	 * @return newParameterValus
 	 */	
	_deleteNullValues: function(parameterValues) {
	
		var newParaValues = new Array();
		for( var j in parameterValues ) {
			if( parameterValues[j] != "'null'" && parameterValues[j] != "''" && parameterValues[j] != "'{Null}'") {
				newParaValues.push(parameterValues[j]);
			} 
		}
		return newParaValues;
	},
	
	/*
 	 * Private Method
 	 * Delete the parameter name array that parameter value is null
 	 * 
 	 * @param Array parameterValues
 	 * @param Array parameterNames
 	 * @return newParameterNames
 	 */	
	_deleteNullNames: function(parameterValues, parameterNames) {
	
		var newParaNames = new Array();
		for( var j in parameterValues ) {
			if( parameterValues[j] != "'null'" && parameterValues[j] != "''" && parameterValues[j] != "'{Null}'") {
				newParaNames.push(parameterNames[j]);
			} 
		}
		return newParaNames;
	},

	
	/*
 	 * Private Method
 	 * 
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */
	_debugParameters: function(parameterNames, parameterValues) {
		for( var i in parameterNames ) {
			console.debug("Parameter Name" + i + ":" + parameterNames[i]);
		}
		
		for( var j in parameterValues ) {
			console.debug("Parameter Value" + j + ":" + parameterValues[j]);
		}
	}			
		
});

})();