/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.RTLMMachineContentProvider");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;
	
	var LAYOUT_TEMPLATE = {
		'machinesection': {
				'row1': ['ipAddress', 'Manufacturer'],
				'row2': ['memory', 'cpu'],
				'row3': ['owner', 'status']
		},
		
		'ossection': {
			'row1': ['operatingsystem']
		},
		
		'softwaresection': {
            'row1': ['SoftwareInstall']
			
		}
	};
		
	var OTHER_CATEGORY = "other";

	dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.RTLMMachineContentProvider", 
	    [com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider],
	{		
		_displayedAttrs: ['Machine.operatingsystem'],
		_templateName: null,
		_customElementProvider: null,
		
		constructor: function(/*com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm*/formLayout, 
								formListener, 
								/*string*/templateName,
								customElementProvider ) {
			com.ibm.rational.test.lm.web.ui.internal.views.machines.RTLMMachineContentProvider.superclass.constructor(formLayout, formListener);
			this._templateName = templateName ? templateName : LAYOUT_TEMPLATE;
			this.setShowPartTypeTitle(false);
			this._customElementProvider = customElementProvider;
		},
	
		getFormInfo: function(/* the content object */content) 
	    {   
	        var sectionKeys = [];
			for(var i in this._templateName) {
				var section = {};
				section.id = i;
				section.path = new Array(i); 
				//Do not translate this.  It should only be translated where it will be displayed for users.
				//Otherwise, it will be needed for a key.
				section.attributeName = i;//RtlmUiUtil.getMessage(i);
				sectionKeys.push(section);
			}

	        var info = {'nColumns': 1, 'tileDirection': SectionLayoutForm.TILE_VERTICAL, 'sections': sectionKeys};
	        return info;
	    },
		
		// Summary:
	    //    Returns information about a section given its key value. The following is the info format:
	    //    {
	    //         title: "string",          the title for the section
	    //         nColumns: integer > 0,    the number of columns of attributes across the section. 
	    //         attributes: ["key",...]   the array of attribute keys** (please see below)
	    //    }
	    //
	    //    ** The attribute keys can be a simple string tag, or an opique object. If it's an opique
	    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
	    //    unique amoung attribute ids. The attribute's key (id) can then be used to reference and access the
	    //    attribute layout.
	    //
		getSectionInfo: function(/* the content object */content, sectionKey)
	    {
			var sectionInfo = {};			
			sectionInfo.title = sectionKey.attributeName;
			sectionInfo.nColumns = 2; //temp			
			
			var categories = this._templateName[sectionKey.path];
    		sectionKey.parentPart = content.getRoot();
			sectionKey.isAttributeGroup = false;
			sectionInfo.attributes = this._createCollectionFromTemplate(sectionKey, categories, sectionKey, content);								
			
			//if(sectionKey.path == "ossection" || sectionKey.path == "softwaresection" )
				sectionInfo.sectionProvider = this._customElementProvider;
			//else
				//sectionInfo.sectionType = 
				//sectionInfo.isCustomSection = true;
				
			return sectionInfo;
	    },
		
		// Summary:
	    //    Returns information about the attribute given its key value. The attribute info is not static for
	    //    all attributes being described. Certain attribute types will have additional data 
	    //    associated with them.
	    //
	    //    The following is the info format:
	    //    {
	    //         /* 
	    //          * Required Info Data 
	    //          */
	    //         type: value,              the type of attribute, one of the SectionLayoutForm.ATTRIBUTE_*
	    //
	    //         /* 
	    //          * Optional Info Data (NOT YET IMPLEMENTED)
	    //          */
	    //         colSpan: <1..section.nCol> Optionally overrides the parent section layout's nCols attribute.
	    //                                    By default, the attribute will occupy one column of the section. This
	    //                                    allows the attribute to occupy more then one column in the section.
	    //                                    For attribute type ATTRIBUTE_COLLECTION, the default is section.nCols. 
	    //                                    It occupies the entire row in the section.
	    //
	    //         /* 
	    //          * Common Info Data for "atomic" attribute types
	    //          */
	    //         title: "string",          the title for the attribute
	    //         readOnly: <true,false>,  
	    //
	    //         /* 
	    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_COLLECTION type
	    //          */
	    //         collection: [keys...],    the array of sub-attribute keys** (please see below)
	    //         title: "string",          Optional, the title for the collection
	    //
	    //         /* 
	    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_ENUM type
	    //          */
	    //         enumChoices:[values...]
	    //    }
	    //
	    //    ** The attribute keys can be a simple string tag, or an opique object. If it's an opique
	    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
	    //    unique amoung ALL attribute ids. The attribute's id can then be used to reference and access the
	    //    attribute layout.
	    //
		getAttributeInfo: function(/* the content object */content, sectionKey, attributeKey)
	    {
			var parentPart = attributeKey.parentPart;
			var keyName = attributeKey.attributeName;
			var keyId = attributeKey.id;
			var path = attributeKey.path;
			var isAttributeGroup = attributeKey.isAttributeGroup;
			var isAttribute = attributeKey.isAttribute;
	        var attrInfo = {};
			var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
			var template = catalog.getTemplate( content.getRoot().getName() );			
				
			if (isAttribute == false) { //this is a collection
			    var isRow = false;
				
				if(keyName.indexOf("row") == -1)
					attrInfo["title"] = RtlmUiUtil.getMessage(keyName);
				else {
                    //Not show title if it's a row number
                    attrInfo["title"] = null;
                    isRow = true;					
				}
				
				attrInfo["type"] = SectionLayoutForm.ATTRIBUTE_COLLECTION;
				attrInfo["readOnly"] = false;
				
				var elements = this._templateName;
				for(var p in path){
					elements = elements[path[p]]; 
				}
				var keys = this._createCollectionFromTemplate(sectionKey, elements, attributeKey, content);
													
				if(keyId.indexOf(OTHER_CATEGORY) > 0) {
					var partAttrKeys = this._createAttributeList(parentPart);
					for(var p in partAttrKeys) {
						var attrId = partAttrKeys[p].id;
						if (!this._arrayContains(this._displayedAttrs, attrId)) {
							partAttrKeys[p].isAttribute = true;
							partAttrKeys[p].isOther = true;
							partAttrKeys[p].catPath = this.findAttributePath( template, partAttrKeys[p].attributeName );
							partAttrKeys[p].showAttribute = this.isAttributeVisisible( partAttrKeys[p].catPath );
							keys.push(partAttrKeys[p]);
						}
					}
				}
				// if this is a row, obey the parent section's column
				var nCols = keys.length;
				
				if(isRow) {
					var sectInfo = this.getSectionInfo(content, sectionKey);
					nCols = sectInfo.nColumns;
				}
				
				attrInfo["collection"] = keys;
				attrInfo["nColumns"] = nCols;
			}
			else { //this is an attribute
				try {
                    var attr = parentPart.getAttribute(keyName);
                    var type = attr.getType();
                    var attrName = attr.getName();
                    var readOnly = false;
                
					attrInfo = this.inherited("getAttributeInfo", arguments);
					attrInfo.isAttribute = true;
					attrInfo.path = path;
					if(attr.isArray() && !attributeKey.type)
						attrInfo.title = "";
					return attrInfo;
		
					// handle the special pseudo attribute type for the part selection
					if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR) {
						attrInfo = this._constructPartTypeSelectorAttrInfo(attributeKey, parentPart, attr);;
						attrInfo.isAttribute = true;
						attrInfo.path = path;
						return attrInfo;
			        }
					
					// handle multi-value attribute intances. Create the 
			        // concrete attribute info for an array value in the attribute.
			        if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE) {
			            attrInfo = this._constructAttributeInfo(attributeKey, parentPart, attr);
						attrInfo.isAttribute = true;
						attrInfo.path = path;
						return attrInfo;
			        }
					
					// Create a collection to hold the array values for this attribute.
			        if(attr.isArray()) {
						 attrInfo = this._constructMultiValueAttrInfo(attributeKey, parentPart, attr);
//						 attrInfo.title = "";
						 attrInfo.isAttributeGroup = true;
						 attrInfo.isAttribute = false;
						 attrInfo.path = path;
						 return attrInfo;
			        }					
		
					if(!attributeKey.isOther && !this._arrayContains(this._displayedAttrs, attr.getPathName()))
						this._displayedAttrs.push(attr.getPathName());
						
					attrInfo = this._constructAttributeInfo(attributeKey, parentPart, attr);
					attrInfo.isAttributeGroup = true;
					 attrInfo.isAttribute = false;
					 attrInfo.path = path;
					 return attrInfo;

					return this._constructAttributeInfo(attributeKey, parentPart, attr);
//					return this.inherited("getAttributeInfo", arguments);
				}
				catch(error) {
					// attribute not yet in the data model (but expected to be). create temporary as read only
					type = SectionLayoutForm.ATTRIBUTE_LABEL;
					readOnly = true;
				}

				if (type == DefaultAssetContentProvider.TYPE_STRING) {
					type = SectionLayoutForm.ATTRIBUTE_TEXT;
				}
				
				attrInfo["title"] = keyName;
				attrInfo["type"] = type;
				attrInfo["readOnly"] = readOnly;
			}
			
			return attrInfo;	
	    },
		
		/**
		 * 
		 * @param {Object} templateElements
		 * @param {Object} attributeKey
		 */
		_createCollectionFromTemplate: function(sectionKey, /*array*/templateElements, attributeKey, contentObject ) {
			var isAttributeGroup = attributeKey.isAttributeGroup;
			var keyId = attributeKey.id;
			var parentPart = attributeKey.parentPart;
			var path = attributeKey.path;
			var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
			var template = catalog.getTemplate( contentObject.getRoot().getName() );
			
			var keys = [];
			for(var i in templateElements) {
				var isAttr;
				var isAttrGroup;
				var attrName;
				var attrId;
				var attrPath;
				var catPath = null;
				var showing;
				
				if (isAttributeGroup) {						
					isAttr = true; //if parent is attribute group, this must be attribute
				 	isAttrGroup = false;
					attrId = keyId + '.' + templateElements[i];
					attrName = templateElements[i];
					attrPath = path;
					catPath = this.findAttributePath( template, attrName );
					showing = this.isAttributeVisible( sectionKey, catPath );
				}
				else {
					isAttr = false; 
					isAttrGroup = dojo.isArray(templateElements[i]);
					attrId = keyId + '.' + i;
					attrName = i;
					// make a copy of the path, otherwise modify attrPath also modify attributeKey.path.
					attrPath = [];
					for(var p in path) {
						attrPath.push(path[p]);
					}
					attrPath.push(i);
					showing = true;
				}
				
				if( showing )
				{
					keys.push({
							'id': attrId,
							'isAttribute': isAttr, 
							'isAttributeGroup': isAttrGroup,
							'attributeName': attrName,
							'path': attrPath,
							'parentPart': parentPart,
							'catPath': catPath,
							'showAttribute': showing
						});
				}
			}
			
			return keys;
		},
		
		 _createMultiValueList: function(parentPart, attr)
	    {
	        var attrKeys = this.inherited("_createMultiValueList", arguments);	        
			for(var i in attrKeys) {
				attrKeys[i]['isAttribute'] = true;
			}
	        return attrKeys;
	    },
		
		_createAttribute: function(attributeKey, fromTemplate) {
			
		},
		
		_arrayContains: function(array, element) {
			if(!dojo.isArray(array) || !element)
				return false;
			
			for(var i in array) {
				if(array[i] == element)
					return true;
			}
			return false;
		},
		
		findAttributePath: function( template, attributeName )
		{
			return( this._findPathForAttribute( template.getRoot(), attributeName ) );
		},
		
		_findPathForAttribute: function( rootPart, attributeName )
		{
			var parts = rootPart.getAttributeNames();
			var path = null;
			
			for( var i = 0; i < parts.length; i++ )
			{
				var attr = rootPart.getAttribute(parts[i]);
				
				if (parts[i] == attributeName) 
				{
					var path = attr.getPathName();
					return( path );
				}
				
				if( attr.getType() == "ConfigurationPartType" )
				{
					path = this._findPathForAttribute( attr.getRootPart(), attributeName );
				}
				
				if( path )
					return( path );
			}
			
			return( "" );		
		}
	
	});

})();
