/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.JobStatusTableProvider"); 

dojo.require( "dijit.Dialog" );

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");
dojo.require("com.ibm.rational.test.lm.web.client.internal.inventory");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var column_types = [
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TIMESTAMP,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TIMESTAMP,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_LINK,
						TableConstants.RENDER_TYPE_TEXT,
						TableConstants.RENDER_TYPE_ACTION ];
						
	var JOB_STATE_CODE = {
       "UNKNOW" : RtlmUiUtil.getMessage("Unknow"),
	   "CREATED": RtlmUiUtil.getMessage("Started"),
	   "WAITING": RtlmUiUtil.getMessage("Pending"),
       "RUNNING": RtlmUiUtil.getMessage("Running"),
	   "STOPPED": RtlmUiUtil.getMessage("Completed")
	};
	
	var JOB_RESULT_CODE = {
	   "CANCELED": RtlmUiUtil.getMessage("Canceled"),
       "FAILED": RtlmUiUtil.getMessage("Failed"),
       "PASSED": RtlmUiUtil.getMessage("Passed"),
       "WAITING": RtlmUiUtil.getMessage("Waiting"),
       "UNKNOW": RtlmUiUtil.getMessage("Unknow")
	};

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.JobStatusTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function( column_headings, id_fields )
	{
		this.setParameters( { "column_headings" : this.getColumnHeadings(),
							  "id_fields" : this.getIdFields(),
							  "column_types" : column_types } );
	},
	
	getColumnHeadings : function()
	{
		return(  [
	      {title: RtlmUiUtil.getMessage("jobId"), native_column: "id", isSortable: true, includeInFilter: true, width: 6},
	      {title: RtlmUiUtil.getMessage("jobName"), native_column: "job.jobName", isSortable: true, includeInFilter: true, width: 11},
          {title: RtlmUiUtil.getMessage("jobStartTime"), native_column: "startTime", isSortable: true, includeInFilter: true, width: 12}, 
		  {title: RtlmUiUtil.getMessage("jobDuration"), native_column: "duration", isSortable: true, includeInFilter: true, width: 8  },
          {title: RtlmUiUtil.getMessage("jobEndTime"), native_column: "endTime", isSortable: true, includeInFilter: true, width: 12  },
          {title: RtlmUiUtil.getMessage("jobState"), native_column: "state.name", isSortable: true, includeInFilter: true, width: 10  },
          {title: RtlmUiUtil.getMessage("jobResult"), native_column: "result.name", isSortable: true, includeInFilter: true, width: 10  },
          {title: RtlmUiUtil.getMessage("jobMachine"), native_column: "machineName", isSortable: true, includeInFilter: true, width: 15 },
          {title: RtlmUiUtil.getMessage("jobLog"), alignment: TableConstants.ALIGN_CENTER, native_column: dojo.hitch( this, this.getJobLog ), isSortable: false, includeInFilter: true, width: 8 },
          {title: RtlmUiUtil.getMessage("Cancel"), alignment: TableConstants.ALIGN_CENTER, native_column: dojo.hitch( this, this.getCancelText ), isSortable: false, includeInFilter: true, width: 8 }		  
        ] );
	},
	
	getIdFields: function()
	{
		return( "job.id" );
	},

    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return( atColumnIndex == 1 || (atColumnIndex >= 5 && atColumnIndex <= 7) ); 
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return this.column_headings[ atColumnIndex ][ "isSortable" ];
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {    
		var type = this.inherited( "getCellDisplayType", arguments );
		var flag = ( ( rowData["state"]["name"] != "STOPPED" ) );
		
		if( atColumnIndex == 9 )
		{
			if( flag )
				return( TableConstants.RENDER_TYPE_ACTION );
			else
				return( TableConstants.RENDER_TYPE_IMAGE );
		}
		else if( atColumnIndex == 8 )
		{
            // only enable the log link if the job is running or has already
            // stopped.
			if( rowData["state"]["name"] == "RUNNING" || rowData["state"]["name"] == "STOPPED")
			{
				type = TableConstants.RENDER_TYPE_ACTION;
			}
		}
        return( type );
    },
	
    // Summary:
    //    Returns the display data of the cell. To support type like 'dropdown' select,
    //    additional data would need to be passed as parameters.
    //
    getCellDisplayData: function(rowData, atColumnIndex)
    {
		var data = this.inherited( "getCellDisplayData", arguments );
		var day = 0;
        var hour = 0;
        var minute = 0;
        var second = 0;
      	var millisecond = 0;
        var timeValue = 0;
		var resultStr = "";
    	
    	if( this.column_headings[ atColumnIndex ][ "native_column" ] == "state.name" )
    	{
    		data = JOB_STATE_CODE[ data ];
    	}
    	else if( this.column_headings[ atColumnIndex ][ "native_column" ] == "result.name" )
    	{
    		data = JOB_RESULT_CODE[ data ];
    	}
		else if( this.column_headings[ atColumnIndex ][ "native_column" ] == "duration" )
		{
			timeValue = data;
			if (0 != (day = Math.floor(timeValue / 86400000))) {
                timeValue = timeValue % 86400000;
                resultStr += day + "d ";
            }
            if (0 != (hour = Math.floor(timeValue / 3600000))) {
                timeValue = timeValue % 3600000;
                resultStr += hour + "h ";
            }
            if (0 != (minute = Math.floor(timeValue / 60000))) {
                timeValue = timeValue % 60000;
                resultStr += minute + "m ";
            }
            if (0 != (second = Math.floor(timeValue / 1000))) {
                timeValue = timeValue % 1000;
                resultStr += second + "s ";
            }
            if (0 <= timeValue && 1000 > timeValue) {
                millisecond = timeValue;
                if (millisecond != 0) {
                    resultStr += millisecond + "ms ";
                }
            }
			
			data = resultStr;
		}
    	
    	return( ( data ) ? data : "" );
    },

    getCellRawData: function(rowData, atColumnIndex)
    {
        var data; 
        
        // needs to sort by the integer value instead of the display value,
        // which would be a string.
        if( this.column_headings[ atColumnIndex ][ "native_column" ] == "duration" )
        {
            data = rowData['duration'];
        }
        else {
            data = this.getCellDisplayData(rowData, atColumnIndex);
        }
        
        return( data );
    },
    
    getJobLog: function( rowData, atColumnIndex )
    {
        // only show the link text when the log is available.
        if( rowData["state"]["name"] == "RUNNING" || rowData["state"]["name"] == "STOPPED")
        {
            return( RtlmUiUtil.getMessage( "Log" ) );
        }
    	return " ";
    },
    
    getCancelText: function( rowData, atColumnIndex )
    {
		var type = this.getCellDisplayType( rowData, atColumnIndex )
    	return( ( type == TableConstants.RENDER_TYPE_ACTION ) ? RtlmUiUtil.getMessage( "Cancel" ) : null );
    },
    
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
        if(atColumnIndex == 8) {
            val = {};
            val['href'] = "";
        }
        else if( atColumnIndex == 7 )
        {
            val = {};
  			var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
            val['href'] = AssetUriUtil.getViewAssetDetailURI( "Machine", rowData[ "machineInternalId" ] );
        }
		else if( atColumnIndex == 9 )
		{
			val = {};
			
			var flag = ( ( rowData["state"]["name"] != "STOPPED" ) );
		
			if( flag )
				val['image'] = ImageCache.DELETE;
			else
				val['image'] = ImageCache.DDELETE;
		}

        return val;
    },
    
    getInitialSortDirection: function()
    {
    	return( TableConstants.SORT_DIR_DESCEND );
    },
    
    getInitialSortByColumn: function()
    {
    	return( 2 );
    }      
    
});

// Summary:
//    Create a simple subclass of the dialog to export out closed event.
//
dojo.declare("JobStatus.Dialog", 
             dijit.Dialog,
{
    hide: function()
    {
        this.inherited("hide", arguments);
        this.closed();  
    },
    closed: function()
    {
        // override to receive dialog closed.
    }
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.JobStatusCellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
	constructor: function( listener )
	{
		this.listener = listener;
	},
	
    cellActionInvoked: function(rowData, columnIndex)
    {
    	if( ( columnIndex == 8 ) )
    	{
			var inven = new com.ibm.rational.test.lm.web.client.internal.inventory();
		   	var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_jobLogSuccess", "_jobLogError");
			inven.getJobLog( responseHandler, 
							 rowData[ "job" ][ "providerName" ],
							 rowData[ "job" ][ "providerInstance" ],
							 rowData[ "id" ] );
            
            // bring up the dialog immediately. Initially shows a
            // loading message.                 
            this._createLogDialog(RtlmUiUtil.getMessage("appWorking"));
	    }
		else if( columnIndex == 9 )
		{
			this.listener.cancelJob( rowData[ "id" ] );
		}
    },
	
	_jobLogSuccess: function( log )
	{
        var text = (log && log.length > 0) ? log : RtlmUiUtil.getMessage("NoLogAvailable");
        if(this.dlgContentNode) {
            this.dlgContentNode.innerHTML = text;
        }
	},
	
	_jobLogError: function( errorObj )
	{
        if(this.dlgContentNode) {
            this.dlgContentNode.innerHTML = RtlmUiUtil.getMessage("ErrorRetrievingLog");
        }
	},
    
    _createLogDialog: function(text)	
    {
        var panel = '<div dojoType="dijit.layout.LayoutContainer" style="padding:5px; width: 620px; height: 400px">' +
                '<div dojoType="dijit.layout.ContentPane" layoutAlign="client" dojoAttachPoint="textNode">' +
                '  <pre style="padding:5px; overflow:scroll"><div dojoAttachPoint="logContentNode"></div></pre>' +
                '</div>' +
            '</div>';
        var dom = new com.ibm.asq.common.web.ui.internal.util.DomConstructor( panel );
        var dlg = new JobStatus.Dialog( { title: RtlmUiUtil.getMessage("jobLog") } );
        this.dlgContentNode = dom.logContentNode;
        
        // Just in case the user close the dialog before the content came back.
        // Clear the node for updating the dialog's content.
        dlg.closed = dojo.hitch(this, function(){
            this.dlgContentNode = null;
        });
        dom.logContentNode.innerHTML = text;
        dlg.containerNode.appendChild( dom.domNode );
        dlg.show();
    }
});
// Keeping below as documentation on how to prevent default actions on a supplied callback for
// objects such as links.
//    _attachCallbackToCell: function( node, callback, rowObject, columnIndex )
//    {
//    	node.onclick = 	function( event )
//		{ 
//			if( event && event.preventDefault )
//			{
//				event.preventDefault();
//			}
//			else
//			{
//				window.event.cancelBubble = true;
//				window.event.returnValue = false;
//			}
//	    	
//	    	callback( node, rowObject, columnIndex );
//	    	
//	    	return( false );	   		
//		}
//    }
})();
