/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestView");

dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestWidget");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var LOADING = RtlmUiUtil.getMessage("reqMsgRetrievingRequestsWorking");
var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;

// Summary:
//    Provides the request query display. This view displays a list of "canned" queries
//    for requests in the system that is specific to the current user. The view displays
//    the results of the selected query in a table. When the page is constructed, it automatically
//    runs the "default" query and display the results. 
// 
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestView",
    com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout,
{
    noline: true,
    
    postCreate: function() {
        this.inherited("postCreate", arguments);

        this.setProgressStart(LOADING);

        // set up the header area display of the view.
        this.setTitle(RtlmUiUtil.getMessage("reqLabel_requests"));
        
        // set up the CS help id
        this.setCSHelpId("rtlm_request_list.html", com.ibm.rational.test.lm.web.CSHelpID);
        this.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", "", RtlmUiUtil.getMessage("reqLabel_requests")));
		
        // create the query select combo and result display
        var div = document.createElement("div");
        this.getContentNode().appendChild(div);
		
		var params = {
			processStartCallback: dojo.hitch(this, "setProgressStart"),
			processEndCallback: dojo.hitch(this, "setProgressEnd")};
		this.queryRequestWidget = new com.ibm.rational.test.lm.web.ui.internal.views.requests.QueryRequestWidget(params, div);
		this.resultTable= this.queryRequestWidget.resultTable;
    },
	
	_onRefreshActionClick: function()
	{
		this.runSelectedQuery();
	},
	
	runSelectedQuery: function() {
		this.queryRequestWidget.runSelectedQuery();
	},

    getController: function() {
        return this;
    },

    // handler on artifacts changed
    onArtifactChanged: function(changeType, artifactType, artifactArray) {
        // only care about requests
        if (artifactType == RtlmUiUtil.TYPE_REQUEST) {
            this.runSelectedQuery();
        }
    }
});

})();
