// ********  REQUEST FORM VIEW *********
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/**
 * OBSOLETE - This is the old request form. This has been replaced by the RequestView widget.
 * 
 */

//
//
// Author: L. To
//
// Summary:
//
//    The RequestFormView implements a RTLM request form. It provides data entries and displays
//    for an RTLM request. An RTLM request has a state associated with it and depends on its state,
//    certain fields can be edited, or become read-only. This information is driven from the request
//    JSON data. This form view configures the data entry fields accordingly. 
//
//    The consumer of the ReqyestFormView needs to initialize the form either with a call to the method: 
//    setRequestItem(id, isNew), or specify them via the constructor parameters. The form widget would not 
//    complete its loading until this method is called, or the proper parameter is set in the constructor
//    as follows:
//
//        formArg = {requestId: id, isNew: true};
//        var requestView = new com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestFormView(formArg);
//
//    The RequestFormView object uses several internal objects to manage its contents. They are the 
//    DataObject, FieldWrapper and its specific field subclasses and the ActionWrapper. See the 
//    internal object's summaries for more details.
//    
//
(function(){

    dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestFormView");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dijit.form.CheckBox");
    
    dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
    dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
    dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
    dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
    dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
        
    dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.AbstractForm");
    dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
    dojo.require("com.ibm.rational.test.lm.web.client.internal.request");
    dojo.require("com.ibm.rational.test.lm.web.client.internal.user");
    dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.RlmDateTextBox");
    dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.YetAnotherMachineConfigPanel");
    dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");
    dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
    dojo.require("com.ibm.asq.common.web.client.internal._catalog.Configuration");
    dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
	dojo.require("com.ibm.asq.common.web.ui.internal.widgets.TimePicker");
    
    
    // Supported Field types in the form.
    //
    
    // dropdown select field
    var DROPDOWN = "dropdown";
    
    // plain text field
    var TEXT = "text";
    
    // text area field
    var TEXTAREA = "textarea";
    
    // date input field
    var DATE = "date";
    
    // display only field
    var LABEL = "label";
    
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
    var UriUtil = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil;
    var LOADING = RtlmUiUtil.getMessage("appWorking");
    var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
    var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	
	var configuration_panel_count = 0;
    
    // Summary:
    //    Internal wrapper object around the request JSON data. It provides the necessary data conversion
    //    from the JSON to script expected values, such as from a time stamp string to a Date object.
    //
    dojo.declare("Request.DataObject", null, {
        data: null,
        requestForm: null,
        
        constructor: function(requestForm, requestData){
            this.requestForm = requestForm;
            this.data = requestData;
        },
        
        getTitle: function(){
            if (this.data.visibleId) 
                return RtlmUiUtil.getMessage("reqLabel_request") + " " + this.data.visibleId;
            return RtlmUiUtil.getMessage("reqLabel_new_request", this.requestForm.requestId);
        },
        
        getSummary: function(){
            if (this.data.summary) 
                return this.data.summary;
            return null;
        },
        getCreateDate: function(){
            if (this.data.createDate && this.data.createDate.length) {
                return RtlmUiUtil.convertTimeStampToDate(this.data.createDate);
            }
            return new Date();
        },
        getCreator: function(){
            if (this.data.requester) 
                return this.data.requester.name;
            return this.requestForm._currentUser.name;
        },
        getCreatorId: function(){
            if (this.data.requester) 
                return this.data.requester.id;
            return this.requestForm._currentUser.itemId;
        },
        getPriority: function(){
            if (this.data.priority) 
                return this.data.priority;
            return null;
        },
        getPriorityValues: function(){
            if (this.data.priorityValues) 
                return this.data.priorityValues;
            return [];
        },
        getAssignee: function(){
            if (this.data.fulfiller) {
                return this.data.fulfiller.name;
            }
            return null;
        },
        getConfirmByDate: function(){
            if (this.data.confirmationDate && this.data.confirmationDate.length) {
                return RtlmUiUtil.convertTimeStampToDate(this.data.confirmationDate);
            }
            return null;
        },
        getDescription: function(){
            if (this.data.description) 
                return this.data.description;
            return null;
        },
        getReserveFromDate: function(){
            if (this.data.reserveFrom && this.data.reserveFrom.length) {
                return RtlmUiUtil.convertTimeStampToDate(this.data.reserveFrom);
            }
            return null;
        },
        getReserveToDate: function(){
            if (this.data.reserveTo && this.data.reserveTo.length) {
                return RtlmUiUtil.convertTimeStampToDate(this.data.reserveTo);
            }
            return null;
        },
        getCurrentState: function(){
            if (this.data.currentState) 
                return this.data.currentState;
            return null;
        },
        getAttributeValueByName: function(attrName){
            return this.data[attrName];
        },
		
        getConfiguration: function(){
            var configStr = this.data.configuration;
            
            if (configStr == null || configStr.length < 1) 
                return null;
            
            var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
            var config = catalog.createConfigurationFromJson(configStr);
            
			return config;
        },
		
		getConfigString: function()
		{
			return this.data.configuration;
		},
        
        getFoundMachines: function(){
            return this.data.foundMachines;
        },
        getReservedMachines: function(){
            return this.data.reservations;
        },
        
        // Summary:
        //    Given an attribute name, returns whether the attribute is editable in the
        //    current request state context.
        // 
        isFieldEditable: function(attrName){
            if (this.data.editableData) {
                for (var i = 0; i < this.data.editableData.length; i++) {
                    if (attrName == this.data.editableData[i]) 
                        return true;
                }
            }
            return false;
        },
        getActionInfo: function(actionName){
            if (this.data.actionInfos) {
                for (var i = 0; i < this.data.actionInfos.length; i++) {
                    if (actionName == this.data.actionInfos[i].action) 
                        return this.data.actionInfos[i];
                }
            }
            return null;
        }
    });
    
    // Summary:
    //    Internal wrapper object over a request form field. It's responsible for creating 
    //    the UI node to represent the request data based on the specified type, if the field
    //    has not been created yet (the field can be declared/created directly in the HTML page). 
    //    It also wraps around fields that are statically created in the HTML. The object also
    //    responsible for loading and retrieving the data to and from the actual UI field. 
    //    
    dojo.declare("Request.FieldWrapper", null, {
        requestForm: null,
        rootNode: null,
        nodeId: null,
        attrNames: null,
        type: null,
        value: null,
        altEmptyValue: null,
        choiceValues: null,
        isCreated: false,
        fieldNode: null,
        tabIndex: -1,
        dateOnly: false,
		RESERVE_FROM_ID: "text_fromdate",
		TIME_ID_SUFFIX: "_time",
        attachPoint: null,
		statics: {incrementor:0},  //Incrementor used to create unique IDs for fields.
        
        constructor: function(reqForm, root, node_id, attrName, type, attachPoint){
            this.requestForm = reqForm;
            this.rootNode = root;
            this.nodeId = node_id;
            this.attrName = attrName;
            this.type = type;
            if (attachPoint != null) {
                this.isCreated = true;
            }
            this.canSubmit = true;
            this.attachPoint = attachPoint;
        },
        
        // Summary:
        //    Set the field's tab position in the form. Only field that can
        //    accept data would honor this flag. This method must be called before
        //    createNode() for the field to take effect.
        //
        setTabIndex: function(index){
            this.tabIndex = index;
        },
        
        getPrimaryAttributeName: function(){
            return this.attrName;
        },
        
        addContainedAttributeName: function(attrName){
            if (!this.altAttrs) {
                this.altAttrs = [];
            }
            this.altAttrs.push(attrName);
        },
        
        containAttribute: function(attrName){
            if (attrName == this.attrName) 
                return true;
            
            if (this.altAttrs) {
                for (var i = 0; i < this.altAttrs.length; i++) {
                    if (this.altAttrs[i] == attrName) 
                        return true;
                }
            }
            return false;
        },
        
        // Summary:
        //    Returns the UI element's node id that implements this field.
        //
        getNodeId: function(){
            return this.nodeId;
        },
        
        // Summary:
        //    Set a default display value for the field if the field value
        //    is null. Otherwise, if the field value isn't present, an empty
        //    string would be displayed.
        //
        setEmptyValueDisplayString: function(value){
            this.altEmptyValue = value;
        },
        
        // Summary:
        //    If the field value is a Date object, dateOnly flag (true) indicates
        //    only the date is displayed, no time portion is shown. If the flag is false,
        //    both date and time are shown. By default, the dateOnly is true.
        //
        setDateDisplayOnly: function(dateOnly){
            this.dateOnly = dateOnly;
        },
        
        // Summary:
        //    Set the flag to indicate whether the field is display data that can be
        //    committed to the back end. Certain fields are 'artificial' fields that display
        //    read-only data and the data it presents cannot/need not be committed back.
        //    By default, the field can submit.
        //
        setFieldCanSubmit: function(canSubmit){
            this.canSubmit = canSubmit;
        },
        
        canFieldSubmit: function(){
            return this.canSubmit;
        },
        
        // Summary:
        //    Set the field's attribute value into the field. This method doesn't display
        //    the value in the UI. It only stores the value into this object. A separate method 
        //    called loadValueToFieldUI displays this set value in the UI.
        //
        setFieldValue: function(value){
            this.value = value;
        },
        
        setFieldEnumValues: function(values){
            this.choiceValues = values;
        },
        
        // Summary:
        //    Construct or attach to the dom node/dijit field that represents the field based on the type.
        //    This method needs to be called before loading the value to field.
        //
        createNode: function(){
            var pNode = this.rootNode;
            var node;
            
            pNode.innerHTML = "";
            
            if (this.type == LABEL) {
                node = document.createElement("span");
				node.id = this.nodeId + "_" + this.statics.incrementor++;  //Create a 'unique' id for the field.  The IDS must be unique across all open Request views
				this.nodeId = node.id;
                node.name = this.attrName;
                pNode.appendChild(node);
            }
            else 
                if (this.type == DROPDOWN) {
                    node = document.createElement("select");
					node.id = this.nodeId + "_" + this.statics.incrementor++;  //Create a 'unique' id for the field.  The IDS must be unique across all open Request views
					this.nodeId = node.id;
                    node.name = this.attrName;
                    node.onchange = dojo.hitch(this, "notifyDropdownValueChanged");
                    
					var selectLabel = document.createElement("label");
					selectLabel.setAttribute("for", this.nodeId);
		
                    if (dojo.isFF) 
                        node.className = "RequestFormDropDownFieldFF";
                    else 
                        node.className = "RequestFormDropDownField";
                   
				    selectLabel.appendChild(node);
                    pNode.appendChild(selectLabel);
                }
                else 
                    if (this.type == TEXT) {
                        var div = document.createElement("div");
                        pNode.appendChild(div);
                        
                        node = new dijit.form.TextBox({
                            name: this.attrName,
                            value: ""
                        }, div);
                        dojo.addClass(node.domNode, "RequestFormField");
                        dojo.style(node.domNode, "width", "98%");
                        node.onkeyup = dojo.hitch(this, "notifyFieldValueChanged");
                        this.nodeId = node.id;
                    }
					else
						if (this.type == TEXTAREA)
						{
							node = document.createElement("textarea");
							node.id = this.nodeId + "_" + this.statics.incrementor++;  //Create a 'unique' id for the field.  The IDS must be unique across all open Request views
							this.nodeId = node.id;
							node.name = this.attrName;
							node.rows=6;
							node.cols=60;
							node.wrap="soft";
							dojo.addClass(node, "RequestFormText");
							node.onkeyup = dojo.hitch(this, "notifyFieldValueChanged");
							node.onchange = dojo.hitch(this, "notifyFieldValueChanged");
							pNode.appendChild(node);
						}
                    else 
                        if (this.type == DATE) {
                            var dateMsg = RtlmUiUtil.getMessage("reqMsgDateInputFormat");
                            var div = document.createElement("div");
                            pNode.appendChild(div);
                            
							// date picker
							var datePickerSpan = document.createElement("span");
							div.appendChild(datePickerSpan);
							
                            node = new com.ibm.rational.test.lm.web.ui.internal.widgets.RlmDateTextBox({
                                name: this.attrName,
                                value: "",
                                invalidMessage: dateMsg,
                                nonExistDateMessage: RtlmUiUtil.getMessage("regMsgNonExistDate")
                            }, datePickerSpan);
                            dojo.addClass(node.domNode, "RequestFormField");
                            node.onChange = dojo.hitch(this, "notifyFieldValueChanged");
                            this.nodeId = node.id;
                            
                            var calendar = document.createElement("button");
                            calendar.id = this.attrName + "Cal";
                            calendar.className = "FormHelperAction";
                            calendar.onclick = dojo.hitch(this, "notifyCalendarPressed");
                            var img = document.createElement("img");
                            img.src = dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/images/icons/obj16/calendar_obj.gif");
                            calendar.appendChild(img);
                            calendar.className = "FormHelperAction";
                            
                            div.appendChild(calendar);
							
							// time picker
							var timePickerSpan = document.createElement("span");
							div.appendChild(timePickerSpan);
							
							var timePicker = new com.ibm.asq.common.web.ui.internal.widgets.TimePicker({}, timePickerSpan);	
                            this.timePickerId = timePicker.id;						
							timePicker.onChange = dojo.hitch(this, "notifyFieldValueChanged");
                        }
            this.fieldNode = node;
            this.isCreated = true;
            return this.nodeId;
        },
        
        notifyCalendarPressed: function(){
            this.requestForm.showCalendar(this.fieldNode);
        },
        
        notifyFieldValueChanged: function(){
            this.requestForm.textFieldValueChanged(this)
        },
        
        notifyDropdownValueChanged: function(){
            this.requestForm.loadAttributeValues(this.nodeId);
        },
        
        // Summary:
        //     Loads the set field value into the UI field.
        //
        loadValueToFieldUI: function(){
            var node = dojo.byId(this.nodeId);
            var val;
            
            // fields has not been attached/created.
            if (!node) 
                return;
            
            if (this.type == LABEL) {
                var val = (this.value == null) ? this.altEmptyValue : this.value;
                
                if (this.value && this.value instanceof Date) {
                    // display only the date for our date fields.
                    if (this.dateOnly) 
                        val = dojo.date.locale.format(this.value, {
                            selector: 'date',
                            formatLength: "short"
                        });
                    else 
                        val = RtlmUiUtil.getDisplayDateTimeString(this.value);
                }
                node.innerHTML = (val == null) ? "" : val;
            }
            else 
                if (this.type == DROPDOWN) {
                    RtlmUiUtil.setComboContent(this.nodeId, this.choiceValues, this.value, true, true);
                }
                else 
                    if (this.type == TEXTAREA) {
                        var val = (this.value) ? this.value : "";
                        this.fieldNode.value = val;
                    }
                    else 
                        if (this.type == TEXT) {
                            var val = (this.value) ? this.value : "";
                            this.fieldNode.setValue(val);
                        }
                        else 
                            if (this.type == DATE) {
                                if (!this.value)
                                    this.fieldNode.textbox.value = "";
                                this.fieldNode.setValue(this.value);
								// set time picker value 
								var timePickerId = this.timePickerId;
								var timePicker = dijit.byId(timePickerId);
								if (timePicker) {
									if (this.value) 
										timePicker.setValue(this.value);
									else {
										var defaultTime = new Date();
										if (this.nodeId == this.requestForm.RESERVE_FROM_ID) {
											defaultTime.setHours(0);
											defaultTime.setMinutes(0);
										}
										else {
											defaultTime.setHours(23);
											defaultTime.setMinutes(59);
										}
										timePicker.setValue(defaultTime);
									}
								}
                            }
        },
        
        getFieldValue: function(){
            // the field is read-only, just return the original data.
            if (this.type == LABEL) {
                // if the value is a date, always returns the time in millisec.
                if (this.value && this.value instanceof Date) {
                    return this.value.getTime();
                }
                return this.value;
            }
            var val = this.requestForm.getValueById(this.nodeId);
            if (val == null) 
                val = "";
            return val;
        },
        
        // Summary:
        //    Provides a field specific error message if the field has invalid/error
        //
        setFieldErrorMsg: function(msg){
            this.errMsg = msg;
        },
        
        hasError: function(){
            // only the date field can have invalid values,
            // only need to check it.
            if (this.type == DATE && this.fieldNode) {
                if (!this.fieldNode.isValid()) 
                    return (this.errMsg) ? this.errMsg : RtlmUiUtil.getMessage("MsgInvalidInput");
            }
            return null;
        }
    });
    
    
    // Summary:
    //    Specialized field wrapper for the configuration field.
    //
    dojo.declare("Request.FieldWrapper.Configuration", Request.FieldWrapper, {
        mode: null,

        
        constructor: function(reqForm, rootId, node_id, attrName, mode, configString){
            this.requestForm = reqForm;
            this.rootNode = rootId;
            this.nodeId = node_id + '_' + configuration_panel_count;
			configuration_panel_count ++;
            this.attrName = attrName;
            this.mode = mode;
            this.configString = configString;
        },
        
        createNode: function(){
            var pNode = this.rootNode;
            pNode.innerHTML = "";
            
            var div = document.createElement("div");
            pNode.appendChild(div);
            
			// construct the configuration widget.
			this.fieldNode = new com.ibm.rational.test.lm.web.ui.internal.widgets.YetAnotherMachineConfigPanel({
				name: this.attrName,
				headerClass: "sectionAreaBar",
				headerLabelClass: "sectionAreaLabel",
				actionLinkClass: "formActionLink",
				title: RtlmUiUtil.getMessage("reqLabel_machine_def"),
				onChange: dojo.hitch(this, "notifyValueChanged"),
				reqForm: this.requestForm
			}, div);
			this.fieldNode.className = "machineConfigContainer";
			this.nodeId = this.fieldNode.id;

			this.isCreated = true;
			return this.nodeId;
        },
        
        csHelpHandler: function(){
            //showHelpContent(this.fieldNode.getCsHelpButton(), 'rtlm_config');
        },
        
        notifyValueChanged: function(){
            this.requestForm.valueChanged(this.nodeId);
        },
        
        loadValueToFieldUI: function(){
            if (!this.isCreated) 
                this.createNode();
            
            this.fieldNode.setConfig(this.mode, this.value);
        },
        
        isEdited: function(){
            if (this.fieldNode) 
                return this.fieldNode.isEdited();
            return false;
        },
        
        getWarning: function(){
            if (this.fieldNode == null) 
                return "";
            return this.fieldNode.getWarning();
        },
        
        getFieldValue: function(){
            if (this.fieldNode == null) 
                return this.value;
            
            var val = this.fieldNode.getConfig(this.requestForm.invokedAction);
            return val;
        },
        
        hasError: function(){
            var error = this.fieldNode.getError();
            return (error.length > 0) ? error : null;
        },
		
		//This is called from the config view when the user wants to create a Request from an existing config
		//We are passed the config ID so we need to load the config.
		setConfigSelection: function(configID) 
		{
			if (this.mode == "EDITABLE" || this.mode == "SELECTABLE") 
			{
		        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadConfigSuccess", "_loadConfigFailure");
        		var configService = new com.ibm.rational.test.lm.web.client.internal.config();
				configService.retrieveConfigurationWithDetails(responseHandler, configID);

			}
		},
		
		_loadConfigSuccess: function(response)
		{
	            var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
				var config = catalog.createConfigurationFromJson(response.configuration); 
				this.fieldNode.setConfig(this.mode, config);
		},
		
		_loadConfigFailure: function(response)
		{
			
		}

    });
    
    // Summary:
    //    Specialized field wrapper for the request title area.
    //
    dojo.declare("Request.FieldWrapper.TitleArea", Request.FieldWrapper, {
        requestIdNode: null,
        
        constructor: function(reqForm, node){
            this.requestForm = reqForm;
            this.fieldNode = node;
        },
        
        createNode: function(){
            /*
             this.requestIdNode = document.createElement("span");
             this.requestIdNode.className = "formSubTitle";
             this.fieldNode.pageTitle.appendChild(this.requestIdNode);
             */
            this.isCreated = true;
            return this.nodeId;
        },
        
        loadValueToFieldUI: function(){
            this.fieldNode.setTitle(this.value);
        },
        
        getFieldValue: function(){
            return null;
        },
        hasError: function(){
            return null;
        }
    });
    
    // Summary:
    //    Internal wrapper object over request form actions. 
    //    
    dojo.declare("Request.ActionWrapper", null, {
        reqForm: null,
        actionId: null,
        nodeId: null,
        label: null,
        isCreated: false,
        isActive: true,
        isVisible: true,
        isDisable: true,
        actionInfo: null,
        actionUiNodes: null,
        requireEditToEnable: false,
        
        // Summary:
        //    Constructs an action wrapper. 
        
        constructor: function(reqForm, nodeId, actionInfo, isCreated, isVisible){
            this.reqForm = reqForm;
            this.nodeId = nodeId;
            this.actionInfo = actionInfo;
            this.isCreated = isCreated;
            this.isActive = (this.actionInfo != null);
            this.actionId = (this.actionInfo != null) ? this.actionInfo.action : nodeId;
            
            // if the caller specified the "isVisible" flag, honor it.
            if (typeof(isVisible) != "undefined") 
                this.isVisible = isVisible;
            
            this.configureActionUI();
        },
        
        getId: function(){
            return this.actionId;
        },
        
        configureActionUI: function(){
            this.actionUiNodes = [];
            
            var node = null;
            
            // either create or attach the node.
            if (this.isCreated) {
                node = this.nodeId;
            }
            else 
                if (this.isActive) {
                    var buttonArea = this.reqForm.buttonAreaRootNode;
                    this.label = RtlmUiUtil.getMessage(this.nodeId);
                    node = document.createElement("button");
                    node.id = this.nodeId;
                    node.innerHTML = this.label;
                    buttonArea.appendChild(node);
                }
            
            if (node) {
                var visible = (this.isActive && this.isVisible) ? "visible" : "hidden";
                
                node.disabled = this.isDisable;
                node.object = this;
                dojo.style(node, "visibility", visible);
                node.onclick = function(){
                    this.object.dispatch(this);
                };
                
                this.actionUiNodes.push(node);
            }
        },
        
        // Summary:
        //    Internal method called by the action's UI node (button, etc.) when 
        //    it's activated. It dispatches to the request form's invokeAction for
        //    processing.
        //
        dispatch: function(button){
            // For a link element ("a"), it doesn't have
            // disabled attribute. Implements the disabled-ness
            // here.
            if (this.isDisable) 
                return;
            
            this.reqForm._invokeAction(this, button);
        },
        
        // Summary:
        //     Returns the action's attributes. 
        //
        getActionInfo: function(){
            return this.actionInfo;
        },
        
        isPageTransition: function(){
            return this.actionInfo && this.actionInfo.pageTransition;
        },
        
        // Summary:
        //    Returns true if the action has actionInfo; this means the action is valid
        //    for the state of the request and the action will actually has a UI to invoke such 
        //    action. If the action doesn't have a actionInfo record, this method returns false.
        //
        isActionActive: function(){
            return this.isActive;
        },
        
        // Summary:
        //    Set the action's UI visibility attribute. It's only visible if this action is active
        //    and it's visible.
        //
        setVisible: function(visible){
            this.isVisible = visible;
            
            var visible = (this.isActive && visible) ? "visible" : "hidden";
            for (var i = 0; i < this.actionUiNodes.length; i++) 
                dojo.style(this.actionUiNodes[i], "visibility", visible);
        },
        
        // Summary:
        //    Set the action's UI enable state. 
        //
        setDisable: function(disable){
            this.isDisable = disable;
            
            for (var i = 0; i < this.actionUiNodes.length; i++) {
                var node = this.actionUiNodes[i];
                node.disabled = disable;
                
                // for action links, change its class to indicate
                // enable/disablement.
                if (node.nodeName && node.nodeName == "A") {
                    var style = (this.isDisable) ? "formActionLinkDisabled" : "formActionLink";
                    node.className = style;
                }
            }
        },
        
        // Summary:
        //    If the action doesn't have any required field, it will be enabled automatically.
        //    Use this method to only enable the action when the form has been edited. By default,
        //    the ActionWrapper does NOT require edit to be enabled.
        // 
        setIsRequireEditToEnable: function(requireEdit){
             this.requireEditToEnable = requireEdit;
        },
        
        // Summary:
        //    If the action doesn't have any required field, it will be enabled automatically.
        //    This indicates whether the action can be enabled if there's no editing in the form.
        // 
        isRequireEditToEnable: function(){
            return this.requireEditToEnable;
        }
    });
    
    //
    // Summary:
    //
    dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestFormView", [dijit._Widget, dijit._Templated, com.ibm.rational.test.lm.web.ui.internal.widgets.AbstractForm], {
        templatePath: dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/views/requests/templates/RequestFormView.html"),
        widgetsInTemplate: true,
        
        requestId: null,
        isNew: false,
		configId: null,
        
        _tabIndex: 1,
        _requestData: null,
        _currentUser: null,
        _invalidInputMsg: null,
        _warningMsg: null,
        _actions: null,
        
        // form display label variable section 
        statusLabel: RtlmUiUtil.getMessage("reqLabel_status"),
        detailSectionLabel: RtlmUiUtil.getMessage("reqLabel_details"),
        createOnLabel: RtlmUiUtil.getMessage("reqLabel_created_on"),
        createdByLabel: RtlmUiUtil.getMessage("reqLabel_created_by"),
        assignedLabel: RtlmUiUtil.getMessage("reqLabel_assigned"),
        priorityLabel: RtlmUiUtil.getMessage("reqLabel_priority"),
        confirmByLabel: RtlmUiUtil.getMessage("reqLabel_confirm_by"),
        descriptionLabel: RtlmUiUtil.getMessage("reqLabel_description"),
        reservationLabel: RtlmUiUtil.getMessage("reqLabel_reservation"),
        fromLabel: RtlmUiUtil.getMessage("reqLabel_from"),
        toLabel: RtlmUiUtil.getMessage("reqLabel_to"),
        findMachineLabel: RtlmUiUtil.getMessage("reqLabel_FindMachines"),
        reservedMachineLabel: RtlmUiUtil.getMessage("reqLabel_ReservedMachines"),
        reserveMachineLabel: RtlmUiUtil.getMessage("reqLabel_reserveMachine"),
        
        // scratch pad area for date inputs
        _reserveFromDate: null,
        _reserveToDate: null,
        _confirmByDate: null,
        
        // attachPoints
        createdOnNode: null,
        creaytedByNode: null,
        descriptionNode: null,
        statusDisplayNode: null,
        buttonAreaRootNode: null,
        
        constructor: function(args){
        
        },
        
        postCreate: function(){
            this.findMachineProgressImgNode.src = dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/images/progress.gif");
            this.viewHeaderNode.showContentArea(false);
            
            // set the CS help id based on user role
            var user = InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer").currentContributor;
            this.viewHeaderNode.setCSHelpId(this.isNew ? "rtlm_new_request.html" : this._hasRequestAdministerPermission() ? "rtlm_request_details_labmgr.html" : "rtlm_request_details.html");
            
            this.refreshButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("Refresh"), dojo.hitch( this, this._onRefreshActionClick ), null, "refresh-icon-image" );
            // if we have the request id set in the widget, complete the loading of
            // the form now.
             if (this.requestId != null) 
            {
                this.setRequestItem(this.requestId, this.isNew);
            }
                
            this.viewHeaderNode.setIconClass("request-icon-48");
            this.viewHeaderNode.setTypeLabel(this.isNew ? "" : RtlmUiUtil.getMessage("reqLabel_request"));
            
            // set the button area root node to the view header right area
            this.buttonAreaRootNode = this.viewHeaderNode.getRightActionsContainer();
        },
        
        _onRefreshActionClick: function()
        {
            this._fetchData();
        },
        
        _fetchDataFailure: function(errorType, errorMessage){
            this.showStatusLine(RtlmUiUtil.getMessage("reqMsgFetchDataError"), ViewHeaderLayout.ERROR);
            this.viewHeaderNode.setProgressEnd();
        },
        
        _fetchRequestDataSuccess: function(response){
            // wrap the data with our object.
            this._requestData = new Request.DataObject(this, response);
            this._configurePage();
            this.viewHeaderNode.setProgressEnd();
			if(this.configId)
				this.setConfigSelection(this.configId);
        },
        
        _fetchRequestData: function(){
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_fetchRequestDataSuccess", "_fetchDataFailure");
            var request = new com.ibm.rational.test.lm.web.client.internal.request();
            
            if (this.isNew) 
                request.getAttributeValues(responseHandler);
            else 
                request.getByUuid(responseHandler, this.requestId);
        },
        
        _fetchUsersSuccess: function(response){
            this._userList = response;
            this._userNameList = [];
            
            for (var i = 0; i < response.length; i++) {
                this._userNameList.push(response[i].name);
            }
            this._fetchRequestData();
        },
        
        _fetchUsers: function(){
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_fetchUsersSuccess", "_fetchDataFailure");
            var user = new com.ibm.rational.test.lm.web.client.internal.user();
            user.getUsers(responseHandler);
        },
        
        // Summary
        //    Fetches essential data from the backend, this includes current user info, lab manager users
        //    and request attributes. This method will fetch data that we already have cached.
        //
        _fetchData: function(){
            this.viewHeaderNode.setProgressStart(LOADING);
            // we already have user info, just fetch the attributes.
            if (this._currentUser) {
                this._fetchUsers();
                return;
            }
            
            var callbackObj = {
                parent: this,
                successFunc: function(response){
                    this.parent._currentUser = response;
                    
                    // fetch the list of users.
                    this.parent._fetchUsers();
                },
                errorFunc: function(errorType, errorMessage){
                    this.parent._fetchDataFailure(errorType, errorMessage);
                }
            };
            
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(callbackObj, "successFunc", "errorFunc");
            var user = new com.ibm.rational.test.lm.web.client.internal.user();
            user.getCurrent(responseHandler);
        },
        
        // Summary
        //   Loads the page's contents. We need to get the request's data/state
        //   from the server first. This method makes a asyc call to the server to
        //   fetch the data. The async return callback will complete the configuration
        //   of the request form.
        //
        setRequestItem: function(id, isNew){
            this.requestId = id;
            this.isNew = isNew;
            
            if( this.isNew )
            {
            	this.refreshButton.setDisabled( true );
            }
            
            this._fetchData();
        },
        
        // Summary:
        //    Configure the page based on the state of the request and the role of the current user.
        //    This includes making fields editable/uneditable and customizing the actions that can be
        //    performed on the request.
        //
        _configurePage: function(){
            this._isLoading = true;
            
            this.isNew = this._requestData.getAttributeValueByName(this.REQ_ID_ATTR) == null ? true : false;
            this._configurePageFields();
            this._configureActions();
            this._loadRequestDataToPage();
            this._checkActionButtonEnablement();
            
            if(!this.isNew) {
                // show the proper asset's name in its parent container.
                dojo.publish(GlobalEvents.UpdateCurrentTabLabel, [this._requestData.getTitle(), "request-icon-16"]);
            }
            this._isLoading = false;
        },
        
        // Summary:
        //    Configure/create the form's data fields. Create a field wrapper objects 
        //    around all fields in the form. The field wrapper object given the type will
        //    construct the proper UI display for the field.
        //
        _configurePageFields: function(){
            this._fieldObjects = [];
			this._fields = {};
            var field, type;
            
            // reset the tab index
            this._tabIndex = 1;
            
            // create the "static" fields in the form. These are the fields
            // that don't change no matter what's the request state is.
            //
            field = new Request.FieldWrapper.TitleArea(this, this.viewHeaderNode);
            field.setFieldValue(this._requestData.getTitle());
            field.setFieldCanSubmit(false);
            field.createNode();
            this._fieldObjects.push(field);
            
            field = new Request.FieldWrapper(this, this.statusDisplayNode, this.STATE_DISPLAY_ID, this.STATE_ATTR, LABEL, null);
            field.setFieldValue(this._requestData.getCurrentState());
            this.STATE_DISPLAY_ID = field.createNode();
            this._fieldObjects.push(field);
            
            type = this.getFieldType(this.SUMMARY_ATTR);
            var label_id = (type == LABEL) ? "reqLabel_summary" : "reqLabel_summary_editable"
            this.summaryLabelNode.innerHTML = RtlmUiUtil.getMessage(label_id);
			field = new Request.FieldWrapper(this, this.summaryRootNode, this.SUMMARY_ID, this.SUMMARY_ATTR, type, null);
            field.setFieldValue(this._requestData.getSummary());
            this.SUMMARY_ID = field.createNode();
            this._fieldObjects.push(field);
            
            field = new Request.FieldWrapper(this, this.createdOnNode, this.CREATED_ON_ID, this.CREATED_ON_ATTR, LABEL, null);
            field.setDateDisplayOnly(false);
            field.setFieldValue(this._requestData.getCreateDate());
            this.CREATED_ON_ID = field.createNode();
            this._fieldObjects.push(field);
            
            field = new Request.FieldWrapper(this, this.createdByNode, this.CREATED_BY_ID, this.CREATED_BY_ATTR, LABEL, null );
            field.setFieldValue(this._requestData.getCreator());
			this.CREATED_BY_ID = field.createNode();
            this._fieldObjects.push(field);
            
            // create the dynamic fields in the form. These are the fields that 
            // could become read-only or hidden based on the state of the request.
            // The field's type is first determined before the field is created.
            //
            type = this.getFieldType(this.ASSIGNED_ATTR);
            field = new Request.FieldWrapper(this, this.assignedRootNode, this.ASSIGNED_ID, this.ASSIGNED_ATTR, type, null);
            field.setEmptyValueDisplayString(RtlmUiUtil.getMessage("reqLabel_unassigned"));
            field.setFieldValue(this._requestData.getAssignee());
            field.setFieldEnumValues(this._userNameList);
            this.ASSIGNED_ID = field.createNode();
            this._fieldObjects.push(field);
            
            type = this.getFieldType(this.PRIORITY_ATTR);
            field = new Request.FieldWrapper(this, this.priorityRootNode, this.PRIORITY_ID, this.PRIORITY_ATTR, type, null);
            field.setFieldValue(this._requestData.getPriority());
            field.setFieldEnumValues(this._requestData.getPriorityValues());
            this.PRIORITY_ID = field.createNode();
            this._fieldObjects.push(field);
            
            type = this.getFieldType(this.CONFIRM_BY_ATTR);
            field = new Request.FieldWrapper(this, this.confirmByRootNode, this.CONFIRM_BY_ID, this.CONFIRM_BY_ATTR, type, null);
            field.setFieldErrorMsg(RtlmUiUtil.getMessage("reqMsgNeededByDateError"));
            this._confirmByDate = this._requestData.getConfirmByDate();
            field.setFieldValue(this._confirmByDate);
            this.CONFIRM_BY_ID = field.createNode();
            this._fieldObjects.push(field);
            
            field = new Request.FieldWrapper(this, this.descriptionNode, this.DESCRIPTION_ID, this.DESCRIPTION_ATTR, TEXTAREA, null);
            field.setFieldValue(this._requestData.getDescription());
            this.DESCRIPTION_ID = field.createNode();
            if (!this._requestData.isFieldEditable(this.DESCRIPTION_ATTR)) 
                field.fieldNode.readOnly = true;
            this._fieldObjects.push(field);
            
            type = this.getFieldType(this.RESERVE_FROM_ATTR);
            field = new Request.FieldWrapper(this, this.reserveFromRootNode, this.RESERVE_FROM_ID, this.RESERVE_FROM_ATTR, type, null);
            field.setFieldErrorMsg(RtlmUiUtil.getMessage("reqMsgFromDateError"));
            this._reserveFromDate = this._requestData.getReserveFromDate();
            field.setFieldValue(this._reserveFromDate);
            this.RESERVE_FROM_ID = field.createNode();
            this._fieldObjects.push(field);
            
            type = this.getFieldType(this.RESERVE_TO_ATTR);
            field = new Request.FieldWrapper(this, this.reserveToRootNode, this.RESERVE_TO_ID, this.RESERVE_TO_ATTR, type, null);
            this._reserveToDate = this._requestData.getReserveToDate();
            field.setFieldValue(this._reserveToDate);
            field.setFieldErrorMsg(RtlmUiUtil.getMessage("reqMsgToDateError"));
            this.RESERVE_TO_ID = field.createNode();
            this._fieldObjects.push(field);
            
            var mode = this.getConfigurationMode();
            field = new Request.FieldWrapper.Configuration(this, this.configAreaRootNode, this.CONFIGURATION_ID, this.CONFIGURATION_ATTR, mode, this._requestData.getConfigString());
            field.setFieldValue(this._requestData.getConfiguration());
            this.CONFIGURATION_ID = field.createNode();
            this._fieldObjects.push(field);
			this._fields[this.CONFIGURATION_ID] = field;
        },
        
        // Summary:
        //    Based on the control data from the back-end, returns the mode for
        //    the configuration field. It can be one of the three modes: editable,
        //    selectable, and readonly.
        //
        getConfigurationMode: function(){
            var editable = this._requestData.isFieldEditable(this.CONFIGURATION_ATTR);
            
            if (!editable) {
                // if we have the find machine action, this means the configuration
                // items can be selectable (toggle on/off).
                if (this._requestData.getActionInfo(this.FIND_MACHINE_ACTION_ID) != null) 
                    return "SELECTABLE"
                return "READONLY";
            }
            return "EDITABLE";
        },
		
		setConfigSelection: function(configId) {
			var mode = this.getConfigurationMode();
			if (mode == "EDITABLE" || mode == "SELECTABLE") {
				var configField = this._fields[this.CONFIGURATION_ID];
				if(configField)
					configField.setConfigSelection(configId);
			}
		},
        
        // Summary:
        //     Given a request attribute, returns its field display type.
        //     If the field is non-editable in the current request state, it's set to LABEL.
        //
        getFieldType: function(attrName){
            if (this._requestData.isFieldEditable(attrName)) {
                if (attrName == this.SUMMARY_ATTR) 
                    return TEXT;
                if (attrName == this.PRIORITY_ATTR ||
                attrName == this.ASSIGNED_ATTR) 
                    return DROPDOWN;
                
                if (attrName == this.CONFIRM_BY_ATTR ||
                attrName == this.RESERVE_FROM_ATTR ||
                attrName == this.RESERVE_TO_ATTR) 
                    return DATE;
            }
            return LABEL;
        },
        
        // Summary
        //    Create the request action buttons based on the current request state and user's role.
        //
        _configureActions: function(){
            var tab = this._tabIndex;
            var action;
            this._actions = [];
            var actionInfo = null;
            
            // clear the action area first.
            var buttonArea = this.buttonAreaRootNode;
            this.buttonAreaRootNode.innerHTML = "";
            
            actionInfo = this._requestData.getActionInfo(this.SUBMIT_ACTION_ID);
            if (actionInfo) {
                action = new Request.ActionWrapper(this, this.SUBMIT_BUTTON_ID, actionInfo, false);
                this._actions.push(action);
            }
            actionInfo = this._requestData.getActionInfo(this.FULFILL_ACTION_ID);
            if (actionInfo) {
                action = new Request.ActionWrapper(this, this.FULFILL_BUTTON_ID, actionInfo, false);
                this._actions.push(action);
            }
            actionInfo = this._requestData.getActionInfo(this.RETURN_ACTION_ID);
            if (actionInfo) {
                action = new Request.ActionWrapper(this, this.RETURN_BUTTON_ID, actionInfo, false);
                this._actions.push(action);
            }
            actionInfo = this._requestData.getActionInfo(this.REOPEN_ACTION_ID);
            if (actionInfo) {
                action = new Request.ActionWrapper(this, this.REOPEN_BUTTON_ID, actionInfo, false);
                this._actions.push(action);
            }
            actionInfo = this._requestData.getActionInfo(this.CLOSE_ACTION_ID);
            if (actionInfo) {
                action = new Request.ActionWrapper(this, this.CLOSE_BUTTON_ID, actionInfo, false);
                this._actions.push(action);
            }
            actionInfo = this._requestData.getActionInfo(this.CANCEL_ACTION_ID);
            if (actionInfo) {
                action = new Request.ActionWrapper(this, this.CANCEL_BUTTON_ID, actionInfo, false);
                this._actions.push(action);
            }
            
            // reset & save buttons are always needed.
            var resetInfo = {};
            resetInfo["action"] = this.RESET_ACTION_ID;
            resetInfo["pageTransition"] = false;
            resetInfo["requiredData"] = [];
            action = new Request.ActionWrapper(this, this.RESET_BUTTON_ID, resetInfo, false);
            
            // the reset button needs editing to be enabled.
            action.setIsRequireEditToEnable(true);
            this._actions.push(action);
            
            actionInfo = this._requestData.getActionInfo(this.SAVE_ACTION_ID);
            if (actionInfo) {
                action = new Request.ActionWrapper(this, this.SAVE_BUTTON_ID, actionInfo, false);
                // the save button needs editing to be enabled.
                action.setIsRequireEditToEnable(true);
                this._actions.push(action);
            }
        },
        
        // Summary:
        //    Returns the ActionWrapper object by the specified actionId.
        //
        _getActionById: function(actionId){
            for (var i = 0; i < this._actions.length; i++) {
                if (this._actions[i].getId() == actionId) 
                    return this._actions[i];
            }
            return null;
        },
        
        // Summary:
        //    Main entry point for invoking an action. This is called by the ActionWrapper's 
        //    dispatch of action UI events.
        //
        _invokeAction: function(/* ActionWrapper */action, /* the ui that invoked the action */ node){
            var needCheckInput = true;
            this.invokedAction = action;
			          
            // the Reset action is a special action that's not defined by
            // the back-end. Handle it differently.
            if (action.getId() == this.RESET_ACTION_ID) {
                this._doReset();
                return;
            }
            
            // warn the use before canceling.
            if (action.getId() == this.CANCEL_ACTION_ID) {
                // confirm with the user first,
                if (!confirm(RtlmUiUtil.getMessage("reqMsgCancelConfirm"))) 
                    return;
                // If the Request hasn't been save we an just exit
                if (!this._requestData.data.visibleId) {
                    this.exitPage();
                    return;
                }
            }
            
            // the find machine action has its own status area, don't
            // need to show the working dialog. 
            this._needShowViewProgress = true;
            
            if (needCheckInput && !this._checkValidInput()) {
                dijit.showTooltip(this._getInvalidInputMessage(), node);
                setTimeout(function(){
					dijit.hideTooltip(node);
				}, 4000);
                return;
            }
            
            // check for warning conditions
            if (needCheckInput && this._checkHasWarning()) {
                if (!confirm(this._warningMsg)) 
                    return;
            }

            // Disable the action to prevent the user from double clicking the button to invoke
            // the action twice.  Cancel and Reset should not be disabled and is already handled
            // and returned earlier.
            //
            action.setDisable(true);
            
            // invoke the backend action.
            //
            var actionInfo = action.getActionInfo();
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_actionSucceedHandler", "_actionFailHandler");
            var request = new com.ibm.rational.test.lm.web.client.internal.request();
            var reqData = this._getFormData();
            
            // fill in the action directive to invoke in the backend.
            reqData[this.DIRECTIVE_ATTR] = actionInfo.action;
            
            // fill in the project area ID to use in the backend.
            reqData[this.PROJECT_AREA_ATTR] = com.ibm.team.process.web.projectArea.itemId; 
            
            //bring up a working/busy dialog while we're running the action.
            if (this._needShowViewProgress) 
                this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("reqMsgPerformRequestAction"));
            
            request.update(responseHandler, reqData);
        },
        
        _actionSucceedHandler: function(response){
            // bring down the working dialog.  
            if (this._needShowViewProgress) 
                this.viewHeaderNode.setProgressEnd();
            
            // if after invoked the action we need to transition the page,
            // exit now.
            if (this.invokedAction && this.invokedAction.isPageTransition()) {
                this.invokedAction = null;
                this.exitPage();
                return;
            }
            
            this.invokedAction = null;
            
            this.showStatusLine(RtlmUiUtil.getMessage("reqMsgRequestActionSucceeded"), ViewHeaderLayout.SUCCESS);
            this._requestData = new Request.DataObject(this, response);
            
            if(this.isNew) {
                this.requestId = response.id;
                dojo.publish(GlobalEvents.UpdateCurrentTabUri, [UriUtil.getEditRequestURI(this.requestId)]);
            }
            
            this._resetEditIndicators();
            this._configurePage();
        },
        
        _actionFailHandler: function(errorType){
            if (this._needShowViewProgress) 
                this.viewHeaderNode.setProgressEnd();

            if (errorType.status == 409) {
                // Stale data
                this.showStatusLine(RtlmUiUtil.getMessage("reqMsgRequestActionFailedStaleData"), ViewHeaderLayout.ERROR);
            }
            else {
                if(errorType.message && errorType.message.toLowerCase().indexOf('license') > -1) {
                    RtlmUiUtil.showServerErrorMessage(MessageUtil.getMessage(MessageBundle.COMMON, "serverLicenseErrMsg"));                  
                }
                this.showStatusLine(RtlmUiUtil.getMessage("reqMsgRequestActionFailed"), ViewHeaderLayout.ERROR);
            }
            this.invokedAction = null;
        },
        
        // Summary:
        //     Prepares the UI to start finding a machine based on the specified
        //     configuration parameters. This creates a progress bar in the 
        //     find machine action area. When the action is completed, the progress
        //     bar needs to be removed.
        //
        _doFindMachines: function(action){
            var message = this.findMachineProgressMsgNode;
            message.innerHTML = RtlmUiUtil.getMessage("MsgRequestMachineSearch");
            dojo.style(this.findMachineProgressNode, "visibility", "visible");
            
            action.setDisable(true);
            
            // clear existing list first, if any.
            var field = this._getFieldObjectByAttributeName(this.FOUND_MACHINE_ATTR);
            field.setFieldValue(null);
        },
        
        
        // Summary:
        //    Gets the data from the UI fields.
        //
        _getFormData: function(){
            var reqData = {};
            
            // Fill in the request data that's not visible/changeable by the user.
            // 
            if (this.isNew) {
                reqData[this.RECIPIENT_ATTR] = this._requestData.getCreatorId();
            }
            else {
                reqData[this.STATE_ID_ATTR] = this._requestData.getAttributeValueByName(this.STATE_ID_ATTR);
                
                // TODO: requestParam uses a different name for the id... change it to a 
                // uniform name, when the backend is done.
                reqData["requestUuid"] = this._requestData.getAttributeValueByName(this.REQ_ID_ATTR);
            }
            
            for (var i = 0; i < this._fieldObjects.length; i++) {
                var field = this._fieldObjects[i];
                
                // skip over fields that don't submit data and 
                // don't have data value.
                if (!field.canFieldSubmit() || field.getFieldValue() == null) 
                    continue;
                
                // convert our internal UI configuration object to a configuration string.
                if (field.containAttribute(this.CONFIGURATION_ATTR)) {
                    var config = field.getFieldValue();
                    reqData[this.CONFIGURATION_ATTR] = config.toJson();
                    continue;
                }
                
                // for fields that involves users, we have to map
                // the display name back to its id.
                // 
                var value = field.getFieldValue();
                if (field.containAttribute(this.ASSIGNED_ATTR)) {
                    if (value) 
                        value = this._getUserIdByName(value);
                }
                if (field.containAttribute(this.CREATED_BY_ATTR)) {
                    value = this._requestData.getCreatorId();
                }
                
                if (field.containAttribute(this.RESERVE_MACHINE_ID_ATTR)) 
				{
					reqData[this.RESERVE_MACHINE_ID_ATTR] = value;
					reqData[this.RESERVE_ASSET_TYPE] = "Machine";  //MFV SHOULD NOT BE HARD CODED
				}
                if (field.containAttribute(this.CONFIRM_BY_ATTR) ||
                    field.containAttribute(this.RESERVE_FROM_ATTR) ||
                    field.containAttribute(this.RESERVE_TO_ATTR)) {
                    // Do nothing, we handle dates below.
                }
				else 
					reqData[field.getPrimaryAttributeName()] = value;
            }
			//get the proper date time
			if (this._reserveFromDate) {
                reqData.reserveFrom = this._reserveFromDate.getTime();
            }
			if (this._reserveToDate) {
                 reqData.reserveTo = this._reserveToDate.getTime();
            }
			if (this._confirmByDate) {
                reqData.confirmationDate = this._confirmByDate.getTime();
            }
            return reqData;
        },
        
        // Summary:
        //    UI only action to reset the current request form.
        //
        _doReset: function(){
            // confirm with the user first, if there're values entered.
            if (this.isEdited()) {
                if (!confirm(RtlmUiUtil.getMessage("MsgClearEditing"))) 
                    return;
            }
            this._resetEditIndicators();
            this.clearStatusLine();
            
            this._isLoading = true;
            this._loadRequestDataToPage();
            this._checkActionButtonEnablement();
            this._isLoading = false;
        },
        
        _checkHasWarning: function(){
            var field = this._getFieldObjectByAttributeName(this.CONFIGURATION_ATTR);
            var warnMsg = field.getWarning();
            if (warnMsg != null && warnMsg.length > 0) {
                this._warningMsg = warnMsg;
                return true;
            }
            return false;
        },
        
        // Summary:
        //    Checks for valid data values before allowing the user to commit the
        //    change.
        _checkValidInput: function(){
            if (this._formHasErrors()) 
                return false;
            
            // check the date fields to ensure that it's not in the past. But only
            // need to check it if it has been edited.            
            if (this._isInEditedList(this.RESERVE_FROM_ID) > -1 &&
            this._reserveFromDate &&
            this._isDateInPast(this._reserveFromDate)) {
                this._invalidInputMsg = RtlmUiUtil.getMessage("reqMsgInvalidFromDate");
                return false;
            }
            if (this._isInEditedList(this.RESERVE_TO_ID) > -1 &&
            this._reserveToDate &&
            this._isDateInPast(this._reserveToDate)) {
                this._invalidInputMsg = RtlmUiUtil.getMessage("reqMsgInvalidToDate");
                return false;
            }
			if((this._isInEditedList(this.RESERVE_FROM_ID) > -1 || this._isInEditedList(this.RESERVE_TO_ID) > -1) &&
				this._reserveFromDate.getTime() >= this._reserveToDate.getTime()) {					
                this._invalidInputMsg = RtlmUiUtil.getMessage("reqMsgNegativeFromToDate");
                return false;
			}
            if (this._isInEditedList(this.CONFIRM_BY_ID) > -1 &&
            this._confirmByDate &&
            this._isDateInPast(this._confirmByDate)) {
                this._invalidInputMsg = RtlmUiUtil.getMessage("reqMsgInvalidConfirmDate");
                return false;
            }
            
            // check the date range, the from date can't be greater then the to date.
            if (this._reserveFromDate && this._reserveToDate &&
            this._reserveFromDate.getTime() > this._reserveToDate.getTime()) {
                this._invalidInputMsg = RtlmUiUtil.getMessage("reqMsgInvalidDateRange");
                return false;
            }
            
            return true;
        },
        
        // Summary:
        //    Returns true if there're errors in any of the input fields.
        //
        _formHasErrors: function(){
            for (var i = 0; i < this._fieldObjects.length; i++) {
                var err = this._fieldObjects[i].hasError();
                if (err) {
                    this._invalidInputMsg = err;
                    return true;
                }
            }
            return false;
        },
        
        // Summary:
        //    Returns an invalid input message. A specific error message is returned
        //    if the member attribute "_invalidInputMsg" is set. Otherwise, a generic
        //    invalid input message is returned.
        //
        _getInvalidInputMessage: function(){
            if (this._invalidInputMsg == null) {
                return RtlmUiUtil.getMessage("MsgInvalidInput");
            }
            return this._invalidInputMsg;
        },
        
        _showServerErrors: function(errorType){
            if (errorType != null && typeof(errorType.message) != "undefined" && errorType.message != null) 
                RtlmUiUtil.showServerErrorMessage(errorType.message);
        },
        
        _getFieldObjectByAttributeName: function(attrName){
            var field = null;
            
            for (var i = 0; i < this._fieldObjects.length; i++) {
                if (this._fieldObjects[i].containAttribute(attrName)) {
                    field = this._fieldObjects[i];
                    break;
                }
            }
            return field;
        },
        
        _getFieldObjectByNodeId: function(id){
            var field = null;
            
            for (var i = 0; i < this._fieldObjects.length; i++) {
                var nodeId = this._fieldObjects[i].getNodeId();
                
                if (id == nodeId) {
                    field = this._fieldObjects[i];
                    break;
                }
            }
            return field;
        },
        
        _getValueByAttributeName: function(attrName){
            var field = this._getFieldObjectByAttributeName(attrName);
            
            if (field) {
                return field.getFieldValue();
            }
            return null;
        },
        
        _getUserIdByName: function(name){
            if (this._userList) {
                for (var i = 0; i < this._userList.length; i++) {
                    if (this._userList[i].name == name) 
                        return this._userList[i].itemId;
                }
            }
            return null;
        },
        
        // Summary:
        //    Returns whether the action's required input have been entered.
        //
        _checkHasActionRequiredInputs: function(/* actionWrapper */action){
            // this action is not active. 
            if (!action.isActionActive()) 
                return false;
            
            var required = action.getActionInfo().requiredData;
            
            if (required && required.length) {
                for (var i = 0; i < required.length; i++) {
                    var val = this._getValueByAttributeName(required[i]);
                    
                    if (val == null || (dojo.isString(val) && val.length == 0)) {
                        return false;
                    }
                }
                
                // if the action requires edits to be enabled. check
                // the form for editing, before enable it.
                if (action.isRequireEditToEnable()) 
                    return this.isEdited();
                
                return true;
            }
            // if there's no require fields, make edited form
            // the require enablement.
            return !action.isRequireEditToEnable() || this.isEdited();
        },
        
        // Summary:
        //    Enable/disable our action buttons based on the current form's edited
        //    and/or fields' value statuses.
        //
        _checkActionButtonEnablement: function(){
            for (var i = 0; i < this._actions.length; i++) {
                if (this._actions[i].isActionActive()) {
                    var hasRequired = this._checkHasActionRequiredInputs(this._actions[i]);
                    this._actions[i].setDisable(!hasRequired);
                }
            }
        },
        
        // Summary:
        //     Loads data into the request form's fields. 
        //
        _loadRequestDataToPage: function(){
            // instruct the field objects to load the 
            // values.
            for (var i = 0; i < this._fieldObjects.length; i++) {
                this._fieldObjects[i].loadValueToFieldUI();
            }
            this._showPriorityIcon(this._requestData.getPriority());
        },
        
        _showPriorityIcon: function(priority){
            var iconMap = {
                'High': 'priority_high_obj.gif',
                'Normal': 'priority_med_obj.gif',
                'Low': 'priority_low_obj.gif'
            };
            
            var iconName = iconMap[priority];
            
            // set a new image path for the icon, or
            // hide it. 
            if (iconName) {
                dojo.style(this.priorityIconNode, "visibility", "visible");
                this.priorityIconNode.src = dojo.moduleUrl("com.ibm.rational.test.lm.web", "ui/internal/images/icons/obj16/" + iconName);
            }
            else {
                dojo.style(this.priorityIconNode, "visibility", "hidden");
            }
        },
        
        // Summary
        // If certain fields shouldn't be decorated with the editing flag, this method should
        // return true. By default, all fields are not excluded. Subclasses need to override this
        // method to exclude certain fields to be decorated when edited.
        // 
        _excludeEditIndicator: function(w_id){
            // there's no edited decoration for the machine
            // config area. Ignore it.
            if (w_id.indexOf(this.CONFIGURATION_ID) == 0) 
                return true;
            return false;
        },
        
        // Summary
        // Determines whether the field value has actually changed. 
        //   
        _isFieldChanged: function(w_id){
            // if the config is area reporting changes,
            // it has added complete param.
            if (w_id.indexOf(this.CONFIGURATION_ID) == 0) {
                var field = this._getFieldObjectByNodeId(w_id);
                return field.isEdited();
            }
            
            // this is a new request, use the parent class's default empty form behavior 
            if (this.isNew) {
                return com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestFormView.superclass._isFieldChanged.apply(this, arguments);
            }
            
            var node = dojo.byId(w_id);
            var val = this.getValueById(w_id);
            var time = null;
            
            if (w_id == this.RESERVE_FROM_ID) {
                var oldVal = this._requestData.getReserveFromDate();
                if (oldVal != null) 
                    time = oldVal.getTime();
                return val != time;
            }
            else 
                if (w_id == this.RESERVE_TO_ID) {
                    var oldVal = this._requestData.getReserveToDate();
                    if (oldVal != null) 
                        time = oldVal.getTime();
                    return val != time;
                }
                else 
                    if (w_id == this.CONFIRM_BY_ID) {
                        var oldVal = this._requestData.getConfirmByDate();
                        if (oldVal != null) 
                            time = oldVal.getTime();
                        return val != time;
                    }
            
            var curVal = this._requestData.getAttributeValueByName(node.name);
            
            // normalize between a null and an empty value before we compare.
            if ((typeof(val) == "undefined") || val == null || val.length == 0) 
                val = null;
            
            if ((typeof(curVal) == "undefined") || curVal == null || curVal.length == 0) 
                curVal = null;
            
            return (val != curVal);
        },
        
        loadAttributeValues: function(w_id){
            this._updateFormEditedState(w_id);
            
            // if the priority changes, change its icon as well.
            if (w_id == this.PRIORITY_ID) {
                var priority = this.getValueById(this.PRIORITY_ID);
                this._showPriorityIcon(priority);
            }
        },
        
        textFieldValueChanged: function(field){
            var w_id = field.nodeId;
            this._updateFormEditedState(w_id);
            
            // save a copy of the date. It is used for validation.
			var timePickerId = field.timePickerId;
            if (w_id == this.RESERVE_FROM_ID) {
				this._reserveFromDate = dijit.byId(w_id).getValue();
				this._reserveFromDate = this.addTimeToDate(this._reserveFromDate, timePickerId);
			}
			else 
				if (w_id == this.RESERVE_TO_ID) {
					this._reserveToDate = dijit.byId(w_id).getValue();
					this._reserveToDate = this.addTimeToDate(this._reserveToDate, timePickerId);
				}
				else 
					if (w_id == this.CONFIRM_BY_ID) {
						this._confirmByDate = dijit.byId(w_id).getValue();
						this._confirmByDate = this.addTimeToDate(this._confirmByDate, timePickerId);
				}
        },
		
		addTimeToDate: function(dateObject, timePickerId) {
			var time = dijit.byId(timePickerId).getValue();
			if (dateObject && time) {
				dateObject.setHours(time.getHours());
				dateObject.setMinutes(time.getMinutes());
				dateObject.setSeconds(time.getSeconds());
			}
			return dateObject;
		},
        
        valueChanged: function(w_id){
            var node = dojo.byId(w_id);
            this._updateEditList(w_id, true);
            this._updateFormEditedState(w_id);
            this._checkActionButtonEnablement();
        },
        
        _getFieldDefaultClassName: function(w_id){
            if (w_id == this.DESCRIPTION_ID) 
                return "RequestFormText";
            
            var field = dijit.byId(w_id);
            if (field && field instanceof dijit.form.TextBox) 
                return "RequestFormField";
            
            if (dojo.isFF) 
                return "RequestFormDropDownFieldFF";
            else 
                return "RequestFormDropDownField";
        },
        
        _getRequestCSHelpId: function(){
            if (this.isNew) 
                return "rtlm_new_request";
            else {
                if (this._hasRequestAdministerPermission()) 
                    return "rtlm_request_details_labmgr";
                else 
                    return "rtlm_request_details";
            }
        },
        
        _hasRequestAdministerPermission: function()
        {
            var perm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_REQUEST);
            return (!perm || perm[PermissionsClient.ACTION_ADMINISTER]);           
        },
    
        // Summary:
        //    Check the date to see if it's in the past. Returns true if the date
        //    is less then today's date.
        //
        _isDateInPast: function(/* a date to check */date){
            var todayDate = new Date();
            todayDate.setHours(0, 0, 0, 0);
            
            // test the date against today's date. If we test the date against the
            // create date of the request, that date could be in the past.
            return (date.getTime() < todayDate.getTime());
        },
        
        // Summary:
        //    Called by the popup calendar to "cross out" invalid selection dates. 
        //
        _disableCalendarDate: function(dateId, date){
            // "cross out" all of the past dates.
            if (this._isDateInPast(date)) 
                return true;
            
            if (dateId == this.RESERVE_FROM_ID) {
                return (this._reserveToDate && dojo.date.compare(date, this._reserveToDate) > 0);
            }
            else 
                if (dateId == this.RESERVE_TO_ID) {
                    return (this._reserveFromDate && dojo.date.compare(date, this._reserveFromDate) < 0);
                }
            return false;
        },
        
        // Summary:
        //    Store the date value into our scratch pad members. These values are used for
        //    validating date valeus in the form.
        //
        _setDateField: function(dateId, date){
            if (dateId == this.RESERVE_FROM_ID) {
                this._reserveFromDate = date;
            }
            else 
                if (dateId == this.RESERVE_TO_ID) {
                    this._reserveToDate = date;
                }
                else 
                    if (dateId == this.CONFIRM_BY_ID) {
                        this._confirmByDate = date;
                    }
        },
        
        // Summary:
        //    Pops up the calendar for a date field. When the user selects a date
        //    in the calendar, the value is entered to the field.
        //
        showCalendar: function(field){
            var dateBox = field;
            
            if (!this._picker) {
                var popupProto = dojo.getObject("dijit._Calendar", false);
                this._picker = new popupProto({
                    _currentDateBox: null,
                    _dateBoxId: null,
                    _parent: this,
                    onValueSelected: function(date){
                        if (this._currentDateBox) {
                            this._currentDateBox.focus();
                            this._currentDateBox.setValue(date);
                            this._parent._updateFormEditedState(this._currentDateBox.id);
                        }
                        dijit.popup.close(this);
                    },
                    lang: dateBox.lang,
                    constraints: dateBox.constraints,
                    isDisabledDate: function(/*Date*/date){
                        return this._parent._disableCalendarDate(this._dateBoxId, date);
                    }
                });
                
                // constraint the size of the calendar. Otherwise, it'll take
                // up the rest of the space of the page.
                dojo.style(this._picker.domNode, "width", "180px");
                dojo.style(this._picker.domNode, "height", "200px");
            }
            this._picker._currentDateBox = dateBox;
            this._picker._dateBoxId = dateBox.id;
            this._picker.setValue(dateBox.getValue() || new Date());
            
            // put the focus to the date box. This will allow
            // the popup to pop down when the user click elsewhere
            // in the form.
            dateBox.focus();
            
            dijit.popup.open({
                parent: dateBox,
                popup: this._picker,
                around: dateBox.domNode,
                onClose: function(){/* do nothing */
                }
            });
        },
        
        clearStatusLine: function(){
            this.viewHeaderNode.clearStatus();
        },
        
        showStatusLine: function(message, status){
            this.viewHeaderNode.setStatus(message, status);
        },
        
        exitToMachinePage: function(machineId)
		{
			var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
			var assetUri = AssetUriUtil.getViewAssetDetailURI("Machine", machineId);
			window.location.href = assetUri;
        },
        
        loadMachinePanel: function(){
        },
        
        // Summary
        //    Exit the request form and return to the request query selection view.
        //
        exitPage: function(){
            // instruct the query page to refresh the query results and remove the view for this request id.
            window.location.href = UriUtil.getQueryRequestURI({
                refresh: true,
                removeViewId: this.requestId
            });
        },
        
        REQUEST_TITLE_ID: "request_title_text",
        
        // Defines the button node ids in HTML
        CANCEL_BUTTON_ID: "cancel_button",
        RESET_BUTTON_ID: "reset_button",
        SAVE_BUTTON_ID: "save_button",
        SUBMIT_BUTTON_ID: "submit_button",
        CLOSE_BUTTON_ID: "close_button",
        REOPEN_BUTTON_ID: "reopen_button",
        FULFILL_BUTTON_ID: "fulfill_button",
        RETURN_BUTTON_ID: "return_button",
        BUTTON_AREA_ID: "bottom_button_area",
        
        // Defines the field node ids in HTML
        // Note:  These strings are NOT static.  They will get changed to
        // the new id that dojo creates for the widget.
        SUMMARY_ID: "text_summary",
        ASSIGNED_ID: "field_assigned",
        PRIORITY_ID: "combo_request_priority",
        CREATED_ON_ID: "field_created_on",
        CREATED_BY_ID: "field_created_by",
        CONFIRM_BY_ID: "text_confirm_by",
        RESERVE_FROM_ID: "text_fromdate",
        RESERVE_TO_ID: "text_todate",
		TIME_ID_SUFFIX: "_time",
        DESCRIPTION_ID: "text_description",
        NOTIFY_REQ_ID: "label_notify_requester",
        NOTIFY_REQ_REC_ID: "label_notify_req_recipient",
        NOTIFY_OTHER_ID: "combo_notify_other",
        STATE_DISPLAY_ID: "state_display",
        CONFIGURATION_AERA_ID: "configuration_panel_cell",
        CONFIGURATION_ID: "configuration_panel",
        FIND_MACHINE_BUTTON_ID: "reqLabel_FindMachines",
        FOUND_MACHINE_LIST_ID: "machine_list",
        RESERVE_MACHINE_BUTTON_ID: "reqLabel_reserveMachine",
        RESERVATION_LABEL_ID: "reqLabel_ReservedMachines",
        
        // Defines the backend attribute names
        SUMMARY_ATTR: "summary",
        STATE_ATTR: "currentState",
        CREATED_ON_ATTR: "createDate",
        CREATED_BY_ATTR: "requester",
        DESCRIPTION_ATTR: "description",
        PRIORITY_ATTR: "priority",
        CONFIRM_BY_ATTR: "confirmationDate",
        RESERVE_FROM_ATTR: "reserveFrom",
        RESERVE_TO_ATTR: "reserveTo",
        ASSIGNED_ATTR: "fulfiller",
        CREATE_DATE_ATTR: "createDate",
        STATE_ID_ATTR: "stateId",
        REQ_ID_ATTR: "id",
        STATE_ATTR: "currentRequestState",
        RECIPIENT_ATTR: "recipient",
        CONFIGURATION_ATTR: "configuration",
        FOUND_MACHINE_ATTR: "foundMachines",
        RESERVE_MACHINE_ID_ATTR: "reserveMachineId",
		RESERVE_ASSET_TYPE: "reserveAssetType",
        RESERVATION_ATTR: "reservations",
        DIRECTIVE_ATTR: "directive",
        PROJECT_AREA_ATTR: "projectAreaItemId",
        
        // Defines the backend action names.
        RESET_ACTION_ID: "reset",
        SAVE_ACTION_ID: "save",
        SUBMIT_ACTION_ID: "submit",
        FULFILL_ACTION_ID: "fulfill",
        RETURN_ACTION_ID: "return",
        REOPEN_ACTION_ID: "reopen",
        CLOSE_ACTION_ID: "close",
        FIND_MACHINE_ACTION_ID: "findMachine",
        RESERVE_MACHINE_ACTION_ID: "reserveMachine",
        CANCEL_ACTION_ID: "cancel"
    });
    
})();
