/**RequestView**************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//
//
// Author: L. To
//
// Summary:
//
//    The RequestView implements a RTLM request form. It provides data entries and displays
//    for an RTLM request. An RTLM request has a state associated with it and depends on its state,
//    certain fields can be edited, or become read-only. This information is driven from the request
//    JSON data. This form view configures the data entry fields accordingly. 
//
//    The consumer of the ReqyestView needs to initialize the form either with a call to the method: 
//    setRequestItem(id, isNew), or specify them via the constructor parameters. The form widget would not 
//    complete its loading until this method is called, or the proper parameter is set in the constructor
//    as follows:
//
//        formArg = {requestId: id, isNew: true};
//        var requestView = new com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestView(formArg);
//
//    The RequestView object uses several internal objects to manage its contents. They are the 
//    DataObject and the ActionWrapper. See the internal object's summaries for more details.
//    
//
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Configuration");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.TimePicker");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.SimpleTimeSelector");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.rational.test.lm.web.client.internal.request");
dojo.require("com.ibm.rational.test.lm.web.client.internal.user");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.ConfigurationDisplay");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil");

//Table viewers for found machines and reserved machines
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestReservationsTableProvider"); 
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestReservedMachinesTableProvider"); 

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var LOADING = RtlmUiUtil.getMessage("appWorking");
var UriUtil = com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestUriUtil;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var ConfigDisplay = com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.ConfigurationDisplay;
var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;

// Summary:
//    Internal wrapper object around the request JSON data. It provides the necessary data conversion
//    from the JSON to script expected values, such as from a time stamp string to a Date object.
//
dojo.declare("Request.DataObject", null, {
    data: null,
    requestForm: null,
    config: null,
    
    constructor: function(requestForm, requestData){
        this.requestForm = requestForm;
        this.data = requestData;
    },
    
    getTitle: function(){
        if (this.data.visibleId) 
            return RtlmUiUtil.getMessage("reqLabel_request") + " " + this.data.visibleId;
        return RtlmUiUtil.getMessage("reqLabel_new_request", this.requestForm.requestId);
    },
    
    getSummary: function(){
        if (this.data.summary) 
            return this.data.summary;
        return null;
    },
    getCreateDate: function(){
        if (this.data.createDate && this.data.createDate.length) {
            return RtlmUiUtil.convertTimeStampToDate(this.data.createDate);
        }
        return new Date();
    },
    getCreator: function(){
        if (this.data.requester) 
            return this.data.requester.name;
        return this.requestForm.currentUser ? this.requestForm.currentUser.name : "";
    },
    getCreatorId: function(){
        if (this.data.requester) 
            return this.data.requester.id;
        return this.requestForm.currentUser ? this.requestForm.currentUser.itemId : null;
    },
    getPriority: function(){
        if (this.data.priority) 
            return this.data.priority;
        return null;
    },
    getPriorityDisplay: function(){
        if (this.data.priority) 
            return RtlmUiUtil.getMessage(this.data.priority);
        return "";
    },
    getPriorityValues: function(){
        var values = [];
        if (this.data.priorityValues) {
            for (var i = 0; i < this.data.priorityValues.length; i++) {
                var priority = {};
                priority['name'] = RtlmUiUtil.getMessage(this.data.priorityValues[i]);
                priority['value'] = this.data.priorityValues[i];
                values.push(priority);
            }
        } 
        return values;
    },
    getAssignee: function(){
        if (this.data.fulfiller) {
            return this.data.fulfiller.name;
        }
        return null;
    },
    getConfirmByDate: function(){
        if (this.data.confirmationDate && this.data.confirmationDate.length) {
            return RtlmUiUtil.convertTimeStampToDate(this.data.confirmationDate);
        }
        return null;
    },
    getConfirmByDateDisplay: function(){
        if (this.data.confirmationDate && this.data.confirmationDate.length) {
            return RtlmUiUtil.getDateDisplay(this.data.confirmationDate, true);
        }
        return "";
    },
    getDescription: function(){
        if (this.data.description) 
            return this.data.description;
        return null;
    },
    getReserveFromDate: function(){
        if (this.data.reserveFrom && this.data.reserveFrom.length) {
            return RtlmUiUtil.convertTimeStampToDate(this.data.reserveFrom);
        }
        return null;
    },
    getReserveFromDateDisplay: function(){
        var date = this.getReserveFromDate();
        return (date) ? RtlmUiUtil.getDisplayDateTimeString(date) : "";
    },
    getReserveToDate: function(){
        if (this.data.reserveTo && this.data.reserveTo.length) {
            return RtlmUiUtil.convertTimeStampToDate(this.data.reserveTo);
        }
        return null;
    },
    getReserveToDateDisplay: function(){
        var date = this.getReserveToDate();
        return (date) ? RtlmUiUtil.getDisplayDateTimeString(date) : ""; ;
    },
    getCurrentState: function(){
        if (this.data.currentState) 
            return this.data.currentState;
        return null;
    },
    getCurrentStateDisplay: function(){
        if (this.data.currentState) 
            return RtlmUiUtil.getMessage(this.data.currentState);
        return null;
    },
    getAttributeValueByName: function(attrName){
        return this.data[attrName];
    },
    getConfiguration: function() {
        if (!this.config) {
            var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
            var configStr = this.data.configuration;
            
            if (configStr && configStr.length > 1) 
                this.config = catalog.createConfigurationFromJson(configStr);
        }
        return this.config;
    },
    getConfigString: function()
    {
        return this.data.configuration;
    },
    getFoundMachines: function(){
        return this.data.foundMachines;
    },
    getReservedMachines: function(){
        return this.data.reservations;
    },
    
    // Summary:
    //    Given an attribute name, returns whether the attribute is editable in the
    //    current request state context.
    // 
    isFieldEditable: function(attrName){
        if (this.data.editableData) {
            for (var i = 0; i < this.data.editableData.length; i++) {
                if (attrName == this.data.editableData[i]) 
                    return true;
            }
        }
        return false;
    },
    getActionInfo: function(actionName){
        if (this.data.actionInfos) {
            for (var i = 0; i < this.data.actionInfos.length; i++) {
                if (actionName == this.data.actionInfos[i].action) 
                    return this.data.actionInfos[i];
            }
        }
        return null;
    }
});

// Summary:
//    Internal wrapper object over request form actions. 
//    
dojo.declare("Request.ActionWrapper", null, {
    reqForm: null,
    actionId: null,
    nodeId: null,
    label: null,
    isCreated: false,
    isActive: true,
    isVisible: true,
    isDisable: true,
    actionInfo: null,
    actionUiNodes: null,
    requireEditToEnable: false,
    canEnableWithInvalidInput: false,
    
    // Summary:
    //    Constructs an action wrapper. 
    
    constructor: function(reqForm, nodeId, actionInfo, isCreated, isVisible){
        this.reqForm = reqForm;
        this.nodeId = nodeId;
        this.actionInfo = actionInfo;
        this.isCreated = isCreated;
        this.isActive = (this.actionInfo != null);
        this.actionId = (this.actionInfo != null) ? this.actionInfo.action : nodeId;
        
        // if the caller specified the "isVisible" flag, honor it.
        if (typeof(isVisible) != "undefined") 
            this.isVisible = isVisible;
        
        this.configureActionUI();
    },
    
    getId: function(){
        return this.actionId;
    },
    
    configureActionUI: function(){
        this.actionUiNodes = [];
        
        var node = null;
        
        // either create or attach the node.
        if (this.isCreated) {
            node = this.nodeId;
        }
        else 
            if (this.isActive) {
                var buttonArea = this.reqForm.buttonAreaRootNode;
                this.label = RtlmUiUtil.getMessage(this.nodeId);
                node = document.createElement("button");
                node.id = this.nodeId;
                node.innerHTML = this.label;
                buttonArea.appendChild(node);
            }
        
        if (node) {
            var visible = (this.isActive && this.isVisible) ? "visible" : "hidden";
            
            node.disabled = this.isDisable;
            node.object = this;
            dojo.style(node, "visibility", visible);
            node.onclick = function(){
                this.object.dispatch(this);
            };
            
            this.actionUiNodes.push(node);
        }
    },
    
    // Summary:
    //    Internal method called by the action's UI node (button, etc.) when 
    //    it's activated. It dispatches to the request form's invokeAction for
    //    processing.
    //
    dispatch: function(button){
        // For a link element ("a"), it doesn't have
        // disabled attribute. Implements the disabled-ness
        // here.
        if (this.isDisable) 
            return;
        
        this.reqForm._invokeAction(this, button);
    },
    
    // Summary:
    //     Returns the action's attributes. 
    //
    getActionInfo: function(){
        return this.actionInfo;
    },
    
    isPageTransition: function(){
        return this.actionInfo && this.actionInfo.pageTransition;
    },
    
    // Summary:
    //    Returns true if the action has actionInfo; this means the action is valid
    //    for the state of the request and the action will actually has a UI to invoke such 
    //    action. If the action doesn't have a actionInfo record, this method returns false.
    //
    isActionActive: function(){
        return this.isActive;
    },
    
    // Summary:
    //    Set the action's UI visibility attribute. It's only visible if this action is active
    //    and it's visible.
    //
    setVisible: function(visible){
        this.isVisible = visible;
        
        var visible = (this.isActive && visible) ? "visible" : "hidden";
        for (var i = 0; i < this.actionUiNodes.length; i++) 
            dojo.style(this.actionUiNodes[i], "visibility", visible);
    },
    
    // Summary:
    //    Set the action's UI enable state. 
    //
    setDisable: function(disable){
        this.isDisable = disable;
        
        for (var i = 0; i < this.actionUiNodes.length; i++) {
            var node = this.actionUiNodes[i];
            node.disabled = disable;
            
            // for action links, change its class to indicate
            // enable/disablement.
            if (node.nodeName && node.nodeName == "A") {
                var style = (this.isDisable) ? "formActionLinkDisabled" : "formActionLink";
                node.className = style;
            }
        }
    },
    
    // Summary:
    //    If the action doesn't have any required field, it will be enabled automatically.
    //    Use this method to only enable the action when the form has been edited. By default,
    //    the ActionWrapper does NOT require edit to be enabled.
    // 
    setIsRequireEditToEnable: function(requireEdit){
         this.requireEditToEnable = requireEdit;
    },
    
    // Summary:
    //    If the action doesn't have any required field, it will be enabled automatically.
    //    This indicates whether the action can be enabled if there's no editing in the form.
    // 
    isRequireEditToEnable: function(){
        return this.requireEditToEnable;
    },
    
    setCanEnableWithInvaildInputs: function(canEnable) {
        this.canEnableWithInvalidInput = canEnable;
    },
    
    canEnableWithInvalidInputs: function() {
        return this.canEnableWithInvalidInput;
    }
});
    

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.requests.RequestView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="request-detail-view" dojoAttachPoint="domNode" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" titleEditable="false" dojoAttachPoint="viewHeaderNode">' +
                    '    <div dojoType="com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection" dojoAttachPoint="reserveAreaNode" open="true">'+
                    '        <table class="header-attr-table">' +
                    '          <tr>' +
                    '            <td dojoAttachPoint="fromLabelNode" class="attr-label">${fromLabel}' +
                    '            </td>' +
                    '            <td dojoAttachPoint="reserveFromNode">' +
                    '            </td>' +
                    '          </tr>' +
                    '          <tr>' +
                    '            <td dojoAttachPoint="toLabelNode" class="attr-label">${toLabel}' +
                    '            </td>' +
                    '            <td dojoAttachPoint="reserveToNode">' +
                    '            </td>' +
                    '          </tr>' +
                    '        </table>' +
                    '    </div>' +
                    '    <div dojoType="com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection" dojoAttachPoint="configSectionNode" open="true">'+
                    '      <div class="config-display-area" dojoAttachPoint="configDisplayNode">'+
                    '      </div>' +
                    '      <div dojoAttachPoint="findResultNode" style="display:none">'+
                    '        <div dojoType="com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer" dojoAttachPoint="findMachineTableNode">'+
                    '        </div>' +
                    '      </div>' +
                    '      <div class="reserved-machine-area" dojoAttachPoint="reserveResultNode" style="display:none">'+
                    '        <div>${reservedMachineLabel}</div>'+
                    '        <div dojoType="com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer" '+
                    '             showTableControls="false" showPagingControl="false" dojoAttachPoint="reservedMachineTableNode">'+
                    '        </div>' +
                    '      </div>' +
                    '    </div>' +
                    '    <div dojoType="com.ibm.asq.common.web.ui.internal.view.common.CollapsibleSection" dojoAttachPoint="commentAreaNode" open="true">'+
                    '      <textarea rows="3" class="text-area" wrap="soft" value="" dojoAttachPoint="descriptionNode" ' +
                    '       dojoAttachEvent="onchange:valueChanged,onkeyup:valueChanged">' +
                    '      </textarea>' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    widgetsInTemplate: true,
    
    // form display label variable section 
    statusLabel: RtlmUiUtil.getMessage("reqLabel_status"),
    createOnLabel: RtlmUiUtil.getMessage("reqLabel_created_on"),
    createdByLabel: RtlmUiUtil.getMessage("reqLabel_created_by"),
    assignedLabel: RtlmUiUtil.getMessage("reqLabel_assigned"),
    priorityLabel: RtlmUiUtil.getMessage("reqLabel_priority"),
    confirmByLabel: RtlmUiUtil.getMessage("reqLabel_confirm_by"),
    descriptionLabel: RtlmUiUtil.getMessage("reqLabel_comments"),
    reservationLabel: RtlmUiUtil.getMessage("reqLabel_reservation"),
    fromLabel: RtlmUiUtil.getMessage("reqLabel_from"),
    toLabel: RtlmUiUtil.getMessage("reqLabel_to"),
    findMachineLabel: RtlmUiUtil.getMessage("reqLabel_FindMachines"),
    reservedMachineLabel: RtlmUiUtil.getMessage("reqLabel_ReservedMachines"),
    reserveMachineLabel: RtlmUiUtil.getMessage("reqLabel_reserveMachine"),
    summaryLabel: RtlmUiUtil.getMessage("reqLabel_summary"),
    configLabel: RtlmUiUtil.getMessage("reqLabel_config"),
    summaryPrompt: RtlmUiUtil.getMessage("reqLabel_summaryPrompt"),
    
    reserveSectInfoText: RtlmUiUtil.getMessage("reqLabel_reserveInfoText"),
    configSectInfoText: RtlmUiUtil.getMessage("reqLabel_configInfoText"),
    configSectSearchInfoText: RtlmUiUtil.getMessage("reqLabel_configInfoSearchText"),
    commentSectInfoText: RtlmUiUtil.getMessage("reqLabel_commentInfoText"),
    
    isNew: false,
    requestId: null,
    configId: null,
    tabIndex: 1,
    _requestData: null,
    currentUser: null,
    invalidInputMsg: null,
    warningMsg: null,
    fields: null,
    _isEdited: false,
    
    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        
        if (!this.currentUser) {
            this.currentUser = com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR;
        }
        
        // set the CS help id based on permissions
        var reqAdminPermission = this._hasRequestAdministerPermission();
		var csHelpId = this.isNew ? "rtlm_new_request.html" : reqAdminPermission ? "rtlm_view_requests_labmgr.html" : "rtlm_request_details.html";
        this.viewHeaderNode.setCSHelpId(csHelpId, com.ibm.rational.test.lm.web.CSHelpID);
		var label = RtlmUiUtil.getMessage("labelHelp", RtlmUiUtil.getMessage("labelNew"), RtlmUiUtil.getMessage("reqLabel_request"));
		if(!this.isNew)
			label = reqAdminPermission ? RtlmUiUtil.getMessage("labelHelp", RtlmUiUtil.getMessage("reqLabel_request"), RtlmUiUtil.getMessage("labelManager"))
				: RtlmUiUtil.getMessage("labelHelp", RtlmUiUtil.getMessage("reqLabel_request"), RtlmUiUtil.getMessage("labelDetail"));;
	    this.viewHeaderNode.setCSHelpButtonLabel(label);
		
        this.viewHeaderNode.setIconClass("request-icon-48");
        this.viewHeaderNode.setTypeLabel(RtlmUiUtil.getMessage("reqLabel_request"));
		this.viewHeaderNode.setNoValueEditableSummaryLabel(this.summaryPrompt);
		this.viewHeaderNode.setSummaryLabel(RtlmUiUtil.getMessage("reqLabel_summary_editable"));
        this.viewHeaderNode.onSummaryEdited = dojo.hitch(this, '_onSummaryChanged');
		if (dojo.isFF == 2) {
			this.viewHeaderNode.getSummaryNode().className = "";
			this.viewHeaderNode.getSummaryNode().style.marginLeft = "12px";
		}
		
        // set the button area root node to the view header right area
        this.buttonAreaRootNode = this.viewHeaderNode.getRightActionsContainer();

        this.reserveAreaNode.setTitle(this.reservationLabel);
        this.reserveAreaNode.summaryNode.innerHTML = this.reserveSectInfoText;
        
        this.configSectionNode.setTitle(this.configLabel);

        this.commentAreaNode.setTitle(this.descriptionLabel);
        this.commentAreaNode.summaryNode.innerHTML = this.commentSectInfoText;
		this.fromLabelNode.style.display = "none";
		this.toLabelNode.style.display = "none";
        
        this.fields = {};
        this._fetchData();
    },

    _hasRequestAdministerPermission: function()
    {
        var perm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_REQUEST);
        return (!perm || perm[PermissionsClient.ACTION_ADMINISTER]);           
    },
    
    _onRefreshActionClick: function()
    {
        // confirm with the user first, if there're values entered.
        if (this.isEdited()) {
            if (!confirm(RtlmUiUtil.getMessage("MsgClearEditing"))) 
                return;
        }
        this.viewHeaderNode.setProgressStart(LOADING);
        this._fetchRequestData();
    },
    
    _fetchDataFailure: function(errorType, errorMessage)
    {
        var details = (errorType && errorType.message) ? errorType.message : null;
        this.showStatusLine(RtlmUiUtil.getMessage("reqMsgFetchDataError"), ViewHeaderLayout.ERROR, details);
        this.viewHeaderNode.setProgressEnd();
        dojo.style(this.viewHeaderNode.containerNode, "visibility", "visible");

        com.ibm.asq.common.web.ui.internal.util.Utility.handleArtifactLoadError(RtlmUiUtil.getMessage("reqLabel_request"), details);  
    },
    
    _fetchRequestDataSuccess: function(response)
    {
        // wrap the data with our object.
        this._requestData = new Request.DataObject(this, response);

        this._configurePage();
        this.viewHeaderNode.setProgressEnd();
        dojo.style(this.viewHeaderNode.containerNode, "visibility", "visible");
        
        if(this.configId)
            this.setConfigSelection(this.configId);		
    },
    
    _fetchRequestData: function()
    {
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_fetchRequestDataSuccess", "_fetchDataFailure");
        var request = new com.ibm.rational.test.lm.web.client.internal.request();
        
        if (this.isNew) 
            request.getAttributeValues(responseHandler);
        else 
            request.getByUuid(responseHandler, this.requestId);
    },
    
    _fetchUsersSuccess: function(response)
    {
        this.userList = response;
        this.userNameList = [];
        
        if (response && dojo.isArray(response)) {
            for (var i = 0; i < response.length; i++) {
                this.userNameList.push(response[i].name);
            }
        }
        this._fetchRequestData();
    },
    
    _fetchUsers: function()
    {
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_fetchUsersSuccess", "_fetchDataFailure");
        var user = new com.ibm.rational.test.lm.web.client.internal.projectmembers();
        user.getMembers(responseHandler);
    },
    
    // Summary
    //    Fetches essential data from the backend, this includes current user info, lab manager users
    //    and request attributes. This method will fetch data that we already have cached.
    //
    _fetchData: function()
    {
        this.viewHeaderNode.setProgressStart(LOADING, /*hideContent*/true);
        dojo.style(this.viewHeaderNode.containerNode, "visibility", "hidden");
        this._fetchUsers();
    },

    // Summary:
    //    Configure the page based on the state of the request and the role of the current user.
    //    This includes making fields editable/uneditable and customizing the actions that can be
    //    performed on the request.
    //
    _configurePage: function()
    {        
        var hasFindAction = this._requestData.getActionInfo(this.FIND_MACHINE_ACTION_ID);
        var msg = (hasFindAction) ? this.configSectSearchInfoText : this.configSectInfoText;
        this.configSectionNode.summaryNode.innerHTML = msg;

        this._isLoading = true;
        this.isNew = this._requestData.getAttributeValueByName(this.REQ_ID_ATTR) == null ? true : false;
        this._configureActions();
        this._configurePageFields();
        this._checkActionButtonEnablement();
        
        if(!this.isNew) {
            // show the proper asset's name in its parent container.
            dojo.publish(GlobalEvents.UpdateTabLabel, [window.location.hash, this._requestData.getTitle(), "request-icon-16"]);
        }
        this._isLoading = false;
		
				
		//update browser title
		this._updateBrowserTitle();
    },

    _getMinDate: function()
    {
        var todayDate = new Date();
        todayDate.setHours(0, 0, 0, 0);
        return todayDate;
    },
    
    _configurePageFields: function()
    {
        var label, attr;
        var editable;
        var dateField;
        var today = this._getMinDate();

        this.viewHeaderNode.setTitle(this._requestData.getTitle());
                     
        this.viewHeaderNode.getInfoAttributesNode().innerHTML = "";        
        this.reserveFromNode.innerHTML = "";
        this.reserveToNode.innerHTML = "";
        this.configDisplayNode.innerHTML = "";
        
		this.fromLabelNode.style.display = "none";
		this.toLabelNode.style.display = "none";
		
        label = document.createElement("span");
        label.innerHTML = this.createdByLabel;
        attr = document.createElement("span");
        attr.innerHTML = this._requestData.getCreator();
        this.viewHeaderNode.addInfoAttribute(label, attr);
        dojo.addClass(label, "header-label");
        
        label = document.createElement("span");
        label.innerHTML = this.statusLabel;
        attr = document.createElement("span"); 
        attr.innerHTML = this._requestData.getCurrentStateDisplay();
        this.viewHeaderNode.addInfoAttribute(label, attr);
        dojo.addClass(label, "header-label");
        
        label = document.createElement("span");
        label.innerHTML = this.priorityLabel;
        editable = this._requestData.isFieldEditable(this.PRIORITY_ATTR);
        
        if(editable) {
            attr = document.createElement("select");  
            RtlmUiUtil.setComboContent(attr, this._requestData.getPriorityValues(), this._requestData.getPriority(), true, true);
            this.fields[this.PRIORITY_ATTR] = attr;
            this.fields[this.PRIORITY_ATTR].name = this.PRIORITY_ATTR;
            this.fields[this.PRIORITY_ATTR].onchange = dojo.hitch(this, "valueChanged");
        }
        else {
            attr = document.createElement("span");  
            attr.innerHTML = this._requestData.getPriorityDisplay();          
        }
        this.viewHeaderNode.addInfoAttribute(label, attr);
        dojo.addClass(label, "header-label");
        
        // Put the next set of attributes in another line
        label = document.createElement("span");
        attr = document.createElement("div");
        attr.className = "header-line-br";
        this.viewHeaderNode.addInfoAttribute(label, attr);
        
        label = document.createElement("span");
        label.innerHTML = this.assignedLabel;
        editable = this._requestData.isFieldEditable(this.ASSIGNED_ATTR);
        
        if(editable) {
            attr = document.createElement("select");  
            RtlmUiUtil.setUserComboContent(attr, this.userList, this._requestData.getAssignee(), true, true);
            this.fields[this.ASSIGNED_ATTR] = attr;
            this.fields[this.ASSIGNED_ATTR].name = this.ASSIGNED_ATTR;
            this.fields[this.ASSIGNED_ATTR].onchange = dojo.hitch(this, "valueChanged");
        }
        else {
            attr = document.createElement("span");  
            attr.innerHTML = this._requestData.getAssignee() ? this._requestData.getAssignee() : RtlmUiUtil.getMessage("reqLabel_unassigned");          
        }
        this.viewHeaderNode.addInfoAttribute(label, attr);
        dojo.addClass(label, "header-label");

        // Put the next set of attributes in another line
        label = document.createElement("span");
        attr = document.createElement("div");
        attr.className = "header-line-br";
        this.viewHeaderNode.addInfoAttribute(label, attr);

        label = document.createElement("span");
        label.innerHTML = this.confirmByLabel;
        editable = this._requestData.isFieldEditable(this.CONFIRM_BY_ATTR);
        
        dateField = null;
        if(editable) {
            dateField = new com.ibm.asq.common.web.ui.internal.widgets.DatePicker({
                        name: this.CONFIRM_BY_ATTR,
                        value: "",
                        minDate:today,
                        invalidMessage: RtlmUiUtil.getMessage("reqMsgDateInputFormat"),
                        onChange: dojo.hitch(this, "valueChanged"),
                        onkeyup: dojo.hitch(this, "valueChanged"),
                        "class": "text-field"
                    });
            attr = dateField.domNode;
            dateField.setValue(this._requestData.getConfirmByDate());
            this.fields[this.CONFIRM_BY_ATTR] = dateField;
        }
        else {
            attr = document.createElement("span");  
            attr.innerHTML = this._requestData.getConfirmByDateDisplay();         
        }
        this.viewHeaderNode.addInfoAttribute(label, attr);
        dojo.addClass(label, "header-label");
        if(dateField)
            dateField.init();

        editable = this._requestData.isFieldEditable(this.SUMMARY_ATTR);
        this.viewHeaderNode.setSummaryEditable(editable);
        this.viewHeaderNode.setSummary(this._requestData.getSummary());
        
        editable = this._requestData.isFieldEditable(this.RESERVE_FROM_ATTR);
        dateField = null;
        if(editable) {
            dateField = new com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker({
						label: this.fromLabel,
                        name: this.RESERVE_FROM_ATTR,
                        date: this._requestData.getReserveFromDate(),
                        minDate:today,
                        invalidMessage: RtlmUiUtil.getMessage("reqMsgDateInputFormat"),
                        onDateChange: dojo.hitch(this, "_onFromDateChanged"),
						onTimeChange: dojo.hitch(this, "valueChanged"),
                        onKeyUp: dojo.hitch(this, "_onFromDateChanged"),
                        "dateClass": "text-field",
						"time": this._requestData.getReserveFromDate()
                    });
            attr = dateField.domNode;
            this.fields[this.RESERVE_FROM_ATTR] = dateField;
        }
        else {
            attr = document.createElement("span");  
            attr.innerHTML = this._requestData.getReserveFromDateDisplay();
			this.fromLabelNode.style.display = "inline";
        }
        this.reserveFromNode.appendChild(attr);

        editable = this._requestData.isFieldEditable(this.RESERVE_TO_ATTR);
        dateField = null;
        if(editable) {
            var timeObj = this._requestData.getReserveToDate() ? this._requestData.getReserveToDate() : this._getEndOfDayTime();
            dateField = new com.ibm.asq.common.web.ui.internal.widgets.datetimepicker.DateTimePicker({
						label: this.toLabel,
                        name: this.RESERVE_TO_ATTR,
                        date: this._requestData.getReserveToDate(),
                        minDate:today,
                        onDateChange: dojo.hitch(this, "_onToDateChanged"),
						onTimeChange: dojo.hitch(this, "valueChanged"),
                        onKeyUp: dojo.hitch(this, "_onToDateChanged"),
                        "dateClass": "text-field",
						"time": timeObj
                    });
            attr = dateField.domNode;
            this.fields[this.RESERVE_TO_ATTR] = dateField;
        }
        else {
            attr = document.createElement("span");  
            attr.innerHTML = this._requestData.getReserveToDateDisplay();
			this.toLabelNode.style.display = "inline";
        }
        this.reserveToNode.appendChild(attr);
        
        var mode = this.getConfigurationMode();
        var config = this._requestData.getConfiguration();
        
        // ensure the reserve & find machine nodes are not shown
        // Later we'll enable either one of them based on the state
        dojo.style(this.reserveResultNode, "display", "none");
        dojo.style(this.findResultNode, "display", "none");

        // create a copy of the config for the user to play with.
        // We don't change the config at this state of the request.
        //
        if(mode == "SELECTABLE" && config) {
            var clone = config.clone();
            config = clone;
        }
        this.configDisplay = new ConfigDisplay({
                'mode': mode,
                'showSingleConfigPart': this.getIsShowSingleConfigPart(),
                'showTitle': false,
                'margin': "0",
                'canBeEmpty': true,
                'onConfigPartSelectionChanged': dojo.hitch(this, "_configPartSelectionChanged"),
                'onValueChanged': dojo.hitch(this, function(){
                    this.valueChanged(this.CONFIGURATION_ATTR);
                })
        });
        
        // load the config after we've construct the display. The display
        // would call back to us if the config is loaded as a part of the 
        // during construction.
		this.configDisplay.setCatalogConfig(config);
        
		if (this.isNew)
			this.configDisplay._addNewPart();
			
        this.configDisplayNode.appendChild(this.configDisplay.domNode);
        this.fields[this.CONFIGURATION_ATTR] = this.configDisplay;
            
        this.descriptionNode.name = this.DESCRIPTION_ATTR;
        this.fields[this.DESCRIPTION_ATTR] = this.descriptionNode;
        var desc = this._requestData.getDescription();
        this.descriptionNode.value = (desc) ? desc : "";
        this.descriptionNode.readonly = !this._requestData.isFieldEditable(this.DESCRIPTION_ATTR);
    },
    
    // Summary:
    //    Based on the control data from the back-end, returns the mode for
    //    the configuration field. It can be one of the three modes: editable,
    //    selectable, and readonly.
    //
    getConfigurationMode: function(){
        var editable = this._requestData.isFieldEditable(this.CONFIGURATION_ATTR);
        
        if (!editable) {
            // if we have the find machine action, this means the configuration
            // items can be selectable (toggle on/off).
            if (this._requestData.getActionInfo(this.FIND_MACHINE_ACTION_ID) != null) 
                return "SELECTABLE"
            return "READONLY";
        }
        return "EDITABLE";
    },

    // Summary:
    //    Based on the control data from the back-end, returns the mode for
    //    the configuration field. It can be one of the three modes: editable,
    //    selectable, and readonly.
    //
    getIsShowSingleConfigPart: function(){
        var editable = this._requestData.isFieldEditable(this.CONFIGURATION_ATTR);
        
        if (!editable) {
            // show one config part at a time when it's in find/reserved resource
            // state.
            return this._requestData.getActionInfo(this.FIND_MACHINE_ACTION_ID) || this._requestData.getReservedMachines();
        }
        return false;
    },
    
    // Summary
    //    Create the request action buttons based on the current request state and user's role.
    //
    _configureActions: function(){
        var action;
        this._actions = [];
        var actionInfo = null;
        
        // clear the action area first.
        var buttonArea = this.buttonAreaRootNode;
        this.buttonAreaRootNode.innerHTML = "";

        if(!this.isNew) {
            var arg = {
                label: RtlmUiUtil.getMessage("Refresh"),
                showLabel: false,
                'iconClass': "refresh-icon-image",
                'class': "left-action",
                onClick: dojo.hitch(this, '_onRefreshActionClick')
            };
            this.refreshButton = new dijit.form.Button(arg);
            this.viewHeaderNode.getRightActionsContainer().appendChild(this.refreshButton.domNode);
        }
        
        actionInfo = this._requestData.getActionInfo(this.FULFILL_ACTION_ID);
        if (actionInfo) {
            action = new Request.ActionWrapper(this, this.FULFILL_BUTTON_ID, actionInfo, false);
            this._actions.push(action);
        }
        actionInfo = this._requestData.getActionInfo(this.RETURN_ACTION_ID);
        if (actionInfo) {
            action = new Request.ActionWrapper(this, this.RETURN_BUTTON_ID, actionInfo, false);
            this._actions.push(action);
        }
        actionInfo = this._requestData.getActionInfo(this.REOPEN_ACTION_ID);
        if (actionInfo) {
            action = new Request.ActionWrapper(this, this.REOPEN_BUTTON_ID, actionInfo, false);
            this._actions.push(action);
        }
        actionInfo = this._requestData.getActionInfo(this.CLOSE_ACTION_ID);
        if (actionInfo) {
            action = new Request.ActionWrapper(this, this.CLOSE_BUTTON_ID, actionInfo, false);
            this._actions.push(action);
        }
        actionInfo = this._requestData.getActionInfo(this.CANCEL_ACTION_ID);
        if (actionInfo) {
            action = new Request.ActionWrapper(this, this.CANCEL_BUTTON_ID, actionInfo, false);
            this._actions.push(action);
        }
        
        // reset & save buttons are always needed.
        var resetInfo = {};
        resetInfo["action"] = this.RESET_ACTION_ID;
        resetInfo["pageTransition"] = false;
        resetInfo["requiredData"] = [];
        action = new Request.ActionWrapper(this, this.RESET_BUTTON_ID, resetInfo, false);
        
        // the reset button needs editing to be enabled.
        action.setIsRequireEditToEnable(true);
        action.setCanEnableWithInvaildInputs(true);
        this._actions.push(action);
        
        actionInfo = this._requestData.getActionInfo(this.SAVE_ACTION_ID);
        if (actionInfo) {
            action = new Request.ActionWrapper(this, this.SAVE_BUTTON_ID, actionInfo, false);
            // the save button needs editing to be enabled.
            action.setIsRequireEditToEnable(true);
            this._actions.push(action);
        }
		
        actionInfo = this._requestData.getActionInfo(this.SUBMIT_ACTION_ID);
        if (actionInfo) {
            action = new Request.ActionWrapper(this, this.SUBMIT_BUTTON_ID, actionInfo, false);
            this._actions.push(action);
        }
    },
    
    isEdited: function()
    {
        return this._isEdited;
    },

    _getDateTimeValueFromField: function(field, /*optional*/timePrecision)
    {
        var val = field.getValue();
        
        if(val) {
            // add the time components to the date
            if(field.timeField && timePrecision) {
                var time = field.timeField.getTime();
                val.setHours(time.getHours(), time.getMinutes(), 0, 0);
            }
            val = val.getTime();
        }
        return val;
    },
    
    _getValueByAttributeName: function(attrName)
    {
        var field = this.fields[attrName];
        var val = null;
        
        if (field) {
            if (attrName == this.PRIORITY_ATTR ||
                attrName == this.ASSIGNED_ATTR) {
                val = this._getSelectValue(field);
                if(val == null)
                    val = "";
            }
            else if (attrName == this.CONFIRM_BY_ATTR ||
                     attrName == this.RESERVE_FROM_ATTR ||
                     attrName == this.RESERVE_TO_ATTR) {
                val = this._getDateTimeValueFromField(field, /*timePrecision*/true);
            }
            else if(attrName == this.CONFIGURATION_ATTR) {
                var config = field.getCatalogConfig();
                val = "";
                
                // once the config field is not editable, ignore
                // the config that is in the field. The field could
                // still modify the config in SELECTABLE mode. We don't
                // want to store that "interrim" config only used for the
                // search action. Just return the "main" config.
                if(this.getConfigurationMode() != "EDITABLE") {
                    config = this._requestData.getConfiguration();
                }
                
                // only return the config value if it really has parts in it.
                if(config) {
                    var rootAttr = config.getRootAttribute();
                    if(rootAttr.getNumberOfParts() > 0)
                        val = config.toJson();
                }
            }
            else if(attrName == this.DESCRIPTION_ATTR) {
                var val = dojo.trim(field.value);
                if(val == null)
                    val = "";
            }
        }
        else if(attrName == this.SUMMARY_ATTR) {
            val = this.viewHeaderNode.getSummary();   
        }
        
        return val;
    },
    
    _getUserIdByName: function(name){
        if (this.userList) {
            for (var i = 0; i < this.userList.length; i++) {
                if (this.userList[i].name == name) 
                    return this.userList[i].itemId;
            }
        }
        return null;
    },
    
    _hasInvalidInputs: function()
    {
        if((this.fields[this.CONFIRM_BY_ATTR] && !this.fields[this.CONFIRM_BY_ATTR].isValid(false)) ||
           (this.fields[this.RESERVE_FROM_ATTR] && !this.fields[this.RESERVE_FROM_ATTR].isValid(false)) ||
           (this.fields[this.RESERVE_TO_ATTR] && !this.fields[this.RESERVE_TO_ATTR].isValid(false)))
           return true;

        var now = (new Date()).getTime();
        var confirmDate = (this.fields[this.CONFIRM_BY_ATTR] && this.fields[this.CONFIRM_BY_ATTR].getValue()) || 0;
        var resFromDate = (this.fields[this.RESERVE_FROM_ATTR] && this.fields[this.RESERVE_FROM_ATTR].getValue()) || 0;
        var resToDate = (this.fields[this.RESERVE_TO_ATTR] && this.fields[this.RESERVE_TO_ATTR].getValue()) || 0;

        if(confirmDate && now > confirmDate.getTime())
            return true;
        
        if(resFromDate && now > resFromDate.getTime())
            return true;
            
        if(resToDate && now > resToDate.getTime())
            return true;
            
        if(resToDate > 0 && resToDate < resFromDate)
            return true;
            
        return false;
    },
	
    _missingDateRange: function()
    {
        var from = (this.fields[this.RESERVE_FROM_ATTR] && this.fields[this.RESERVE_FROM_ATTR].getValue()) || 0;
        var to = (this.fields[this.RESERVE_TO_ATTR] && this.fields[this.RESERVE_TO_ATTR].getValue()) || 0;
        
        return from > 0 && to == 0 || from == 0 && to > 0;
    },
    
    // Summary:
    //    Returns whether the action's required input have been entered.
    //
    _checkHasActionRequiredInputs: function(/* actionWrapper */action){
        // this action is not active. 
        if (!action.isActionActive()) 
            return false;
        
        var required = action.getActionInfo().requiredData;
        
        if (required && required.length) {
            for (var i = 0; i < required.length; i++) {
                var val = this._getValueByAttributeName(required[i]);
                
                if (val == null || (dojo.isString(val) && val.length == 0)) {
                    return false;
                }
            }
            
            // if the action requires edits to be enabled. check
            // the form for editing, before enable it.
            if (action.isRequireEditToEnable()) 
                return this.isEdited();
        }
        // if there's no require fields, make edited form
        // the require enablement.
        return !action.isRequireEditToEnable() || this.isEdited();
    },
    
    // Summary:
    //    Enable/disable our action buttons based on the current form's edited
    //    and/or fields' value statuses.
    //
    _checkActionButtonEnablement: function(){
        var hasInvalidInputs = this._hasInvalidInputs();

        this.clearStatusLine();
        
        for (var i = 0; i < this._actions.length; i++) {
            if (this._actions[i].isActionActive()) {
                var hasRequired = this._checkHasActionRequiredInputs(this._actions[i]);
                var disableWhenInvalidInput = (hasInvalidInputs && !this._actions[i].canEnableWithInvalidInputs());
                
                this._actions[i].setDisable(!hasRequired || disableWhenInvalidInput);
            }
        }
        var val = this._getValueByAttributeName(this.SUMMARY_ATTR);
        if(!val || val.length == 0) {
            this.showStatusLine(RtlmUiUtil.getMessage("reqMsgRequiredValueMissing"), ViewHeaderLayout.ERROR);            
        }
		else if(this._missingDateRange()) {
            this.showStatusLine(RtlmUiUtil.getMessage("reqMsgMissingDateRange"), ViewHeaderLayout.ERROR);                        
        }
        else if(this.isEdited()) {
            // set a reminder message in the save status
            var msgId = (this.isNew) ? "saveNewReminderMsg" : "saveChangeReminderMsg";
            this.showStatusLine(RtlmUiUtil.getMessage(msgId), ViewHeaderLayout.INFO);
        }
    },

    // Summary:
    //    Gets the data from the UI fields.
    //
    _getFormData: function(){
        var reqData = {};
        
        // Fill in the request data that's not visible/changeable by the user.
        // 
        if (this.isNew) {
            reqData[this.RECIPIENT_ATTR] = this._requestData.getCreatorId();
        }
        else {
            reqData[this.STATE_ID_ATTR] = this._requestData.getAttributeValueByName(this.STATE_ID_ATTR);
            
            // TODO: requestParam uses a different name for the id... change it to a 
            // uniform name, when the backend is done.
            reqData["requestUuid"] = this._requestData.getAttributeValueByName(this.REQ_ID_ATTR);
        }
        reqData[this.STATE_ATTR] = this._requestData.getCurrentState();
        reqData[this.CREATED_BY_ATTR] = this._requestData.getCreatorId();
        reqData[this.CREATED_ON_ATTR] = this._requestData.getCreateDate().getTime();
        
        for (var name in this.fields) {
            var val = this._getValueByAttributeName(name);

            if(val != null) {
                reqData[name] = val;
            }
        }
        if(this._summaryEdited) {
            reqData[this.SUMMARY_ATTR] = this.viewHeaderNode.getSummary();
        }
        
        // we have to resend the date values to the request, even if they're not
        // editable. Otherwise, they'll be interpreted as clearing the values.
        // 
        var confirmBy = this._requestData.getConfirmByDate();
        if(!reqData[this.CONFIRM_BY_ATTR] && confirmBy) {
            reqData[this.CONFIRM_BY_ATTR] = confirmBy.getTime();
        }
        var reserveFrom = this._requestData.getReserveFromDate();
        if(!reqData[this.RESERVE_FROM_ATTR] && reserveFrom) {
            reqData[this.RESERVE_FROM_ATTR] = reserveFrom.getTime();
        }
        var reserveTo = this._requestData.getReserveToDate();
        if(!reqData[this.RESERVE_TO_ATTR] && reserveTo) {
            reqData[this.RESERVE_TO_ATTR] = reserveTo.getTime();
        }
        return reqData;
    },

    // Summary:
    //    Main entry point for invoking an action. This is called by the ActionWrapper's 
    //    dispatch of action UI events.
    //
    _invokeAction: function(/* ActionWrapper */action, /* the ui that invoked the action */ node){
        var needCheckInput = true;
        this.invokedAction = action;
        
        // the Reset action is a special action that's not defined by
        // the back-end. Handle it differently.
        if (action.getId() == this.RESET_ACTION_ID) {
            this._doReset();
            return;
        }
        
        // warn the use before canceling.
        if (action.getId() == this.CANCEL_ACTION_ID) {
            // confirm with the user first,
            if (!confirm(RtlmUiUtil.getMessage("reqMsgCancelConfirm"))) 
                return;
                
            // If the Request hasn't been save we an just exit
            if (!this._requestData.data.visibleId) {
                this.exitPage();
                return;
            }
        }
        
        // the find machine action has its own status area, don't
        // need to show the working dialog. 
        this._needShowViewProgress = true;

        // Disable the action to prevent the user from double clicking the button to invoke
        // the action twice.  Cancel and Reset should not be disabled and is already handled
        // and returned earlier.
        //
        action.setDisable(true);
        
        // invoke the backend action.
        //
        var actionInfo = action.getActionInfo();
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_actionSucceedHandler", "_actionFailHandler");
        var request = new com.ibm.rational.test.lm.web.client.internal.request();
        var reqData = this._getFormData();
        
        // fill in the action directive to invoke in the backend.
        reqData[this.DIRECTIVE_ATTR] = actionInfo.action;
        
        // fill in the project area ID to use in the backend.
        reqData[this.PROJECT_AREA_ATTR] = com.ibm.team.process.web.projectArea.itemId; 
        
        //bring up a working/busy dialog while we're running the action.
        if (this._needShowViewProgress) 
            this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("reqMsgPerformRequestAction"));
        
        request.update(responseHandler, reqData);
    },
    
    _actionSucceedHandler: function(response){
        // bring down the working dialog.  
        if (this._needShowViewProgress) 
            this.viewHeaderNode.setProgressEnd();
        
        // if after invoked the action we need to transition the page,
        // exit now.
        var exitPage = this.invokedAction && this.invokedAction.isPageTransition();

        this.invokedAction = null;
        this._isEdited = false;        
        
        this.showStatusLine(RtlmUiUtil.getMessage("reqMsgRequestActionSucceeded"), ViewHeaderLayout.SUCCESS);
        this._requestData = new Request.DataObject(this, response);
        
        if(this.isNew) {
            this.requestId = response.id;
            
            dojo.publish(GlobalEvents.ArtifactsAdded, [ RtlmUiUtil.TYPE_REQUEST, [ this._requestData ], UriUtil.getEditRequestURI(this.requestId) ]);
            // don't have to update the current page's location, if we're
            // exiting. The page will be removed.
            if(!exitPage)
                dojo.publish(GlobalEvents.UpdateTabUri, [window.location.hash, UriUtil.getEditRequestURI(this.requestId)]);
        } else {
            dojo.publish(GlobalEvents.ArtifactsUpdated, [ RtlmUiUtil.TYPE_REQUEST, [ this._requestData ], UriUtil.getEditRequestURI(this.requestId) ]);
        }
        
        // we're either exiting the page, which closes the page, or keep the
        // page in view, in which case we'll reconfigure the page in case of
        // state changes.
        if (exitPage) {
            this.exitPage();
        }
        else {
            this._configurePage();   
             
            // refresh the title too, in case this was new or there's a name change
            this._updateBrowserTitle();
            this._updateTabLabel(this._isEdited);
        }
    },
    
    _actionFailHandler: function(errorType){
        this.viewHeaderNode.setProgressEnd();
        var details = (errorType && errorType.message) ? errorType.message : null;
        
        if (errorType.status == 409) {
            // Stale data
            this.showStatusLine(RtlmUiUtil.getMessage("reqMsgRequestActionFailedStaleData"), ViewHeaderLayout.ERROR, details);
        }
        else {
            if(errorType.message && errorType.message.toLowerCase().indexOf('license') > -1) {
                RtlmUiUtil.showServerErrorMessage(MessageUtil.getMessage(MessageBundle.COMMON, "serverLicenseErrMsg"));                  
            }
            this.showStatusLine(RtlmUiUtil.getMessage("reqMsgRequestActionFailed"), ViewHeaderLayout.ERROR, details);
        }
        this.invokedAction = null;
    },

    // Summary:
    //    UI only action to reset the current request form.
    //
    _doReset: function(){
        // confirm with the user first, if there're values entered.
        if (this.isEdited()) {
            if (!confirm(RtlmUiUtil.getMessage("MsgClearEditing"))) 
                return;
        }
        this.clearStatusLine();
        
        this._isEdited = false;
        this._isLoading = true;
        this._configurePageFields();
        this._checkActionButtonEnablement();
        this._updateTabLabel(this._isEdited);
        this._isLoading = false;
    },

    valueChanged: function(fieldId)
    {
        if(this._isLoading)
            return;

        // if the user is just messing around with the configuration
        // attributes, don't change the form's edit state because of it.
        // It's only transient data change.
        if(fieldId && fieldId == this.CONFIGURATION_ATTR && 
           this.getConfigurationMode() == "SELECTABLE")
            return;
            
        this._isEdited = true;
        this._checkActionButtonEnablement();
        this._updateTabLabel(this._isEdited);
    },
    
    _onSummaryChanged: function()
    {
        if(this._isLoading)
            return;

        this._isEdited = true;
        this._summaryEdited = true;
        this._checkActionButtonEnablement();
        this._updateTabLabel(true);
    },

    _getSelectValue: function(node)
    {
        var val = null;
        for(var i = 0; i < node.options.length; i++) {
            if(node.options[i].selected) {
                val = node.options[i].value;
                break;
            }
        }
        return val;
    },

    _onFromDateChanged: function()
    {
        if(this._isLoading)
            return;
            
        var fromField = this.fields[this.RESERVE_FROM_ATTR];
        
        // if the new value of the from field is good, change the
        // constraint value of the to field with the new value.
        //
        if(fromField.isValid()) {
            var minDate = this._getDateTimeValueFromField(fromField, /*timePrecision*/false);
            
            if (minDate) {
                var toField = this.fields[this.RESERVE_TO_ATTR];
                var date = new Date();
                date.setTime(minDate);
                toField.dateNode.constraints['min'] = date;
                
                // force an update of the calendar in the field.
                // Otherwise, the calendar would not pick up the new
                // constraints.
                if (toField.dateNode._picker) {
                    var picker = toField.dateNode._picker;
                    delete picker;
                    toField.dateNode._picker = null;
                }
                toField.dateNode.isValid(false);
            }
        }
        this.valueChanged();
    },

    _onToDateChanged: function()
    {
        if(this._isLoading)
            return;
            
        var toField = this.fields[this.RESERVE_TO_ATTR];
        
        // if the new value of the to field is good, change the
        // constraint value of the from field with the new value.
        //
        if (toField.isValid()) {
            var maxDate = this._getDateTimeValueFromField(toField, /*timePrecision*/ false);
            
            if (maxDate) {
                var fromField = this.fields[this.RESERVE_FROM_ATTR];
                var date = new Date();
                date.setTime(maxDate);
                
                fromField.dateNode.constraints['max'] = date;
                
                // force an update of the calendar in the field.
                // Otherwise, the calendar would not pick up the new
                // constraints.
                if (fromField.dateNode._picker) {
                    var picker = fromField.dateNode._picker;
                    delete picker;
                    fromField.dateNode._picker = null;
                }
                fromField.dateNode.isValid(false);
            }
        }
        this.valueChanged();
    },

    _getEndOfDayTime: function()
    {
        var date = new Date();
        date.setHours(23, 59, 0, 0);
        return date;    
    },
    
    clearStatusLine: function(){
        this.viewHeaderNode.clearStatus();
    },
    
    showStatusLine: function(message, status, details){
        this.viewHeaderNode.setStatus(message, status, details);
    },

	_updateTabLabel: function(modified)
	{
		if(!this.isNew) 
		{
            // show the proper asset's name in its parent container.
            dojo.publish(GlobalEvents.UpdateTabLabel, [window.location.hash, this._requestData.getTitle(), "request-icon-16", modified]);
        }
	},
	
    setConfigSelection: function(configID) 
    {
        var mode = this.getConfigurationMode();
        
        if (mode == "EDITABLE" || mode == "SELECTABLE") {
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadConfigSuccess", "_fetchDataFailure");
            var configService = new com.ibm.rational.test.lm.web.client.internal.config();
            configService.retrieveConfigurationWithDetails(responseHandler, configID);
        }
    },
    
    _loadConfigSuccess: function(response)
    {
        var catalog = new com.ibm.asq.common.web.client.internal.Catalog();
        var config = catalog.createConfigurationFromJson(response.configuration); 
        var configField = this.fields[this.CONFIGURATION_ATTR];
                
        configField.setCatalogConfig(config);
    },

    exitToMachinePage: function(machineId)
    {
        var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
        var assetUri = AssetUriUtil.getViewAssetDetailURI("Machine", machineId);
        window.location.href = assetUri;
    },
    
    // Summary
    //    Exit the request form and return to the request query selection view.
    //
    exitPage: function(){
        var uri = window.location.hash;
        
        // remove the page from view
        dojo.publish(GlobalEvents.CloseTab, [uri]);
    },


    /**
     * Configuration related methods. 
     */
    
    // Summary:
    //   Handle the configuration's part changed. That is if the config is a multi-part
    //   one, in find/reserve machine mode, we only show one part at a time. This notifies
    //   us that the part display has changed.
    //
    _configPartSelectionChanged: function(tag)
    {
        var findAction = this._requestData.getActionInfo(this.FIND_MACHINE_ACTION_ID);
        var config = this._requestData.getConfiguration();
        var resId = null;
        var reservation = null;
        
        if (config) {
            // the config is cloned when the user is fulfilling the request.
            // We allow the admin to change the config during machine search
            // but that config would never be copied back into th request.
            // So, the tag we're using is in config of the display. Use that config
            // to correlate to the original config's index to get the reservation
            // ID. 
            var config = this.configDisplay.catalogConfig;
            var index = config.getRootAttribute().getIndexByTag(tag);

            resId = this._requestData.getConfiguration().getIdentifier(index);
            
            if (resId) 
                reservation = this._getReservationById(resId);
        }
                        
        if (resId && reservation) {
            this._loadReservationTable(reservation);
            return "READONLY";
        }
        if (config && findAction) {
            this._configureFindMachineTable();
            return "SELECTABLE";
        }
    },
    
    _configureFindMachineTable: function()
    {
        if (!this.configuredFindMachineTable) {
            this.findMachineTableNode.setTableRowsCheckable(true, true);
            this.findMachineTableNode.setContentProvider(new com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestReservationsTableProvider());
            this.findMachineTableNode.setTableSummary(RtlmUiUtil.getMessage("labelTableSummary", RtlmUiUtil.getMessage("resMachineName")));

            //Here's the reserve machine button
            this.reserveButton = this.findMachineTableNode.addAction("reserve", RtlmUiUtil.getMessage("reqLabel_reserveMachine"), null, "reserve-icon-16");
            
            // create a table listener to monitor table row events and actions firing
            var tableListener = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener();
            tableListener.rowCheckStateChanged = dojo.hitch(this, function(){
                this.reserveButton.setDisabled(this.findMachineTableNode.getCheckedRows().length == 0);
            });
            tableListener.tableContentActionInvoked = dojo.hitch(this, function(tableViewer, actionId, callbackData, callback){
                if (actionId == "reserve") 
                    this._reserveMachine();
            });
            this.findMachineTableNode.setTableListener(tableListener);
            
            var findAction = document.createElement("a");
			findAction.setAttribute("href", "javascript:{}");      
            findAction.innerHTML = this.findMachineLabel;
            findAction["class"] = "find-action-link";
            findAction.onclick = dojo.hitch(this, "_findMachines");
            this.findMachineTableNode.contentControlNode.appendChild(findAction);    

            this.configuredFindMachineTable = true;
        }
        dojo.style(this.reserveResultNode, "display", "none");
        dojo.style(this.findResultNode, "display", "block");
        
        this.reserveButton.setDisabled(true);
        var emptyContent = {'Data':[]};
        this.findMachineTableNode.setContent(emptyContent);
        this.findMachineTableNode.noItemMessage = " ";
        this.findMachineTableNode.show();
    },
    
    _findMachines : function()
    {
        //Retrieve the data from the request form.  This includes the From and To dates 
        // the user specified for the request
        var reqData = this._getFormData();

        // ensure the request has a date range specified.
        if(!reqData[this.RESERVE_FROM_ATTR] || !reqData[this.RESERVE_TO_ATTR]) {
            var dlg = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({message:RtlmUiUtil.getMessage("reqMsgCantFindMachine")});
            dlg.open();
            return;
        }
        
        this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("MsgRequestMachineSearch"));
        this.findMachineTableNode._clearTableContents();              
        this.findMachineTableNode.showLoadingProgress();
        
        // We need to create a config from the Part that is in use.
        // That config will be used in the search. The config display
        // is in single part display mode and it returns the single part
        // config.
        var config = this.configDisplay.getCatalogConfig();
        
        reqData[this.CONFIGURATION_ATTR] = config.toJson();

        // fill in the action directive to invoke in the backend.
        reqData[this.DIRECTIVE_ATTR] = this.FIND_MACHINE_ACTION_ID;
            
        // fill in the project area ID to use in the backend.
        reqData[this.PROJECT_AREA_ATTR] = com.ibm.team.process.web.projectArea.itemId;
    
        // Find the machines
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_findMachineSuccessHandler", "_findMachineFailHandler");
        var request = new com.ibm.rational.test.lm.web.client.internal.request();       
        
       request.update(responseHandler, reqData);
    },
    
    //Hand success from the find machines
    _findMachineSuccessHandler : function(response)
    {
        // Kill the progress indicator
        this.viewHeaderNode.setProgressEnd();
        
        // use the default no item message, we want to show
        // a message for no-matching machines.
        this.findMachineTableNode.noItemMessage = RtlmUiUtil.getMessage("reqMsgNoMachineFound");
        this.findMachineTableNode.setContent(response[this.FOUND_MACHINE_ATTR]);
        this.findMachineTableNode.show();
        this.findMachineTableNode.endLoadingProgress();
    },

    _findMachineFailHandler : function(response)
    {
        // Kill the progress indicator
        this.viewHeaderNode.setProgressEnd();
        
        // use the default no item message, we want to show
        // a message for no-matching machines.
        var emptyContent = {'Data':[]};
        this.findMachineTableNode.noItemMessage = RtlmUiUtil.getMessage("reqMsgNoMachineFoundErr");
        this.findMachineTableNode.setContent(emptyContent);
        this.findMachineTableNode.show();
        this.findMachineTableNode.endLoadingProgress();
    },
    
    //Handle the reserve machine action
    _reserveMachine :function()
    {
        //Get the selected machine id
        var checkedRows = this.findMachineTableNode.getCheckedRows()
        var machineID = null;
        if (checkedRows.length > 0)
            machineID = checkedRows[0]["machine.id"];

        //If the user did not select a machine, prompt them to do so
        if (machineID == null)
        {
            var dlg = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({message:RtlmUiUtil.getMessage("reqMsgSelectMachine")});
            dlg.open();
            return;
        }            
        //Start the progress indicator
        this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("MsgReserveMachine"));

        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_reserveMachineSuccessHandler", "_actionFailHandler");
        var request = new com.ibm.rational.test.lm.web.client.internal.request();       
        
        //Get the data from the request form
        var reqData = this._getFormData();
        
        //Save this for use in the success handler
        this.reservedMachineID = machineID;
        
        //Set the id of the machine to reserve into the request data
        reqData[this.RESERVE_MACHINE_ID_ATTR] = machineID;
        // fill in the action directive to invoke in the backend.
        reqData[this.DIRECTIVE_ATTR] = this.RESERVE_MACHINE_ACTION_ID;
            
        // fill in the project area ID to use in the backend.
        reqData[this.PROJECT_AREA_ATTR] = com.ibm.team.process.web.projectArea.itemId;
        
        reqData[this.RESERVE_ASSET_TYPE] = "machine";  //MFV Some day we'll be able to reserve other stuff too
        
        //Reserve the machine
        request.update(responseHandler, reqData);   
    },
    
    //Handle successfully reserving a machine
    _reserveMachineSuccessHandler : function(response)
    {
        //Get ID of the machine we reserved
        var machineID = this.reservedMachineID;
        
        //Update the request Data object from the response object.
        //THIS IS NECESSARY BECAUSE THE DATA CONTAINS STATE CHANGE INFORMATION THAT IS USED
        //ON SUBSEQUENT BACK END CALLS
        this._requestData = new Request.DataObject(this, response);

        //Get the reservations array (although there should be only 1)
        var reservations = response.reservations;
		var fulfiller = response.fulfiller;
        
        //Save the id of the reservation into the config and the original config

        //The response back from the reserve call is a RequestDTO which contains ALL the reservations for this request
        //We need to find the reservation that references a machine with the same ID as the one we used to make the request
        var resID = null;
        var thisReservationArr = [];
        for (var i = 0; i < reservations.length; i++) {
            if (reservations[i].configuration.getId() == machineID) {
                resID = reservations[i].id;
                //Put the matching reservation into an array for using in the UI/Table below
                thisReservationArr.push(reservations[i]);  
                break;
            }
        }
        if (resID != null) {
            // the config is cloned when the user is fulfilling the request.
            // We allow the admin to change the config during machine search
            // but that config would never be copied back into th request.
            // So, the tag we're using is in config of the display. Use that config
            // to correlate to the original config's index to write the reservation
            // ID into the config part.
            var tag = this.configDisplay.getCurrentConfigPartTag();
            var config = this.configDisplay.catalogConfig;
            var index = config.getRootAttribute().getIndexByTag(tag);
            
            // write the reservation id into the config. Next time we can fine it
            // when we display the config part.
            this._requestData.getConfiguration().getRootAttribute().setIdentifier(index, resID);

            // Show the reserved machine in the reserved machine table
            // refresh the config display as well to change it to readonly mode
            this.configDisplay.mode = "READONLY";
            this.configDisplay.refresh();           
            this._loadReservationTable(thisReservationArr);
            
            //A little klugey......
            //We need to save the request now in order to ensure that the reservation ID that we
            //set into the config is saved.  Without this we could not correlate the reserved machine back to this part
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_saveSuccessHandler", "_actionFailHandler");
            var request = new com.ibm.rational.test.lm.web.client.internal.request();       
            
            var reqData = this._getFormData();
			
			//fill in the fulfiller if there is not one yet.  In this case, fulfiller is just the assigned to person
			if (reqData[this.ASSIGNED_ATTR] == null || reqData[this.ASSIGNED_ATTR] == "")
				reqData[this.ASSIGNED_ATTR] = fulfiller.id;
            
            // fill in the action directive to invoke in the backend.
            reqData[this.DIRECTIVE_ATTR] = this.SAVE_ACTION_ID;
                
            // fill in the project area ID to use in the backend.
            reqData[this.PROJECT_AREA_ATTR] = com.ibm.team.process.web.projectArea.itemId;
            
            // Use our copy of the config to store the reservation id.
            config = this._requestData.getConfiguration();
            reqData[this.CONFIGURATION_ATTR] = config.toJson();
            
            request.update(responseHandler, reqData);   
        }
    },
    
    _loadReservationTable: function(content)
    {
        dojo.style(this.findResultNode, "display", "none");
        
        this.reservedMachineTableNode.setContentProvider( new com.ibm.rational.test.lm.web.ui.internal.views.providers.RequestReservedMachinesTableProvider() );
        this.reservedMachineTableNode.setContent(content);
		this.reservedMachineTableNode.setTableSummary(RtlmUiUtil.getMessage("labelTableSummary", RtlmUiUtil.getMessage("labelReservedMachine")));
		
        this.reservedMachineTableNode.show();
        dojo.style(this.reserveResultNode, "display", "block");
    },
    
    _saveSuccessHandler : function(response)
    {
        this.viewHeaderNode.setProgressEnd();   
        this._requestData = new Request.DataObject(this, response);
		
		//The assigned to person may have changed by virtue of the reservateion being made so update the assigned widget.
		RtlmUiUtil.setUserComboContent(this.fields[this.ASSIGNED_ATTR], this.userList, this._requestData.getAssignee(), true, true);
    },

    _getReservationById: function(resID)
    {
        //Get the list of reservations from the Request Data
        var reservations = this._requestData.getReservedMachines();
        var retVal = null;

        //Loop over the reservations looking for one that matches the id
        if (reservations) {
            for (var i = 0; i < reservations.length; i++) {
                var reservation = reservations[i];
                
                if (reservation.id == resID) {
                    //The table display needs and array so we'll return one
                    retVal = [];
                    retVal.push(reservation);
                    break;
                }
            }
        }
        return retVal;
    },
	
	isDirty: function()
	{
		return( this._isEdited );
	},
	
	_updateBrowserTitle: function()
	{
		if (this.page) {
			var	name;
			if (this.isNew) {
				name = RtlmUiUtil.getMessage("labelCapitalNew");
			}
			else {
				name = this.viewHeaderNode.getTitle();
			}
			this.page.browserTitle = RtlmUiUtil.getMessage("pageTitleRequest", name);
			net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this.page.browserTitle );
			net.jazz.ajax.ui.PlatformUI.getWorkbench().browserTitle = this.page.browserTitle;
		}
	},
    
    // Defines the backend attribute names
    SUMMARY_ATTR: "summary",
    STATE_ATTR: "currentState",
    CREATED_ON_ATTR: "createDate",
    CREATED_BY_ATTR: "requester",
    DESCRIPTION_ATTR: "description",
    PRIORITY_ATTR: "priority",
    CONFIRM_BY_ATTR: "confirmationDate",
    RESERVE_FROM_ATTR: "reserveFrom",
    RESERVE_TO_ATTR: "reserveTo",
    ASSIGNED_ATTR: "fulfiller",
    CREATE_DATE_ATTR: "createDate",
    STATE_ID_ATTR: "stateId",
    REQ_ID_ATTR: "id",
    STATE_ATTR: "currentRequestState",
    RECIPIENT_ATTR: "recipient",
    CONFIGURATION_ATTR: "configuration",
    FOUND_MACHINE_ATTR: "foundMachines",
    RESERVE_MACHINE_ID_ATTR: "reserveMachineId",
    RESERVE_ASSET_TYPE: "reserveAssetType",
    RESERVATION_ATTR: "reservations",
    DIRECTIVE_ATTR: "directive",
    PROJECT_AREA_ATTR: "projectAreaItemId",
    
    // Defines the button node ids in HTML
    CANCEL_BUTTON_ID: "cancel_button",
    RESET_BUTTON_ID: "reset_button",
    SAVE_BUTTON_ID: "save_button",
    SUBMIT_BUTTON_ID: "submit_button",
    CLOSE_BUTTON_ID: "close_button",
    REOPEN_BUTTON_ID: "reopen_button",
    FULFILL_BUTTON_ID: "fulfill_button",
    RETURN_BUTTON_ID: "return_button",
    BUTTON_AREA_ID: "bottom_button_area",

    // Defines the backend action names.
    RESET_ACTION_ID: "reset",
    SAVE_ACTION_ID: "save",
    SUBMIT_ACTION_ID: "submit",
    FULFILL_ACTION_ID: "fulfill",
    RETURN_ACTION_ID: "return",
    REOPEN_ACTION_ID: "reopen",
    CLOSE_ACTION_ID: "close",
    FIND_MACHINE_ACTION_ID: "findMachine",
    RESERVE_MACHINE_ACTION_ID: "reserveMachine",
    CANCEL_ACTION_ID: "cancel"
    
});

})();
