/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Summary:
//   Subclass of TitlePane to allow opening and closing of the contents by
//   clicking on the arrow icon at the top left corner only, not the entire
//   title bar.  Action tools can be added to the right of the title bar
//   and there are optional action buttons area under the contents area.
//
// Usage (div attributes):
//   dojoType = custom.widgets.ActionTitlePane
//   title = header title string
//   collapsible = true or false to show or hide the twisty arrow button at the left of the header
//   toolsHTML = innerHTML for the toolbar <div> at the right of the header
//   actionsHTML = innerHTML for the actionbar <div> in the bottom below the content <div>
//   mainclass = the man class as prefix for group of classes for various parts
//
// Usage (methods):
//   addTool(tool) - add the DOM element (tool) to the toolbar
//   addAction(action) add the DOM element (action) to the action bar
//   setTitle(title) - superclass method to set the header title
//
// Notes:
//   - The id of the content <div>, toolbar <div> and the action <div> are "_Content", "_ToolBar",
//     and "_ActionBar" respectively and all are prefixed with the widget id.
//   - The attribute names to override the mainClass are started with upper case.
//   - Reference the test page for example and specifically the CSS for image sizes and positions.
//
// Known Issues:
//   - The dojoType attributes specified in the toolsHTML and actionsHTML somehow is ignored therefore
//     we can not add dojo buttons from HTML declaration.
//   - Buttons added to the tool bar and action bar programmatically do not seem to obey the style
//     specified.
//   
  
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane");

dojo.require("dijit.TitlePane");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

dojo.declare(
  "com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane",
  [dijit.TitlePane],
{
  templateString: '<div id="${id}" dojoAttachPoint="domNode">' +
                    '<div class="actionTitlePaneBorder"></div>' +
  					'<table border="0" cellpadding="0" cellspacing="0" dojoAttachPoint="focusNode">' +
                    '<tr class="${mainclass} actionTitlePaneHeader">' +
                      '<td valign="top" class="${mainclass} actionTitlePaneHeader">' +
                        '<div class="${mainclass} actionTitlePaneOpenCloseArrow" name="${openCloseArrow}" dojoAttachPoint="arrowNode" waiRole="button" dojoAttachEvent="onclick: toggle"></div>' +
                        '<div class="${mainclass} actionTitlePaneA11yArrow" dojoAttachPoint="arrowNodeInner" waiRole="button" dojoAttachEvent="onclick: toggle"></div>' +
					  '</td>' +
                      '<td valign="middle" class="${mainclass} actionTitlePaneHeader"><div class="${mainclass} actionTitlePaneHeaderIcon"></div></td>' +
                      '<td valign="middle" class="${mainclass} actionTitlePaneHeader" nowrap>' +
                      '  <span class="${mainclass} actionTitlePaneHeaderTitle" dojoAttachPoint="titleNode" dojoAttachEvent="onclick: toggle, onkeypress: _onTitleKey"></span>' +
                      '  <span dojoAttachPoint="titleHelpNode"></span>' +                    
                      '</td>' +
                      '<td valign="top" class="${mainclass} actionTitlePaneHeader" width="100%">' +
                        '<div class="${mainclass} actionTitlePaneToolBar" dojoAttachPoint="toolsNode" align="left" waiRole="region">' +
                          '<span style="width: 1px;">&nbsp;</span>' +
                        '</div>' +
                      '</td>' +
                    '</tr>' +
                    '</table>' +
                    '<div dojoAttachPoint="hideNode">' +
                      '<div dojoAttachPoint="wipeNode">' +
                        '<div class="${mainclass} actionTitlePaneContent" dojoAttachPoint="containerNode,contentNode" waiRole="region" tabindex="-1">' +
                        /* From parent TitlePane:
                         *   Nested divs because wipeIn()/wipeOut() doesn't work right on node w/padding etc.
                         *   Put padding on inner div.
                         */
                        '</div>' +
                      '</div>' +
                    '</div>' +
                    '<div class="${mainclass} actionTitlePaneActionBar" dojoAttachPoint="actionsNode" waiRole="region" align="right"></div>' +
//                    '<div class="actionTitlePaneBorder"></div>' +
                  '</div>',


  /* innerHTML for toolbar tools */
  toolsHTML: "",
  
  /* innerHTML for action buttons */
  actionsHTML: "",
  
  /* optionl to make pane not collapsible, ensure use appropriate icon in CSS for open arrow */
  collapsible: true,

  /* a main class as prefix for group of classes for various parts */
  mainclass: "",
  
  tabIndex: null,
  
  contextHelpId: null,
  csHelpProviderId: null,
  csHelpEngine: null,
  cshelp: null,
  
  openCloseArrow: com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil.getMessage("labelOpenClose"),
  
  postCreate: function(){
    com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane.superclass.postCreate.apply(this, arguments);
    
    /* create the tool bar buttons if the HTML text is specified */
    if (this.toolsHTML != "")
      this.toolsNode.innerHTML = this.toolsHTML;
    
    /* create the action bar buttons if the HTML text is specified */
    if (this.actionsHTML != "")
      this.actionsNode.innerHTML = this.actionsHTML;
    
    /* support static pane that is not collapsible and hide the twisty arrow */
    if (!this.collapsible) {
      dojo.style(this.arrowNode, "display", "none");
      this.open = this.collapsible;
    }
    
    this.setCSHelpId(this.contextHelpId, this.csHelpProviderId, this.csHelpEngine);

    /* hide the action bar if the pane is initially collapsed or there is no action button */
    if (this.open && (this.actionsHTML != ""))
      dojo.style(this.actionsNode, "display", "block");
    else
      dojo.style(this.actionsNode, "display", "none");
    

    // need 1 pixel to show widget in focus otherwise its size will be
    // increased for the added margin which forces the scroll bars to
    // appear
    dojo.style(this.containerNode, "margin", "1px");
    dojo.style(this.hideNode, "margin", "1px");
    dojo.style(this.wipeNode, "margin", "1px");
    
    if (this.tabIndex) {
      this.titleNode.tabIndex = this.tabIndex;
    } else {
      this.titleNode.tabIndex = 0;
    }
    this._setCss();
  },

    setCSHelpId: function(helpId, csHelpProviderId, csHelpEngine) {
        this.contextHelpId = helpId;
        this.csHelpProviderId = csHelpProviderId;
        this.csHelpEngine = csHelpEngine;
        if (this.titleHelpNode) {
            dojox.data.dom.removeChildren(this.titleHelpNode);
            if (this.contextHelpId) {
                var args = {
                    'contextID': helpId,
                    'csHelpEngine': this.csHelpEngine,
                    'csHelpProviderId': this.csHelpProviderId
                };
                this.cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.titleHelpNode.appendChild(this.cshelp.domNode);
            }
        }
    },
    
	setCSHelpButtonLabel: function(label) {
		if(this.cshelp)
			this.cshelp.setButtonLabel(label);
	},
  // Parent class TitlePane changed the on click handler name from "_onTitleClick" to "toggle",
  // but the parent's on key pressed handler uses "_onTitleClick()" invoke toggle!?  Defect on
  // them possibly.  Since we are using the parent's key pressed handler, define _onTitleClick()
  // here to simply call toggle().
  //
  _onTitleClick: function() {
    this.toggle();
  },
  
  toggle: function() {
    /* override the method to allow non-collapsible pane */
    if (this.collapsible == true) {
      /* let parent class do the animation to hide the contents */
      com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane.superclass.toggle.apply(this, arguments);
      
      /* check if there is any tools and actions */
      var tools = dojo.isIE ? this.toolsNode.children : this.toolsNode.childNodes;
      var actions = dojo.isIE ? this.actionsNode.children : this.actionsNode.childNodes;

      if (this.open) {
        if (actions.length > 0)
          dojo.style(this.actionsNode, "display", "block");
      } else {
        if (actions.length > 0)
          dojo.style(this.actionsNode, "display", "none");
      }
    }
  },
  
  addTool: function(label, handler, callbackData, iconClass) {
        var button = this._addButtonToArea(this.toolsNode, label, handler, callbackData, iconClass);
        return button;
  },

    addAction: function(label, handler, callbackData, iconClass) {
        var button = this._addButtonToArea(this.actionsNode, label, handler, callbackData, iconClass);
        
        if (this.open) /* action may be initially hidden if there is no actions */
            dojo.style(this.actionsNode, "display", "block");
        return button;
    },
    
    _addButtonToArea: function(area, label, handler, callbackData, iconClass)
    {
        var arg = {
            label: label,
            showLabel: true,  // will not show label if icon is specified
            onClick: function( e ) {
                if (!this.parent.open) {
                    this.parent.toggle();
                }
                this.clientCallback(this.callbackData, e);
            },
            'class': "actionTitlePaneContent-button"
        };
        
        if (iconClass) {
            arg["iconClass"] = iconClass;
            arg["showLabel"] = false;
        }
        var button = new dijit.form.Button(arg);
        
        // store the client callback and data into the button
        button.parent = this;
        button.clientCallback = handler;
        button.callbackData = callbackData;
        
        area.appendChild(button.domNode);
        return button;
        
    }
    
  
});
