/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.attachmentViewer.AttachmentTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.attachmentViewer.AttachmentTextCellRenderer");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");

(function() {

var Constants = com.ibm.asq.common.web.client.internal.Constants;
var TableConstants 				= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     				= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var TeamServerClient 			= com.ibm.team.repository.web.transport.TeamServerClient;
var DOWNLOAD_SERVICE_URI 		= "com.ibm.rqm.planning.service.internal.rest.IAttachmentRestService/"; //RQM Downaload Service

dojo.declare("AttachmentTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{        
	setRowCheckListener: function(listener) {
	},
	
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
	},
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData)
    {                
        if (callbackData != null && dojo.isFunction(callbackData)) {
            callbackData();
        } 
    }    
});

dojo.declare("AttachmentTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    constructor: function(args) {
        dojo.mixin(this, args);
    },

	setChangeListener: function(listener)
	{
		this.changeListener = listener
	},

    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
    },
	
	cellActionInvoked: function(rowObject, columnIndex)
    {
		var listener = this.changeListener;//Handles Remove and Undo remove
		if(listener && listener.method)
			listener.method.apply(listener.context, [rowObject]);
    }
});

dojo.declare("AttachmentTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "labelAddCustCrtl");
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("AttachmentTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    showPath: true,
	editable: false,
	
	constructor: function(args) {
        dojo.mixin(this, args);
    },
     
    // Summary:
    //
    getNumColumns: function(content)
    {
        return this.editable && this.showPath ? 5 : (this.editable || this.showPath ? 4 : 3);
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        var headings = [];
        headings.push(
            MessageUtil.getMessage(MessageBundle.COMMON, "labelId"),
			MessageUtil.getMessage(MessageBundle.COMMON, "labelFileName"),
			MessageUtil.getMessage(MessageBundle.COMMON, "labelSize")
        );
		if(this.showPath)
			headings.push(MessageUtil.getMessage(MessageBundle.COMMON, "labelPath"));
		
		if(this.editable)
			headings.push(MessageUtil.getMessage(MessageBundle.COMMON, "labelRemove"));
        return headings[index];
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
       return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return false;   
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        switch(atColumnIndex) {
			case 0: 
                return this.showPath && this.editable ? 10 : (this.showPath || this.editable ? 10 : 25);
            case 1: 
                return this.showPath && this.editable ? 30 : 50;
            case 2:
				return this.showPath && this.editable ? 10 : (this.showPath || this.editable ? 15 : 25); 
			case 3: 
                return 25;
			case 4:
				return 25;	
            
        }
        return 0;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_attachments_" + rowData.id;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = "RenderType.AttachmentCell";

		atColumnIndex = !this.showPath ? atColumnIndex + 1 : atColumnIndex;
        switch(atColumnIndex) {
			case 4:
				type = TableConstants.RENDER_TYPE_ACTION;
				break;								 
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
		var val;
        switch(atColumnIndex) {
            case 0: 
                val = rowData.id;
                break;
			case 1: 
                val = rowData.fileName;
                break;
            case 2: 
                val = this.getSizeInKb(rowData);
                break; 
			case 3: 
                val = this.showPath ? (rowData.path ? rowData.path : "") : (rowData.isRemoved ? MessageUtil.getMessage(MessageBundle.COMMON, "labelUndo")+" "+ MessageUtil.getMessage(MessageBundle.COMMON, "labelRemove") : MessageUtil.getMessage(MessageBundle.COMMON, "labelRemove")); 
                break; 	         
            case 4: 
				val = rowData.noRemovable ? "" : (rowData.isRemoved ? MessageUtil.getMessage(MessageBundle.COMMON, "labelUndo")+" "+ MessageUtil.getMessage(MessageBundle.COMMON, "labelRemove") : MessageUtil.getMessage(MessageBundle.COMMON, "labelRemove"));
				break;
        }
        return val;
    },

	getSizeInKb: function(attachment)
	{
			var sizeText = "" + Math.ceil((attachment.size != null ? attachment.size : 0) / 1024);
			var rgx = /(\d+)(\d{3})/;
			while (rgx.test(sizeText)) {
				sizeText = sizeText.replace(rgx, '$1' + ',' + '$2');
			}
			sizeText = sizeText + " "+MessageUtil.getMessage(MessageBundle.COMMON, "labelKB");
			return sizeText;
	},
    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
 		 var val = {};
		if(atColumnIndex == 1){
            val.href = TeamServerClient._getServicePrefix() + DOWNLOAD_SERVICE_URI + rowData.uuid;
        }
		atColumnIndex = !this.showPath ? atColumnIndex + 1 : atColumnIndex;
		if(atColumnIndex == 4){
		   if(rowData.isNew)
		   		rowData.action = "remove"
			else if(rowData.isRemoved)
				rowData.action = "undoremove"
			else
				rowData.action = "remove"
					
        }
		return val;
	}
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.attachmentViewer.AttachmentTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
	                '   <div dojoAttachPoint="_querySection">' +
					'      <span dojoAttachPoint="_queryTitle" class="query-title">${labelCurrentQuery}</span>' +
					'      <span dojoAttachPoint="_queryName" class="query-name"></span>' +
					'   </div>' +
					'</div>',

    editable: true,
	printable: false,
    widgetsInTemplate: true,
	labelCurrentQuery: Constants.getMessage("labelCurrentQuery"),
	
    constructor: function() {
        this.actions = [];
        this._actionButtons = [];
		this.attachments = [];
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        
		//create the table viewer
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            //turn off all controls if in print mode 
            showPagingControl: !this.printable,
            showGroupFilterControls: !this.printable,
            showPagingActionsControls: !this.printable          
        });
		       
        this._tableViewerNode.appendChild(tableViewer.domNode);
		          
        var content = this.attachments ? this.attachments : [];
        if (this.queryName) {
            this._queryName.innerHTML = this.queryName;
            dojo.removeClass(this._querySection, "hidden");
        } else {
            dojo.addClass(this._querySection, "hidden");
        }		  
        tableViewer.setContentPageSize(10);
        tableViewer.setContent(content);
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", this.queryName ? this.queryName: Constants.getMessage("labelAttachments")));
		
		var showArgs = {editable: this.editable, showPath: this.showPath};
        tableViewer.setContentProvider(new AttachmentTableView.TableProvider(showArgs));
		
		var tableListener = new AttachmentTableView.TableListener(showArgs);
		tableViewer.setTableListener(tableListener);
		
		var cellModifier = new AttachmentTableView.CellModifier(showArgs)
		cellModifier.setChangeListener(this.removeListener)
		tableViewer.setCellModifier(cellModifier);		
		
		
		tableViewer.registerCellRenderer("RenderType.AttachmentCell", 
			new com.ibm.asq.common.web.ui.internal.widgets.attachmentViewer.AttachmentTextCellRenderer());
				
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            var action = this.actions[i];
            var actionButton = tableViewer.addAction(action.actionId, action.label, action.actionHandler, action.iconClass);
            if (action.disabled && action.disabled == true) {
                actionButton.setDisabled(true);
            }
            if (action.hidden && action.hidden == true) {
                dojo.addClass(actionButton.domNode, "hidden");
            }
            this._actionButtons.push(actionButton);
        }            

		tableViewer.setShowingFilterControl(false);
        tableViewer.show();
		tableViewer.setEditable(this.editable);
		this.tableViewer = tableViewer;
    },
 
	 // API
	 clearContent: function()
	 {
	 	this.tableViewer.setContent([]);
		this.tableViewer.show();
	 },
	 
    getAttachments: function() {
        return this.tableViewer.getContent();
    },
   
    appendAttachment: function(attachment) {
    	var content = this.tableViewer.getContent();
        content.push(attachment);
        this.tableViewer.setContent(content);
        this.tableViewer.show();
   }
	
});

})();
