/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//    A ConfigurationDisplay is a UI display for the catalog configuration
//    model object. It has three modes of display: "EDITABLE", "READONLY", and "SELECTABLE".
//    If the attribute 'showSingleConfigPart' is false (default), each PART in the configuration is shown
//    as a panel and each panel shows the attribute values for the part. In "EDITABLE" mode,
//    the ConfigurationDisplay also has actions to add a new PART, or to add parts 
//    from existing configurations. In "SELECTABLE" mode, attributes of a PART are shown with
//    a toggle. User can deselect an attribute by checking it off.
//
//    If the attribute 'showSingleConfigPart' is true, the display only show one PART at a time. 
//    If more then one PART is in the configuration, a dropdown selector is provided to the user
//    to chose to show each PART. 
//      
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.ConfigurationDisplay");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Attribute");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Configuration");

dojo.require("com.ibm.rational.test.lm.web.client.internal.config");
dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.CatalogAttrBuilder");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var ConfigPartBuilder = com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.CatalogAttrBuilder;
var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.catalogAttrBuilder.ConfigurationDisplay",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-rational-test-lm-web-ui-configuration-display">' +
                    '  <div class="com-ibm-rational-test-lm-web-ui-configuration-display-action-area" dojoAttachPoint="configContainerActionNode">' +
                    '  </div>' +
                    '  <div class="com-ibm-rational-test-lm-web-ui-configuration-display-title" dojoAttachPoint="titleNode">' +
                    '  </div>' +
                    '  <div dojoAttachPoint="selectorNode" style="display:none">' +
                    '    <span dojoAttachPoint="selectorLabelNode">${selectorLabel}</span>' +
                    '  </div>' +
                    '  <div class="config-part-area" dojoAttachPoint="configContainerNode">' +
                    '  </div>' +
                    '</div>',

    selectorLabel: RtlmUiUtil.getMessage("cfgSelectLRD"),
    noNameLabel: RtlmUiUtil.getMessage("cfgNonameLRD"),
    
    // a configuration display can be in one of three modes: "EDITABLE", "SELECTABLE", and "READONLY"                    
    mode: "EDITABLE",
    
    // the config model object.
    catalogConfig: null,
        
    assetType: "Machine",
    
    showAllRootTypes: false,
    
    showTitle: true,
    
    showActions: true,
    
    title: null,
    
    // specifies the margin space, expressed in em (like 0.5, 1, etc.) between the title and 
    // PART panel displays.
    margin: null,
    
    // if set to true and the mode is EDITABLE, the user can delete all PARTs including the last one.
    // Otherwise, the user cannot delete the last PART in the configuration.
    canBeEmpty: false,
    
    selectorPrompt: "",
    
    // controls whether to show all PARTs in the configuration, or to show on at a time.
    showSingleConfigPart: false,
    
    postCreate: function() 
    {
        this.theCatalog = new com.ibm.asq.common.web.client.internal.Catalog();
        
        if(this.showTitle) {
          //  var title = (this.title) ? this.title : RtlmUiUtil.getMessage("cfgAssetDescription");
            this.titleNode.innerHTML = "";
        }
        
        if (this.mode == "EDITABLE" && this.showActions) 
            this._configureActions();
        
        if(this.margin) {
            dojo.style(this.configContainerActionNode, "paddingTop", this.margin + "em");
            dojo.style(this.configContainerActionNode, "paddingBottom", this.margin + "em");
        }
        
        // load the config display.
        if(this.catalogConfig)
            this.setCatalogConfig(this.catalogConfig);
    },

    _configureActions: function()
    {
        var args = {
            'showLabel': false,
            'class': "com-ibm-rational-test-lm-web-ui-configuration-display-action"
        };
        var button;
        
        args['label'] = RtlmUiUtil.getMessage("cfgAddPart");
        args['iconClass'] = "addassetdesc-icon-image";
        args['onClick'] = dojo.hitch(this, "_addNewPart");
        button = new dijit.form.Button(args);
        this.configContainerActionNode.appendChild(button.domNode);
        
        args['label'] = RtlmUiUtil.getMessage("cfgAddConfig");
        args['iconClass'] = "addfromconfig-icon-image";
        args['onClick'] = dojo.hitch(this, "_addConfigPart");
        
        button = new dijit.form.Button(args);
        this.configContainerActionNode.appendChild(button.domNode);
    },
    
    _loadConfigPartSelectors: function()
    {
        var rootAttr; 
        var nParts = 0;
        var parts = null;
        
        if (this.catalogConfig) {
            rootAttr = this.catalogConfig.getRootAttribute();
            parts = rootAttr.getArrayValues();
            nParts = parts.length;
        }
        
        // compact the extra vertical space in the title/action line
        // We don't need the title or action.       
        dojo.style(this.titleNode, "display", "none");
        dojo.style(this.configContainerActionNode, "display", "none");
        dojo.style(this.selectorNode, "display", "block");

        // No configuration, or no parts, show an empty resource message.
        if (nParts < 1) {
            this.selectorNode.innerHTML = "<i>" + RtlmUiUtil.getMessage("cfgEmptyConfig") + "</i>";
            return;
        }
        
        if (nParts > 1) {
			var selectLabel = document.createElement("label");
			this.partSelector = document.createElement("select");
			
            dojo.addClass(this.partSelector, "select-config-part-area");
            this.partSelector.onchange = dojo.hitch(this, "_changeConfigPart");

            for (var i = 0; i < nParts; i++) {
                var partTag = parts[i].getTag();
                var partLabel = this.catalogConfig.getLabelByTag(partTag);
                
                if (partLabel == null || partLabel == "") {
                    partLabel = this.noNameLabel + " " + (i + 1);
                }
                this.partSelector.options[i] = new Option(partLabel, partTag);
            }
			selectLabel.appendChild(this.partSelector);
            this.selectorNode.appendChild(selectLabel);
            this._changeConfigPart();
        }
        else {
            var partTag = parts[0].getTag();
            var partLabel = this.catalogConfig.getLabelByTag(partTag);
            this.selectorLabelNode.innerHTML = (partLabel) ? ("<b>" + partLabel + "</b>") : "";

            // notify the part change.
            var mode = this.onConfigPartSelectionChanged(partTag); 
            if(mode && this.mode != mode)
                this.mode = mode;        
            this._showSingleConfigPart(partTag);
        }
    },
    
    _changeConfigPart: function()
    {
        for(var i = 0; i < this.partSelector.options.length; i++) {
            if(this.partSelector.options[i].selected) {
                var tag = this.partSelector.options[i].value;
                // notify the part change.
                var mode = this.onConfigPartSelectionChanged(tag); 
                
                if(mode && this.mode != mode)
                    this.mode = mode;        
                this._showSingleConfigPart(tag);                
                return;
            }
        }        
    },
    
    _showSingleConfigPart: function(tag)
    {
        this.configContainerNode.innerHTML = "";
        this.currentPartTag = tag;
        
        var configPartDisplay = new ConfigPartBuilder({
            'mode': this.mode == "SELECTABLE" ? "EDITABLE" : this.mode,  //Even in "Selectable" Mode we want to be able to edit the config part
            'style': "plain",
            'catalogAttribute': this.catalogConfig.getRootAttribute(),
            'arrayAttrTag': tag,
            'canRemove': false,
            'onValueChanged' : dojo.hitch(this, "_configPartValueChanged")
        });
        this.configContainerNode.appendChild(configPartDisplay.domNode);
    },
    
    _loadConfigPartEditors: function()
    {   
        if(!this.catalogConfig)
            return;
                
        var rootAttr = this.catalogConfig.getRootAttribute();
        var parts = rootAttr.getArrayValues();
        var nParts = parts.length;
        
        this.configContainerNode.innerHTML = "";
        this.partBuilders = [];
        
        for(var i = 0; i < nParts; i++) {
            var partTag = parts[i].getTag();
            var partLabel = this.catalogConfig.getLabelByTag(partTag);
            var editLabel = false;
            
            if (partLabel == null || partLabel == "") {
                partLabel = RtlmUiUtil.getMessage("cfgLabelPromptResourceNumber", (i+1));
            }
            
            var configPartDisplay = new ConfigPartBuilder({
                'mode': this.mode,
                'catalogAttribute': rootAttr,
                'arrayAttrTag': partTag,
                'title': partLabel,
                'setTitleFocus': editLabel,
                'showAllRootTypes': this.showAllRootTypes,
                'canRemove': (this.canBeEmpty || nParts > 1),
                'onRemove': dojo.hitch(this, "_removeConfigPart"),
                'onValueChanged' : dojo.hitch(this, "_configPartValueChanged"),
                'onTitleChanged': dojo.hitch(this, "_configPartTitleChanged")
            });
            this.partBuilders.push(configPartDisplay);
            dojo.addClass(configPartDisplay.domNode, "config-part");
            this.configContainerNode.appendChild(configPartDisplay.domNode);
        }
    },

    _removeConfigPart: function(partBuilderWidget)
    {
		var self = this;
		var args = {
			title: RtlmUiUtil.getMessage("msgConfirmTitle"),
			message: RtlmUiUtil.getMessage("msgConfirmRemoveResourceDescription"),
			yesCallback: function(){
		        var rootAttr = self.catalogConfig.getRootAttribute();
		        rootAttr.removeArrayValueByTag(partBuilderWidget.arrayAttrTag);
		        
		        setTimeout(dojo.hitch(self, "_loadConfigPartEditors"), 1);
		        self.onValueChanged();
				return true;          
			},
			noCallback: function() {
				return false;
			}
		}
		var dialog = new YesNoDialog(args);
        dialog.setSecondaryTitle(args.title);
        dialog.open();
    },
    
    _configPartTitleChanged: function(partBuilderWidget, newTitle)
    {
        this.catalogConfig.setLabelByTag(partBuilderWidget.arrayAttrTag, newTitle);
        this.onValueChanged();
    },

    _configPartValueChanged: function(partBuilderWidget, newTitle)
    {
        this.onValueChanged();
    },
    
    _addNewPart :function()
    {
        // auto create the config, if we don't have one.
        if(!this.catalogConfig) {
            this.catalogConfig = this.theCatalog.createConfiguration(this.assetType);
        }
        else {
            this.catalogConfig.addRoot(this.assetType);
        }
		this._loadConfigPartEditors();

        this.onValueChanged();
    },
    
    _addConfigPart :function()
    {
        this.allConfigs = [];
        var configs = new com.ibm.rational.test.lm.web.client.internal.config();
        var responseHandler= new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadAllConfigsSuccessHandler", "_loadAllConfigsErrorHandler");
    
        configs.getAllWithDetails(responseHandler);
    },
    
    _loadAllConfigsSuccessHandler :function(response)
    {
        this.selectDlg = new com.ibm.rational.test.lm.web.ui.internal.widgets.FilteredSelectionDialog({title:"Select Test Environments"});
        this.selectDlg.setOKaction( dojo.hitch( this, this._handleSelectOK ) );
        if (response) 
        {
            for (var i = 0; i < response.length; i++) 
            {
                var name = response[i].name;
                var description = response[i].description;
                var id = response[i].id;
                
                this.selectDlg.addListItem(id, name);
                this.allConfigs[i] = {id:id, config:response[i].configuration};
            }
        }
        this.selectDlg.show();
    },
    
    _handleSelectOK :function()
    {
        var newItems = this.selectDlg.getSelectedItems();
        var inserted = false;
        
        for (var i = 0; i < newItems.length; i++) {
            var id = newItems[i].id;
            var configString = null;
            
            for (var j = 0; j < this.allConfigs.length; j++) {
                if (this.allConfigs[j].id == id) {
                    configString = this.allConfigs[j].config;
                    break;
                }
            }
            if (configString != null) {
                var configToInsert = this.theCatalog.createConfigurationFromJson(configString);
                if(!this.catalogConfig) {
                    this.catalogConfig = this.theCatalog.createConfiguration(this.assetType);
                    var rootAtt = this.catalogConfig.getRootAttribute();
                    
                    //Remove the 0th element which is the existing part
                    rootAtt.removeArrayValue(0);
                }
                this.catalogConfig.insertConfig(configToInsert);
                this._loadConfigPartEditors();
                inserted = true;
            }
        }
        
        if (inserted) {
            this.onValueChanged();
        }
    },
    
    _loadAllConfigsErrorHandler :function(errorType, errorMessage)
    {
        
    },

    setCatalogConfig: function(config) 
    {
        this.catalogConfig = config;

        // loads the display with either the editor style, or the
        // selector style based on the mode of display.
        //
        if(this.showSingleConfigPart)           
            this._loadConfigPartSelectors();
        else
            this._loadConfigPartEditors();
    },

    refresh: function()
    {
        if (this.showSingleConfigPart) {
            this._showSingleConfigPart(this.currentPartTag);
        }
        else {
            this._loadConfigPartEditors();
        }
    },
    
    getCurrentConfigPartTag: function()
    {
        return this.currentPartTag;
    },
    
    getCatalogConfig: function() 
    {
        if(this.catalogConfig && this.showSingleConfigPart) {
            var cat = new com.ibm.asq.common.web.client.internal.Catalog();
            var config = cat.createConfiguration(this.assetType);
            var rootAtt = config.getRootAttribute();

            // Remove the 0th element which is the existing part
            rootAtt.removeArrayValue(0);
            
            var currPart = this.catalogConfig.getRootAttribute().getArrayValueByTag(this.currentPartTag);
            rootAtt.addArrayValue(currPart);
            
            return config;
        }
        return this.catalogConfig;
    },
    
    // Summary:
    //    Notification when the input field value has changed. This includes any children 
    //    attribute down the recurse level. Client can override this method to receive the 
    //    notification.
    //
    onValueChanged: function()
    {
    },
    
    // Summary:
    //    Notification when the configuration display is in single part display mode and a
    //    part display has changed. Client can override this method to receive the 
    //    notification. In addition, if this method returns a different mode for the
    //    display, the new mode will be applied to the configuration part being displayed.
    //
    onConfigPartSelectionChanged: function(/* the part's tag being shown */tag)
    {
        // default to not override the current mode of display.
        // could be one of three modes: "EDITABLE", "SELECTABLE", and "READONLY"  
        return null;
    },
    
    // Summary:
    //    Returns true if there's input error in the input field. This reports any children 
    //    attribute errors down the recurse level.
    //
    hasError: function()
    {
        for(var i = 0; i < this.partBuilders.length; i++) {
            var partBuilder = this.partBuilders[i];
            
            if(partBuilder.hasError()) {
                return true;
            }
        }
        return false;        
    }
});

})();
