/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormDateField");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.AbstractFormField");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var ValidateTextBox = com.ibm.rational.test.lm.web.ui.internal.widgets.RlmValidateTextBox;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormDateField", 
    com.ibm.rational.test.lm.web.ui.internal.widgets.layout.AbstractFormField,
{
    currentValue: null,
    
    textFieldInvalidFormatMsg: RtlmUiUtil.getMessage("regMsgNonExistDate"),
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.dateField = new com.ibm.asq.common.web.ui.internal.widgets.DatePicker({
                    name: "dateField",
                    value: "",
                    invalidMessage: RtlmUiUtil.getMessage("reqMsgDateInputFormat"),
                    onChange: dojo.hitch(this, "_valueChangedHandler"),
                    onkeyup: dojo.hitch(this, "_valueChangedHandler"),
                    'class': this._getBaseClassName()
                });
        this.fieldHolderNode.appendChild(this.dateField.domNode);
        this.dateField.init();
        
        this.formField = this.dateField.domNode;
    },

    _valueChangedHandler: function()
    {
        if(this.inSetValue)
            return;
            
        var oldValue = this.currentValue;
        this.currentValue = this.getValue(); 
        
        this.setHasError(!this.dateField.isValid());
        this.onValueChanged(this, oldValue, this.currentValue);
    },
    
    setHasError: function(hasError) {
        if (!hasError && this.textFieldInvalidFormatMsg != "")
            // restore the original invalid format message on no error
            this.setErrorMessage(this.textFieldInvalidFormatMsg);
            
        this.inherited("setHasError", arguments);
    },
    
    setErrorMessage: function(message)
    {
        if (this.textFieldInvalidFormatMsg == "")
            // save the original invalid format message
            this.textFieldInvalidFormatMsg = this.dateField.getErrorMessage();
            
        this.dateField.invalidMessage = message;
    },
    
    setEdited: function(isEdited)
    {
        if (!this.dateField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setEdited", arguments);
        }
    },
    
    setModified: function(isEdited)
    {
        if (!this.dateField.isValid()) {
            // use error condition for invalid entry 
            this.setHasError(true);
        } else {
            this.inherited("setModified", arguments);
        }
    },
    
    setValue: function(value)
    {
        var val = (value) ? value : "";
        var intVal = parseInt(val);
        
        // It's an empty value, don't have to
        // set it to the date box.
        if(intVal <= 0)
            return;
            
        var date = new Date();
        date.setTime(intVal);
        
        this.inSetValue = true;
        this.dateField.setValue(date);
        this.inSetValue = false;
    },

    // Summary:
    //    Returns the value from the field. Subclass must override. A null is returned if
    //    no value is specified.
    //
    getValue: function()
    {
        var val = this.dateField.getValue();
        if(val)
            return val.getTime() + "";
        return "";    
    },
    
    // Summary:
    //    This method is called when the field's value has been changed. Consumer of the widget 
    //    overrides this method to receive notification of the value changed event.
    //
    onValueChanged: function(oldValue, newValue)
    {
    },
    
    _getBaseClassName: function()
    {
        return "text-field";
    }
});

})();
