/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.EditableTextCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.EditableTextCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
        var contentProvider = viewer.getContentProvider();
        var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
        
        if (editable) {
			var inputLabel = document.createElement("label");
			node = document.createElement("input");
            node.type = "text";
			node.value = cellData;
            var widthStyled = false;
            
            if(extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) {
                // Ignore sizing information if the cell size hint is provided.
                // the cell's size hint takes precedance.
    			if (!cellWidthHint && extendedDisplayInfo['textWidth']) {
                    node.style.width = extendedDisplayInfo['textWidth'];
                    widthStyled = true;
                }    
    			if (!cellWidthHint && extendedDisplayInfo['size']) {
                    node.size = extendedDisplayInfo['size'];
                    widthStyled = true;
                }
                if(extendedDisplayInfo['class'])
                    dojo.addClass(node, extendedDisplayInfo['class']);
            }
            // by default, if not IE always add style to stretch the field
            // to the table cell width. 
            if(!widthStyled && !dojo.isIE) {
                dojo.addClass(node, "editText-cell");
            }
            // restrict the size of the box to the width hint.
            if(cellWidthHint && dojo.isIE)
                node.style.width = "99%";
                
            node.cellModifier = cellModifier;
            node.rowObject = rowObject;
            node.columnIndex = columnIndex;
            node.oldValue = cellData;
                    
            node.onkeyup = function() {
                var oldValue = this.oldValue;
                this.oldValue = this.value;
                
                if (oldValue != this.value && this.cellModifier)
                    this.cellModifier.cellValueChanged(oldValue, this.value, this.rowObject, this.columnIndex);
            }
			inputLabel.appendChild(node);
			if(extendedDisplayInfo && dojo.isObject(extendedDisplayInfo))
            	this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);
			return inputLabel;
        } 
        else {
            node = document.createElement("span");
            this._renderCellText(viewer, cellData, extendedDisplayInfo, node, cellWidthHint);
			return node;
        }
    }
});

})();
