/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
// Summary:
//
//    +-------------------+--------------------------+
//    | toggle   |  (optional)image title            |  
//    +-------------------+--------------------------+
//
//    
// 
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GroupController");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");

(function() {

var ImageCache      = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

    
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.GroupController", 
    [dijit._Widget, dijit._Templated],
{
    templateString: '<div id="${id}" class="com-ibm-asq-common-tableViewer-group" dojoAttachPoint="domNode">' +
                    '<div class="focus-node" dojoAttachPoint="focusNode" dojoAttachEvent="onclick: toggle">' +
                    '  <button dojoType="dijit.form.CheckBox" dojoAttachPoint="checkBox" dojoAttachEvent="onChange: _checkBoxStateChanged">' +
                    '  </button>' +
                    '  <button dojoAttachPoint="arrowButtonNode" class="group-toggle" dojoAttachEvent="onclick: toggle">' +
                    '    <img alt="${toggleLabel}" dojoAttachPoint="arrowImgNode"></img>' +
                    '  </button>' +
                    '  <img class="group-image" dojoAttachPoint="imageNode" style="display: none"></img>' +
                    '  <span class="group-title" dojoAttachPoint="titleNode"></span>' +
                    '  <span class="group-detail" dojoAttachPoint="detailNode"></span>' +
                    '  <span class="group-extra" dojoAttachPoint="extraNode" align="right"></span>' +
                    '</div>' +
                    '</div>',

    // the label to be displayed in the attribute Label node.
    widgetsInTemplate: true,
    title: "",
    isOpen: true,
    index: -1,
    image: null,
	toggleLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelToggle"),
    
    // create a check box in the group control, if set to true.
    showGroupCheckBox: false,
    groupChecked: false,
    groupCheckTooltipMsg: null,
    
    postCreate: function() {
        if(this.title)
            this.setTitle(this.title);
        if(this.image)
            this.setImage(this.image);
		if(this.extra)
			this.setExtra(this.extra);
        this._setToggleImage();
        
        var showChbox = (this.showGroupCheckBox) ? "inline" : "none";
        dojo.style(this.checkBox.domNode, "display",showChbox);
        
        if(this.showGroupCheckBox) {
            // FF3 forwards the event up to the parent which would 
            // cause the group to collapse/expanded when the toggle is
            // clicked. Stop the event from going up.
            this.checkBox._onClick = function(event) { 
                if(event)
                    event.cancelBubble = true;
                this._clicked(event);
            };
            this.checkBox.setChecked(this.groupChecked);
        }
    },

    // Summary:
    //    Set the label string for the attribute field.
    // 
    setTitle: function(title) 
    {
        this.titleNode.innerHTML = "";
        this.titleNode.appendChild(document.createTextNode(title));
    },

    setImage: function(img)
    {
        this.imageNode.src = img;
        dojo.style(this.imageNode, "display", "inline");        
    },
    
	setExtra: function(extra)
	{
		this.extraNode.appendChild( extra );
	},
	
    // Summary:
    //    Set the label string for the attribute field.
    // 
    setDetails: function(info) 
    {
        this.detailNode.innerHTML = "";
        this.detailNode.appendChild(document.createTextNode(info));
    },
    
    getIsOpen: function()
    {
        return this.isOpen;    
    },
    
    toggle: function(event)
    {
        // don't propagate the event up, if it's the button.
        // Otherwise, the outer div will handle it again and cancel
        // the toggling of the group.
        if (event && event.target && event.target == this.arrowButtonNode) {
            event.cancelBubble = true;
        } 
        this.isOpen = !this.isOpen;
        this._setToggleImage();
        this.onGroupToggled(this);
    },
    
    _setToggleImage: function()
    {
        this.arrowImgNode.src = (this.isOpen) ? ImageCache.SOLID_ARROW_OPEN : ImageCache.SOLID_ARROW_CLOSED;
    },

    _checkBoxStateChanged: function()
    {
        this.onGroupCheckBoxStateChanged(this, this.checkBox.checked);
    },
    
    // Summary:
    //   This method is called when the group's show/collapse (toggle) state is changed.
    //
    onGroupToggled: function()
    {
    },
    
    // Summary:
    //   This method is called when the group's check box state is changed. This is only
    //   active if the 'showGroupCheckBox' attribute is set to true. Otherwise, no group
    //   check box would be created.
    //
    onGroupCheckBoxStateChanged: function(groupCtrl, checked)
    {
    }
});

})();
